@@ -739,7 +739,7 @@ interact (AnnInfo *ap)
 	lrx = lry = lx = ly = -1000;
 	lm = 0;
 	for (nc = 0; nc < 2; ) {
-	    int x, y, rx, ry, cx, cy;
+	    int x = 0, y = 0, rx, ry, cx, cy;
 	    Window rw, cw;
 	    unsigned int m;
 	    int click;
@@ -1087,4 +1087,4 @@ ano_save (char *fn)
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: annotmenu.c,v $ $Date: 2009/01/05 20:55:54 $ $Revision: 1.21 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: annotmenu.c,v $ $Date: 2015/04/09 00:19:20 $ $Revision: 1.22 $ $Name:  $"};
