@@ -7,14 +7,7 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <math.h>
-
-#if defined(__STDC__)
 #include <stdlib.h>
-typedef const void * qsort_arg;
-#else
-typedef void * qsort_arg;
-extern void *calloc(), *malloc(), *realloc();
-#endif
 
 #include <Xm/Xm.h>
 #include <Xm/Form.h>
@@ -29,43 +22,7 @@ extern void *calloc(), *malloc(), *realloc();
 #include <Xm/Separator.h>
 #include <Xm/TextF.h>
 
-#include "P_.h"
-#include "astro.h"
-#include "circum.h"
-#include "preferences.h"
-
-extern Widget	toplevel_w;
-extern Colormap xe_cm;
-
-extern FILE *fopenh P_((char *name, char *how));
-extern Now *mm_get_now P_((void));
-extern Obj *db_scan P_((DBScan *sp));
-extern char *getPrivateDir P_((void));
-extern char *syserrstr P_((void));
-extern double atod P_((char *buf));
-extern int confirm P_((void));
-extern int db_n P_((void));
-extern int existsh P_((char *filename));
-extern int isUp P_((Widget w));
-extern void db_scaninit P_((DBScan *sp, int mask, ObjF *op, int nop));
-extern void db_update P_((Obj *op));
-extern void defaultTextFN P_((Widget w, int setcols, char *x, char *y));
-extern void fs_pangle P_((char out[], double a));
-extern void get_something P_((Widget w, char *resource, XtArgVal value));
-extern void get_xmstring P_((Widget w, char *resource, char **txtp));
-extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
-extern void pm_set P_((int percentage));
-extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
-extern void query P_((Widget tw, char *msg, char *label1, char *label2,
-    char *label3, void (*func1)(), void (*func2)(), void (*func3)()));
-extern void set_something P_((Widget w, char *resource, XtArgVal value));
-extern void set_xmstring P_((Widget w, char *resource, char *txt));
-extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
-extern void sv_point P_((Obj *op));
-extern void timestamp P_((Now *np, Widget w));
-extern void watch_cursor P_((int want));
-extern void xe_msg P_((char *msg, int app_modal));
-extern void wtip P_((Widget w, char *tip));
+#include "xephem.h"
 
 
 #define	MINSEP	degrad(.1)	/* smallest sep we allow.
@@ -91,35 +48,35 @@ typedef struct {
     float sep;		/* separation, rads (float to save memory) */
 } Pair;
 
-static void c_create_shell P_((void));
-static void c_help_cb P_((Widget w, XtPointer client, XtPointer call));
-static void c_helpon_cb P_((Widget w, XtPointer client, XtPointer call));
-static void c_popdown_cb P_((Widget w, XtPointer client, XtPointer call));
-static void c_close_cb P_((Widget w, XtPointer client, XtPointer call));
-static void c_flist_cb P_((Widget w, XtPointer client, XtPointer call));
-static void c_actlist_cb P_((Widget w, XtPointer client, XtPointer call));
-static void c_skypoint_cb P_((Widget w, XtPointer client, XtPointer call));
-static void c_go_cb P_((Widget w, XtPointer client, XtPointer call));
-static int sky_point P_((void));
-static void do_search P_((void));
-static int get_options P_((double *sp, double *mp, int *wp));
-static void free_band P_((Band bp[], int nb));
-static int init_bands P_((double sep, double mag, Band **bpp));
-static int add_obj P_((Band *bp, Obj *op));
-static int find_close P_((Band bp[], int nb, double maxsep, int wander,
-    Pair **ppp));
-static int add_pair P_((Obj *op1, Obj *op2, double sep, Pair **ppp,
-    int *mp, int *np));
-static int add_pair P_((Obj *op1, Obj *op2, double sep, Pair **ppp,
-    int *mp, int *np));
-static int dspl_pairs P_((Pair p[], int np, double sep));
+static void c_create_shell (void);
+static void c_help_cb (Widget w, XtPointer client, XtPointer call);
+static void c_helpon_cb (Widget w, XtPointer client, XtPointer call);
+static void c_popdown_cb (Widget w, XtPointer client, XtPointer call);
+static void c_close_cb (Widget w, XtPointer client, XtPointer call);
+static void c_flist_cb (Widget w, XtPointer client, XtPointer call);
+static void c_actlist_cb (Widget w, XtPointer client, XtPointer call);
+static void c_skypoint_cb (Widget w, XtPointer client, XtPointer call);
+static void c_go_cb (Widget w, XtPointer client, XtPointer call);
+static int sky_point (void);
+static void do_search (void);
+static int get_options (double *sp, double *mp, int *wp, int *op);
+static void free_band (Band bp[], int nb);
+static int init_bands (double sep, double mag, Band **bpp);
+static int add_obj (Band *bp, Obj *op);
+static int find_close (Band bp[], int nb, double maxsep, int wander, int
+    ownmoons, Pair **ppp);
+static int add_pair (Obj *op1, Obj *op2, double sep, Pair **ppp,
+    int *mp, int *np);
+static int add_pair (Obj *op1, Obj *op2, double sep, Pair **ppp,
+    int *mp, int *np);
+static int dspl_pairs (Pair p[], int np, double sep);
 
-static void c_create_flist_w P_((void));
-static void c_flistok_cb P_((Widget w, XtPointer client, XtPointer call));
-static void flistok_append_cb P_((void));
-static void flistok_overwrite_cb P_((void));
-static void make_flist P_((char *name, char *how));
-static void write_flist P_((FILE *fp));
+static void c_create_flist_w (void);
+static void c_flistok_cb (Widget w, XtPointer client, XtPointer call);
+static void flistok_append_cb (void);
+static void flistok_overwrite_cb (void);
+static void make_flist (char *name, char *how);
+static void write_flist (FILE *fp);
 
 
 static Widget cshell_w;		/* main shell */
@@ -130,7 +87,9 @@ static Widget count_w;		/* label for count of objects in list */
 static Widget list_w;		/* the list of close objects */
 static Widget flist_w;		/* the file list dialog */
 static Widget wander_w;		/* toggle to omit pairs of fixed objs*/
+static Widget ownmoons_w;	/* toggle to omit a planet's own moons */
 static Widget autoup_w;		/* toggle to automatically redo on updates */
+static Widget autols_w;		/* toggle to automatically list on updates */
 
 static char closecategory[] = "Close pairs"; 	/* Save category */
 
@@ -187,10 +146,10 @@ c_create_shell ()
 	    char *key;		/* string to call hlp_dialog() */
 	} HelpOn;
 	static HelpOn helpon[] = {
-	    {"Intro...",	"Close Objects - intro"},
-	    {"on Control...",	"Close Objects - control"},
-	    {"on Options...",	"Close Objects - options"},
-	    {"Misc...",		"Close Objects - misc"},
+	    {"Intro...",	"CloseObjects"},
+	    {"on Control...",	"CloseObjects_control"},
+	    {"on Options...",	"CloseObjects_options"},
+	    {"Misc...",		"CloseObjects_misc"},
 	};
 	Widget mb_w, pd_w, cb_w;
 	Widget s_w;
@@ -210,6 +169,7 @@ c_create_shell ()
 	XtSetArg (args[n], XmNdeleteResponse, XmUNMAP); n++;
 	cshell_w = XtCreatePopupShell ("CloseObjs", topLevelShellWidgetClass,
 							toplevel_w, args, n);
+	setup_icon (cshell_w);
 	set_something (cshell_w, XmNcolormap, (XtArgVal)xe_cm);
 	XtAddCallback (cshell_w, XmNpopdownCallback, c_popdown_cb, NULL);
 	sr_reg (cshell_w, "XEphem*CloseObjs.width", closecategory, 0);
@@ -248,28 +208,11 @@ c_create_shell ()
 	    /* make the Go button */
 
 	    n = 0;
-	    w = XmCreatePushButton (pd_w, "Search", args, n);
+	    w = XmCreatePushButton (pd_w, "Run", args, n);
 	    XtAddCallback (w, XmNactivateCallback, c_go_cb, 0);
-	    set_xmstring(w, XmNlabelString, "Go");
-	    wtip (w, "Scan one time for close pairs");
+	    wtip (w, "Perform one scan for close pairs");
 	    XtManageChild (w);
 
-	    /* add a separator */
-
-	    n = 0;
-	    w = XmCreateSeparator (pd_w, "Sep2", args, n);
-	    XtManageChild (w);
-	    
-	    /* make the auto update button */
-
-	    n = 0;
-	    XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
-	    autoup_w = XmCreateToggleButton (pd_w, "AutoUp", args, n);
-	    set_xmstring(autoup_w, XmNlabelString, "Auto update");
-	    wtip (autoup_w, "When on, perform a new scan after each Main menu Update");
-	    XtManageChild (autoup_w);
-	    sr_reg (autoup_w, NULL, closecategory, 1);
-
 	    /* make the sky point button */
 
 	    n = 0;
@@ -313,17 +256,49 @@ c_create_shell ()
 	    cb_w = XmCreateCascadeButton (mb_w, "OptionsCB", args, n);
 	    set_xmstring (cb_w, XmNlabelString, "Options");
 	    XtManageChild (cb_w);
+
+	    /* make the auto update button */
+
+	    n = 0;
+	    XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
+	    autoup_w = XmCreateToggleButton (pd_w, "AutoUp", args, n);
+	    set_xmstring(autoup_w, XmNlabelString, "Auto run");
+	    wtip (autoup_w, "Whether to automatically perform a new scan after each Main menu Update.");
+	    XtManageChild (autoup_w);
+	    sr_reg (autoup_w, NULL, closecategory, 1);
 	    
-	    /* make the button to select all or just wanderers */
+	    /* make the auto list button */
+
+	    n = 0;
+	    XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
+	    autols_w = XmCreateToggleButton (pd_w, "AutoList", args, n);
+	    set_xmstring(autols_w, XmNlabelString, "Auto list");
+	    wtip (autols_w,
+	     "Whether to automatically append each new scan to the list file.");
+	    XtManageChild (autols_w);
+	    sr_reg (autols_w, NULL, closecategory, 1);
+
+	    /* make the button to omit fixed pairs */
 
 	    n = 0;
 	    XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
 	    wander_w = XmCreateToggleButton (pd_w, "OmitFixedPairs", args, n);
 	    set_xmstring(wander_w, XmNlabelString, "Omit fixed pairs");
-	    wtip (wander_w, "When on, both objects in a pair can not be of type FIXED");
+	    wtip (wander_w, "Whether pairs of Fixed objects are not shown.");
 	    XtManageChild (wander_w);
 	    sr_reg (wander_w, NULL, closecategory, 1);
 
+	    /* make the button to omit planets and their own satellites */
+
+	    n = 0;
+	    XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
+	    ownmoons_w = XmCreateToggleButton (pd_w, "OmitOwnMoons", args, n);
+	    set_xmstring(ownmoons_w, XmNlabelString, "Omit planet's own moons");
+	    wtip (ownmoons_w,
+		"Whether to skip pairs that are a planet and it's own moons.");
+	    XtManageChild (ownmoons_w);
+	    sr_reg (ownmoons_w, NULL, closecategory, 1);
+
 	/* make the help pulldown */
 
 	n = 0;
@@ -469,7 +444,7 @@ XtPointer call;
 "pairs of objects separated by less than the given angular distance.",
 };
 	
-	hlp_dialog ("Close Objects", msg, XtNumber(msg));
+	hlp_dialog ("CloseObjects", msg, XtNumber(msg));
 }
 
 /* called when any of the individual help entries are selected.
@@ -540,7 +515,7 @@ XtPointer client;
 XtPointer call;
 {
 	if (sky_point() < 0)
-	    xe_msg ("First select a line from the list.", 1);
+	    xe_msg (1, "First select a line from the list.");
 }
 
 /* called when the Perform Search pushbutton is activated */
@@ -593,12 +568,8 @@ sky_point()
 	/* point sky if found */
 	if (op)
 	    sv_point (op);
-	else {
-	    char msg[128];
-
-	    (void) sprintf (msg, "Can not find `%s' in memory!", objname);
-	    xe_msg (msg, 1);
-	}
+	else
+	    xe_msg (1, "Can not find `%s' in memory!", objname);
 
 	return (0);
 }
@@ -606,9 +577,7 @@ sky_point()
 /* given two pointers to Obj *, return how they sort by dec in qsort-style.
  */
 static int
-racmp_f (v1, v2)
-qsort_arg v1;
-qsort_arg v2;
+racmp_f (const void *v1, const void *v2)
 {
 	Obj *op1 = *((Obj **)v1);
 	Obj *op2 = *((Obj **)v2);
@@ -625,16 +594,18 @@ qsort_arg v2;
 static void
 do_search()
 {
+	int als = XmToggleButtonGetState(autols_w);
 	double sep;	/* desired max separation, rads */
 	double mag;	/* limiting magnitude */
-	int wander;	/* 1 if want to limit list to at least one wanderer */
+	int wander;	/* 1 to limit list to at least one wanderer */
+	int ownmoons;	/* 1 to omit planet's own moons */
 	Band *bp = NULL;/* malloced array of nb bands */
 	int nb;		/* number of dec (or alt) bands */
 	Pair *pp = NULL;/* malloced list of close pairs */
 	int np;		/* number of pairs */
 
 	/* retrieve user option settings */
-	if (get_options (&sep, &mag, &wander) < 0)
+	if (get_options (&sep, &mag, &wander, &ownmoons) < 0)
 	    return;
 
 	/* ok, let's get to the real work */
@@ -652,7 +623,7 @@ do_search()
 	/* look through bands for all pairs closer than sep.
 	 * form sorted lists of them in malloced pp array.
 	 */
-	np = find_close (bp, nb, sep, wander, &pp);
+	np = find_close (bp, nb, sep, wander, ownmoons, &pp);
 
 	/* finished with the bands now */
 	free_band (bp, nb);
@@ -660,8 +631,12 @@ do_search()
 	/* report what we found */
 	if (np > 0)
 	    (void) dspl_pairs (pp, np, sep);
-	else
-	    xe_msg ("No such pairs found.", 1);
+	else if (!als)
+	    xe_msg (1, "No such pairs found.");
+
+	/* add to file if on */
+	if (als)
+	    flistok_append_cb ();
 
 	/* finished */
 	if (pp)
@@ -673,10 +648,11 @@ do_search()
  * return 0 if ok, else -1
  */
 static int
-get_options (sp, mp, wp)
+get_options (sp, mp, wp, op)
 double *sp;	/* max separation, rads */
 double *mp;	/* limiting magnitude */
 int *wp;	/* wanderer flag */
+int *op;	/* ownmoons flag */
 {
 	char *str;
 
@@ -684,11 +660,8 @@ int *wp;	/* wanderer flag */
 	*sp = degrad(atod(str));
 	XtFree (str);
 	if (*sp < MINSEP) {
-	    char msg[128];
-
-	    (void)sprintf(msg, "Please specify a max separation >= %g degrees.",
+	    xe_msg (1, "Please specify a max separation >= %g degrees.",
 	    							raddeg(MINSEP));
-	    xe_msg (msg, 1);
 	    return (-1);
 	}
 
@@ -698,6 +671,8 @@ int *wp;	/* wanderer flag */
 
 	*wp = XmToggleButtonGetState (wander_w);
 
+	*op = XmToggleButtonGetState (ownmoons_w);
+
 	return (0);
 }
 
@@ -739,7 +714,6 @@ Band **bpp;
 	double bandsiz;
 	Band *bp;
 	DBScan dbs;
-	char msg[1024];
 	int nobj;
 	int nb;
 	Obj *op;
@@ -748,7 +722,7 @@ Band **bpp;
 	/* compute number of bands and band size */
 	nb = (int)ceil(PI/sep);
 	if (nb < 1) {
-	    xe_msg ("Please decrease max separation to < 180 degrees.", 1);
+	    xe_msg (1, "Please decrease max separation to < 180 degrees.");
 	    return (-1);
 	}
 	bandsiz = PI/nb;
@@ -756,10 +730,9 @@ Band **bpp;
 	/* malloc list of zero'd Bands */
 	bp = (Band *) calloc (nb, sizeof(Band));
 	if (!bp) {
-	    (void) sprintf (msg,
+	    xe_msg (1,
 	    	"Could not malloc %d bands -- try increasing max separation.",
 									nb);
-	    xe_msg (msg, 1);
 	    return(-1);
 	}
 
@@ -785,8 +758,7 @@ Band **bpp;
 	    /* find which band op fits into */
 	    b = BAND(op->s_dec);
 	    if (b < 0 || b > nb) {
-		(void) sprintf (msg, "%s is out of band: %d.", op->o_name, b);
-		xe_msg (msg, 0);
+		xe_msg (0, "%s is out of band: %d.", op->o_name, b);
 		continue;
 	    }
 
@@ -830,7 +802,7 @@ Obj *op;
 
 	    newmem = malloc (BCHUNKS * sizeof(Obj *));
 	    if (!newmem) {
-		xe_msg ("Can not malloc temp Objects.", 1);
+		xe_msg (1, "Can not malloc temp Objects.");
 		return (-1);
 	    }
 
@@ -845,7 +817,7 @@ Obj *op;
 
 	    newmem = realloc((char *)bp->opp, (bp->nmem+BCHUNKS)*sizeof(Obj *));
 	    if (!newmem) {
-		xe_msg ("Can not realloc more temp Objects.", 1);
+		xe_msg (1, "Can not realloc more temp Objects.");
 		return (-1);
 	    }
 
@@ -863,11 +835,12 @@ Obj *op;
  * N.B. caller must free *ppp if it is != NULL no matter what we return.
  */
 static int
-find_close (bp, nb, maxsep, wander, ppp)
+find_close (bp, nb, maxsep, wander, ownmoons, ppp)
 Band bp[];	/* list of bands to scan */
 int nb;		/* number of bands */
 double maxsep;	/* max sep we are looking for, rads */
 int wander;	/* 1 if limit to pairs with at least one wanderer (ss object) */
+int ownmoons;	/* 1 if limit planets with their own moons */
 Pair **ppp;	/* return list of pairs we malloc */
 {
 	double stopsep = maxsep*2.3; /* stop scan at maxsep*sqrt(1**2 + 2**2) */
@@ -910,6 +883,11 @@ Pair **ppp;	/* return list of pairs we malloc */
 		    if (wander && is_type(opi,FIXEDM) && is_type(opj,FIXEDM))
 			continue;
 
+		    /* omit if both are in the same planet system */
+		    if (ownmoons && is_type(opi,PLANETM) && is_type(opj,PLANETM)
+					&& opi->pl_code == opj->pl_code)
+			continue;
+
 		    /* find the exact separation */
 		    cb = sin(opj->s_dec); /* j is b */
 		    sb = cos(opj->s_dec);
@@ -957,7 +935,7 @@ int *np;	/* pointer to number of *ppp actually in use */
 
 	    newmem = malloc (PCHUNKS * sizeof(Pair));
 	    if (!newmem) {
-		xe_msg ("Can not malloc any Pairs.", 1);
+		xe_msg (1, "Can not malloc any Pairs.");
 		return (-1);
 	    }
 
@@ -972,7 +950,7 @@ int *np;	/* pointer to number of *ppp actually in use */
 
 	    newmem = realloc((char *)*ppp, (*mp+PCHUNKS) * sizeof(Pair));
 	    if (!newmem) {
-		xe_msg ("Can not realloc more Pairs.", 1);
+		xe_msg (1, "Can not realloc more Pairs.");
 		return (-1);
 	    }
 
@@ -1003,9 +981,7 @@ int *np;	/* pointer to number of *ppp actually in use */
  *   objects will sort adjacent, and is used later to elminate dups.
  */
 static int
-pair_cmpf (v1, v2)
-qsort_arg v1;
-qsort_arg v2;
+pair_cmpf (const void *v1, const void *v2)
 {
 	Pair *p1 = (Pair *)v1;
 	Pair *p2 = (Pair *)v2;
@@ -1062,8 +1038,7 @@ double sep;
 	 */
 	xmstrtbl = (XmString *) malloc (np * sizeof(XmString));
 	if (!xmstrtbl) {
-	    (void) sprintf (buf, "No memory for %d XmStrings", np);
-	    xe_msg (buf, 1);
+	    xe_msg (1, "No memory for %d XmStrings", np);
 	    return (-1);
 	}
 	for (ns = i = 0; i < np; i++) {
@@ -1125,14 +1100,14 @@ c_create_flist_w()
 
 	n = 0;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	XtSetArg(args[n], XmNmarginWidth, 10);  n++;
-	XtSetArg(args[n], XmNmarginHeight, 10);  n++;
+	XtSetArg(args[n], XmNmarginWidth, 20);  n++;
+	XtSetArg(args[n], XmNmarginHeight, 20);  n++;
 	flist_w = XmCreatePromptDialog (cshell_w, "CloseList", args,n);
 	set_something (flist_w, XmNcolormap, (XtArgVal)xe_cm);
 	set_xmstring (flist_w, XmNdialogTitle, "xephem Close list");
 	set_xmstring (flist_w, XmNselectionLabelString, "File name:");
 	tw = XmSelectionBoxGetChild(flist_w, XmDIALOG_TEXT);
-	defaultTextFN (tw, 1, getPrivateDir(), "closelist.txt");
+	defaultTextFN (tw, 1, "closelist.txt", NULL);
 	sr_reg (tw, NULL, closecategory, 1);
 	XtUnmanageChild (XmSelectionBoxGetChild(flist_w, XmDIALOG_HELP_BUTTON));
 	XtAddCallback (flist_w, XmNokCallback, c_flistok_cb, NULL);
@@ -1154,21 +1129,21 @@ XtPointer call;
 	get_xmstring(w, XmNtextString, &name);
 
 	if (strlen(name) == 0) {
-	    xe_msg ("Please enter a file name.", 1);
+	    xe_msg (1, "Please enter a file name.");
 	    XtFree (name);
 	    return;
 	}
 
 	get_something (list_w, XmNitemCount, (XtArgVal)&icount);
 	if (icount == 0) {
-	    xe_msg ("No items in list -- no file action.", 1);
+	    xe_msg (1, "No items in list -- no file action.");
 	    XtFree (name);
 	    return;
 	}
 
-	if (existsh (name) == 0 && confirm()) {
-	    (void) sprintf (buf, "%s exists: Append or Overwrite?", name);
-	    query (toplevel_w, buf, "Append", "Overwrite", "Cancel",
+	if (existd (name,NULL) == 0 && confirm()) {
+	    (void) sprintf (buf, "%s exists:\nAppend or Overwrite?", name);
+	    query (cshell_w, buf, "Append", "Overwrite", "Cancel",
 				flistok_append_cb, flistok_overwrite_cb, NULL);
 	} else
 	    flistok_overwrite_cb();
@@ -1204,16 +1179,14 @@ make_flist (name, how)
 char *name;
 char *how;
 {
-	FILE *fp = fopenh (name, how);
+	FILE *fp = fopend (name, NULL, how);
 
 	if (fp) {
 	    write_flist (fp);
 	    (void) fclose (fp);
 	} else {
-	    char buf[1024];
-
-	    (void) sprintf (buf, "Can not open %s: %s", name, syserrstr());
-	    xe_msg (buf, 1);
+	    XmToggleButtonSetState (autols_w, False, False);
+	    XmToggleButtonSetState (autoup_w, False, False);
 	}
 }
 
@@ -1252,4 +1225,4 @@ FILE *fp;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: closemenu.c,v $ $Date: 2000/07/20 04:16:29 $ $Revision: 1.8 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: closemenu.c,v $ $Date: 2004/03/16 18:44:38 $ $Revision: 1.19 $ $Name:  $"};
