@@ -2,84 +2,50 @@
  */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <ctype.h>
 #include <math.h>
-
-#if defined(__STDC__)
-#include <stdlib.h>
-#endif
+#include <errno.h>
+#include <string.h>
 
 #include <Xm/Xm.h>
 #include <Xm/Form.h>
 #include <Xm/Frame.h>
 #include <Xm/Label.h>
 #include <Xm/PushB.h>
-#include <Xm/PanedW.h>
 #include <Xm/CascadeB.h>
 #include <Xm/ToggleB.h>
 #include <Xm/RowColumn.h>
-#include <Xm/FileSB.h>
 #include <Xm/ScrolledW.h>
 #include <Xm/Text.h>
 #include <Xm/Separator.h>
 
-#include "P_.h"
-#include "astro.h"
-#include "circum.h"
-#include "db.h"
+#include "xephem.h"
 
-extern Widget	toplevel_w;
-#define	XtD	XtDisplay(toplevel_w)
-extern Colormap xe_cm;
-
-extern FILE *fopenh P_((char *name, char *how));
-extern Obj *db_scan P_((DBScan *sp));
-extern char *expand_home P_((char *path));
-extern char *getShareDir P_((void));
-extern char *getPrivateDir P_((void));
-extern char *syserrstr P_((void));
-extern int confirm P_((void));
-extern int db_n P_((void));
-extern int isUp P_((Widget w));
-extern void db_read P_((char *fn, int nodups));
-extern void all_newdb P_((int appended));
-extern void db_clr_cp P_((void));
-extern void db_connect_fifo P_((void));
-extern void db_scaninit P_((DBScan *sp, int mask, ObjF *op, int nop));
-extern void get_something P_((Widget w, char *resource, XtArgVal value));
-extern void get_xmstring P_((Widget w, char *resource, char **txtp));
-extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
-extern void prompt_map_cb P_((Widget w, XtPointer client, XtPointer call));
-extern void query P_((Widget tw, char *msg, char *label1, char *label2,
-    char *label3, void (*func1)(void), void (*func2)(void),
-    void (*func3)(void)));
-extern void set_something P_((Widget w, char *resource, XtArgVal value));
-extern void set_xmstring P_((Widget w, char *resource, char *txt));
-extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
-extern void watch_cursor P_((int want));
-extern void wtip P_((Widget w, char *tip));
-extern void xe_msg P_((char *msg, int app_modal));
-
-static void db_create_shell P_((void));
-static void initFSB P_((Widget fsb_w));
-static void db_set_report P_((void));
-static void db_load_cb P_((Widget w, XtPointer client, XtPointer data));
-static void db_openfifo_cb P_((Widget w, XtPointer client, XtPointer data));
-static void db_delall_cb P_((Widget w, XtPointer client, XtPointer data));
-static void db_help_cb P_((Widget w, XtPointer client, XtPointer data));
-static void db_helpon_cb P_((Widget w, XtPointer client, XtPointer data));
-static void db_sharedir_cb P_((Widget w, XtPointer client, XtPointer data));
-static void db_privdir_cb P_((Widget w, XtPointer client, XtPointer data));
-static void db_close_cb P_((Widget w, XtPointer client, XtPointer data));
-static void db_1catrow P_((Widget rc_w, DBCat *dbcp));
+static void db_create_shell (void);
+static void db_set_report (void);
+static void db_openfifo_cb (Widget w, XtPointer client, XtPointer data);
+static void db_index_cb (Widget w, XtPointer client, XtPointer data);
+static void db_delall_cb (Widget w, XtPointer client, XtPointer data);
+static void db_popdown_cb (Widget w, XtPointer client, XtPointer data);
+static void db_relall_cb (Widget w, XtPointer client, XtPointer data);
+static void db_help_cb (Widget w, XtPointer client, XtPointer data);
+static void db_helpon_cb (Widget w, XtPointer client, XtPointer data);
+static void db_loadpb_cb (Widget w, XtPointer client, XtPointer data);
+static void db_close_cb (Widget w, XtPointer client, XtPointer data);
+static void db_1catrow (Widget rc_w, DBCat *dbcp);
+static void dbhdr_close_cb (Widget w, XtPointer client, XtPointer data);
+static void db_showhdr(char *fn);
 
 static Widget dbshell_w;	/* main shell */
 static Widget catsw_w;		/* scrolled window managing the cat list */
 static Widget report_w;		/* label with the dbstats */
-static Widget nodups_w;		/* TB whether to avoid dups on file reads */
-static Widget load1_w;		/* TB whether to load ObjX from .edb with 1 */
+static Widget load1_w;		/* TB whether to load Fav from .edb with 1 */
+static Widget altnames_w;	/* TB whether to load alternate names */
 static Widget ncats_w;		/* number of catalogs label */
 static Widget nobjs_w;		/* number of objects label */
+static Widget dbf_w;		/* form for displaying catalog header */
+static Widget dbt_w;		/* T widget for display catalog header */
 
 /* bring up the db menu, creating if first time */
 void
@@ -93,7 +59,7 @@ db_manage()
 	set_something (dbshell_w, XmNiconic, (XtArgVal)False);
 }
 
-/* called when the database beyond NOBJ has changed in any way.
+/* called when the database beyond the builtin ones has changed in any way.
  * as odd as this seems since *this* menu controls the db contents, this was
  *   added when the db fifo input path was added. it continued to be handy
  *   when initial db files could be loaded automatically and we introduced
@@ -109,16 +75,13 @@ int appended;
 	    db_set_report();
 }
 
-/* called when any of the user-defined objects have changed.
- * all we do is update our tally, if we are up at all.
- */
-/* ARGSUSED */
-void
-db_newobj (id)
-int id;	/* OBJXYZ */
+/* return whether to check for alternate names when reading a catalog */
+int
+db_chkAltNames(void)
 {
-	if (isUp(dbshell_w))
-	    db_set_report();
+	if (!altnames_w)
+	    db_create_shell();
+	return (XmToggleButtonGetState (altnames_w));
 }
 
 /* update the list of catalogs.
@@ -133,26 +96,12 @@ int ndbcp;
 	Widget ww;
 	int n;
 
-#if 0
-	for (n = 0; n < ndbcp; n++) {
-	    int t;
-	    printf ("%-6.6s:", dbcp[n].name);
-	    for (t = 0; t < NOBJTYPES; t++)
-		printf (" %6d", dbcp[n].start[t]);
-	    printf (",");
-	    for (t = 0; t < NOBJTYPES; t++)
-		printf (" %4d", dbcp[n].n[t]);
-	    printf ("\n");
-	}
-	printf ("\n");
-#endif
-
 	/* create if not already */
 	if (!dbshell_w)
 	    db_create_shell();
 
 	/* set count label */
-	sprintf (buf, "%d Loaded Catalogs", ndbcp);
+	sprintf (buf, "%d Loaded Catalog%c", ndbcp, ndbcp==1?' ':'s');
 	set_xmstring (ncats_w, XmNlabelString, buf);
 
 	/* replace workWindow */
@@ -165,8 +114,8 @@ int ndbcp;
 	set_something (catsw_w, XmNworkWindow, (XtArgVal)ww);
 
 	/* fill with each cat info */
-	while (ndbcp--)
-	    db_1catrow (ww, dbcp++);
+	for (n = 0; n < ndbcp; n++)
+	    db_1catrow (ww, dbcp+n);
 
 	/* ok */
 	XtManageChild (ww);
@@ -188,7 +137,7 @@ Cursor c;
 	}
 }
 
-/* return 1 if want to autoload ObjXYZ if read a .edb with 1 entry, else 0 */
+/* return 1 if want to autoload favorite if read a .edb with 1 entry, else 0 */
 int
 db_load1()
 {
@@ -209,20 +158,21 @@ db_create_shell ()
 	    char *tip;		/* button tip text */
 	} HelpOn;
 	static HelpOn helpon[] = {
-	    {"Intro...", "Database - intro",
+	    {"Intro...", "Database",
 		"How to load and delete .edb catalogs files"},
-	    {"on Control...", "Database - control",
+	    {"on Control...", "Database_control",
 		"Information about the Control pulldown menu"},
-	    {"File format...", "Database - files",
+	    {"File format...", "Database_files",
 		"Definition of the XEphem .edb file format"},
-	    {"Notes...", "Database - notes",
+	    {"Notes...", "Database_notes",
 		"Additional info about the XEphem data base"},
 	};
 	Widget mb_w, pd_w, cb_w;
-	Widget pw_w, rc_w, fsb_w;
+	Widget rc_w;
 	Widget dbform_w;
 	XmString str;
 	Widget w;
+	char *s[2];
 	Arg args[20];
 	int n;
 	int i;
@@ -231,12 +181,14 @@ db_create_shell ()
 
 	n = 0;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
-	XtSetArg (args[n], XmNtitle, "xephem Database Load/Delete"); n++;
+	XtSetArg (args[n], XmNtitle, "xephem Database Files"); n++;
 	XtSetArg (args[n], XmNiconName, "DB"); n++;
 	XtSetArg (args[n], XmNdeleteResponse, XmUNMAP); n++;
 	dbshell_w = XtCreatePopupShell ("DB", topLevelShellWidgetClass,
 						    toplevel_w, args, n);
+	setup_icon (dbshell_w);
 	set_something (dbshell_w, XmNcolormap, (XtArgVal)xe_cm);
+	XtAddCallback (dbshell_w, XmNpopdownCallback, db_popdown_cb, NULL);
 	sr_reg (dbshell_w, "XEphem*DB.x", dbcategory, 0);
 	sr_reg (dbshell_w, "XEphem*DB.y", dbcategory, 0);
 	sr_reg (dbshell_w, "XEphem*DB.height", dbcategory, 0);
@@ -269,6 +221,15 @@ db_create_shell ()
 	    set_xmstring (cb_w, XmNlabelString, "Control");
 	    XtManageChild (cb_w);
 
+	    /* handy Index */
+
+	    n = 0;
+	    w = XmCreatePushButton (pd_w, "Idx", args, n);
+	    XtAddCallback (w, XmNactivateCallback, db_index_cb, 0);
+	    set_xmstring(w, XmNlabelString, "Index...");
+	    wtip (w, "Search and inspect loaded objects");
+	    XtManageChild (w);
+
 	    /* delete all */
 
 	    n = 0;
@@ -278,6 +239,15 @@ db_create_shell ()
 	    wtip (w, "Remove all files from memory");
 	    XtManageChild (w);
 
+	    /* reload all */
+
+	    n = 0;
+	    w = XmCreatePushButton (pd_w, "RelAll", args, n);
+	    XtAddCallback (w, XmNactivateCallback, db_relall_cb, 0);
+	    set_xmstring(w, XmNlabelString, "Reload all");
+	    wtip (w, "Reload all catalogs already in memory");
+	    XtManageChild (w);
+
 	    /* make the open fifo button */
 
 	    n = 0;
@@ -287,28 +257,28 @@ db_create_shell ()
 	    wtip (w, "Activate import path for Object definitions");
 	    XtManageChild (w);
 
-	    /* make the no-dups toggle button */
-
-	    n = 0;
-	    XtSetArg (args[n], XmNvisibleWhenOff, True);  n++;
-	    nodups_w = XmCreateToggleButton (pd_w, "NoDups", args, n);
-	    set_xmstring(nodups_w, XmNlabelString, "No Dups");
-	    wtip (nodups_w,
-		"When on, Loading skips objects from other catalogs whose name already exists in memory");
-	    XtManageChild (nodups_w);
-	    sr_reg (nodups_w, NULL, dbcategory, 1);
-
 	    /* make the load-1 toggle button */
 
 	    n = 0;
 	    XtSetArg (args[n], XmNvisibleWhenOff, True);  n++;
 	    load1_w = XmCreateToggleButton (pd_w, "Load1", args, n);
-	    set_xmstring(load1_w, XmNlabelString, "Load Obj when 1");
+	    set_xmstring(load1_w, XmNlabelString, "Make Favorite when read 1");
 	    wtip (load1_w,
-		"When on, reading a file containing 1 entry also loads ObjX, Y or Z");
+		"Reading a file containing 1 entry also adds it to Favorites");
 	    XtManageChild (load1_w);
 	    sr_reg (load1_w, NULL, dbcategory, 1);
 
+	    /* make the Record duplicate names */
+
+	    n = 0;
+	    XtSetArg (args[n], XmNvisibleWhenOff, True);  n++;
+	    altnames_w = XmCreateToggleButton (pd_w, "CheckAltNames", args, n);
+	    set_xmstring(altnames_w, XmNlabelString, "Check alternate names");
+	    wtip (altnames_w,
+		"Check and record alternate names when loading; nice but slow");
+	    XtManageChild (altnames_w);
+	    sr_reg (altnames_w, NULL, dbcategory, 1);
+
 	    /* add a separator */
 
 	    n = 0;
@@ -323,6 +293,13 @@ db_create_shell ()
 	    wtip (w, "Close this window");
 	    XtManageChild (w);
 
+	/* make the Files pulldown */
+
+	s[0] = ".edb";
+	s[1] = ".tle";
+	cb_w = createFSM (mb_w, s, 2, "catalogs", db_loadpb_cb);
+	wtip (cb_w, "Select catalog file to load");
+
 	/* make the help pulldown */
 
 	n = 0;
@@ -373,7 +350,7 @@ db_create_shell ()
 	    XtSetArg (args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
 	    XtSetArg (args[n], XmNcursorPositionVisible, False); n++;
 	    XtSetArg (args[n], XmNblinkRate, 0); n++;
-	    XtSetArg (args[n], XmNrows, 14); n++;
+	    XtSetArg (args[n], XmNrows, 16); n++; /* must match db_set_report */
 	    XtSetArg (args[n], XmNcolumns, 32); n++;
 	    report_w = XmCreateText (rc_w, "Report", args, n);
 	    wtip (report_w,
@@ -393,7 +370,9 @@ db_create_shell ()
 	set_xmstring (ncats_w, XmNlabelString, "0 Loaded Catalogs");
 	XtManageChild (ncats_w);
 
-	/* put the Catalogs and FSB in a paned window */
+	/* create a RC in a SW for showing and deleting loaded catalogs.
+	 * the RC is replaced each time the list changes.
+	 */
 
 	n = 0;
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
@@ -401,127 +380,31 @@ db_create_shell ()
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNmarginHeight, 0); n++;
-	XtSetArg (args[n], XmNmarginWidth, 1); n++;
-	XtSetArg (args[n], XmNsashHeight, 10); n++;
-	XtSetArg (args[n], XmNspacing, 14); n++;
-	pw_w = XmCreatePanedWindow (dbform_w, "Pane", args, n);
-	XtManageChild (pw_w);
-
-	/* create a RC in a SW for showing and deleting loaded catalogs.
-	 * the RC is replaced each time the list changes.
-	 */
-
-	n = 0;
 	XtSetArg (args[n], XmNscrollingPolicy, XmAUTOMATIC); n++;
-	catsw_w = XmCreateScrolledWindow (pw_w, "CatSW", args, n);
+	catsw_w = XmCreateScrolledWindow (dbform_w, "CatSW", args, n);
 	XtManageChild (catsw_w);
 
 	    n = 0;
 	    w = XmCreateRowColumn (catsw_w, "CatRCDummy", args, n);
 	    XtManageChild (w);
-
-	/* make the file selection box */
-
-	n = 0;
-	XtSetArg (args[n], XmNskipAdjust, True); n++; /* PW constraint */
-	XtSetArg (args[n], XmNresizePolicy, XmRESIZE_NONE); n++;
-	fsb_w = XmCreateFileSelectionBox (pw_w, "FSB", args, n);
-	initFSB (fsb_w);	/* do this before managing for Ultrix :-) */
-	XtManageChild (fsb_w);
-
-#if XmVersion >= 1001
-	w = XmFileSelectionBoxGetChild (fsb_w, XmDIALOG_OK_BUTTON);
-	XmProcessTraversal (w, XmTRAVERSE_CURRENT);
-	XmProcessTraversal (w, XmTRAVERSE_CURRENT); /* yes, twice!! */
-#endif
 }
 
-/* init the directory and pattern resources of the given FileSelectionBox.
- * we try to pull these from the basic program resources.
- * also redefine the button for our purposes.
+/* a PB in a file PD has been chosen.
  */
 static void
-initFSB (fsb_w)
-Widget fsb_w;
-{
-	Arg args[20];
-	char buf[1024];
-	Widget w;
-	int n;
-
-	/* helpful tips -- Buttons are really gadgets */
-	w = XmFileSelectionBoxGetChild (fsb_w, XmDIALOG_FILTER_TEXT);
-	wtip (w,
-	    "Any file name pattern; or use buttons below for handy presets.");
-	w = XmFileSelectionBoxGetChild (fsb_w, XmDIALOG_DIR_LIST);
-	wtip (w, "Browse directories, double-click to move");
-	w = XmFileSelectionBoxGetChild (fsb_w, XmDIALOG_LIST);
-	wtip (w, "Files in chosen directory that match pattern; double-click to load");
-	w = XmFileSelectionBoxGetChild (fsb_w, XmDIALOG_TEXT);
-	wtip (w, "File name; enter manually or by browsing with above lists");
-
-	/* set dir and pattern */
-	(void) sprintf (buf, "%s/catalogs",  getShareDir());
-	set_xmstring (fsb_w, XmNdirectory, expand_home(buf));
-	set_xmstring (fsb_w, XmNpattern, "*.edb");
-
-	/* use the Ok button to mean Load */
-	XtAddCallback (fsb_w, XmNokCallback, db_load_cb, NULL);
-	set_xmstring (fsb_w, XmNokLabelString, "Load");
-
-	/* use Cancel for handy Shared dir */
-	w = XmFileSelectionBoxGetChild (fsb_w, XmDIALOG_CANCEL_BUTTON);
-	XtAddCallback(w, XmNactivateCallback, db_sharedir_cb, (XtPointer)fsb_w);
-	set_xmstring (w, XmNlabelString, "Shared\nDir");
-	n = 0;
-	XtSetArg (args[n], XmNmarginWidth, 2); n++;
-	XtSetArg (args[n], XmNmarginLeft, 2); n++;
-	XtSetArg (args[n], XmNmarginRight, 2); n++;
-	XtSetValues (w, args, n);
-
-	/* use Help for handy Private dir */
-	w = XmFileSelectionBoxGetChild (fsb_w, XmDIALOG_HELP_BUTTON);
-	XtRemoveAllCallbacks (w, XmNactivateCallback);
-	XtAddCallback(w, XmNactivateCallback, db_privdir_cb, (XtPointer)fsb_w);
-	set_xmstring (w, XmNlabelString, "Private\nDir");
-	n = 0;
-	XtSetArg (args[n], XmNmarginWidth, 3); n++;
-	XtSetArg (args[n], XmNmarginLeft, 3); n++;
-	XtSetArg (args[n], XmNmarginRight, 3); n++;
-	XtSetValues (w, args, n);
-}
-
-/* callback from the Shared dir PB.
- * client is the FSB
- */
-/* ARGSUSED */
-static void
-db_sharedir_cb (w, client, call)
+db_loadpb_cb (w, client, call)
 Widget w;
 XtPointer client;
 XtPointer call;
 {
-	Widget fsb_w = (Widget)client;
-	char buf[1024];
+	char *fn;
 
-	(void) sprintf (buf, "%s/catalogs", getShareDir());
-	set_xmstring (fsb_w, XmNdirectory, expand_home(buf));
-}
-
-/* callback from the Private dir PB.
- * client is the FSB
- */
-/* ARGSUSED */
-static void
-db_privdir_cb (w, client, call)
-Widget w;
-XtPointer client;
-XtPointer call;
-{
-	Widget fsb_w = (Widget)client;
-
-	set_xmstring (fsb_w, XmNdirectory, getPrivateDir());
+	watch_cursor(1);
+	get_xmstring (w, XmNlabelString, &fn);
+	db_read (fn);
+	XtFree (fn);
+	all_newdb(1);
+	watch_cursor(0);
 }
 
 /* compile the stats report into report_w.
@@ -532,96 +415,91 @@ db_set_report()
 {
 	DBScan dbs;
 	char report[1024];
-	int mask = ALLM & ~PLANETM;
 	Obj *op;
-	int nes=0, ne=0, np=0, nh=0;
-	int nc=0, ng=0, nj=0, nn=0, npn=0, nl=0, nq=0, nr=0, ns=0, no=0;
-	int t=0;
+	int nes=0, ne=0, np=0, nh=0, npm=0, nbs=0;
+	int nc=0, ng=0, nj=0, nn=0, npn=0, nl=0, nq=0, nr=0, ns=0, no=0, ny=0;
+	int n=0;
 
-	for (db_scaninit(&dbs, mask, NULL, 0); (op = db_scan(&dbs))!=NULL; ){
+	for (db_scaninit(&dbs, ALLM, NULL, 0); (op=db_scan(&dbs)) != NULL; ){
 	    switch (op->o_type) {
 	    case FIXED:
 		switch (op->f_class) {
-		case 'C': case 'U': case 'O': nc++; t++; break;
-		case 'G': case 'H': case 'A': ng++; t++; break;
-		case 'N': case 'F': case 'K': nn++; t++; break;
-		case 'J': nj++; t++; break;
-		case 'P': npn++; t++; break;
-		case 'L': nl++; t++; break;
-		case 'Q': nq++; t++; break;
-		case 'R': nr++; t++; break;
+		case 'C': case 'U': case 'O':
+		    nc++;
+		    break;
+		case 'G': case 'H': case 'A':
+		    ng++;
+		    break;
+		case 'N': case 'F': case 'K':
+		    nn++;
+		    break;
+		case 'J':
+		    nj++;
+		    break;
+		case 'P':
+		    npn++;
+		    break;
+		case 'L':
+		    nl++;
+		    break;
+		case 'Q':
+		    nq++;
+		    break;
+		case 'R':
+		    nr++;
+		    break;
 		case 'T': case 'B': case 'D': case 'M': case 'S': case 'V': 
-		    ns++; t++; break;
-		default: no++; t++; break;
+		    ns++;
+		    break;
+		case 'Y':
+		    ny++;
+		    break;
+		default: 
+		    no++;
+		    break;
 		}
 		break;
-	    case ELLIPTICAL: ne++; t++; break;
-	    case HYPERBOLIC: nh++; t++; break;
-	    case PARABOLIC: np++; t++; break;
-	    case EARTHSAT: nes++; t++; break;
-	    case UNDEFOBJ: break;
+	    case BINARYSTAR: nbs++; break;
+	    case PLANET:     npm++; break;
+	    case ELLIPTICAL: ne++;  break;
+	    case HYPERBOLIC: nh++;  break;
+	    case PARABOLIC:  np++;  break;
+	    case EARTHSAT:   nes++; break;
+	    case UNDEFOBJ:          break;
 	    default:
 		printf ("Unknown object type: %d\n", op->o_type);
-		exit (1);
+		abort();
 	    }
+	    n++;
 	}
 
 	(void) sprintf (report, "\
+%6d Solar -- planets and moons\n\
 %6d Solar -- elliptical\n\
 %6d Solar -- hyperbolic\n\
 %6d Solar -- parabolic\n\
 %6d Earth satellites\n\
 %6d Clusters (C,U,O)\n\
 %6d Galaxies (G,H,A)\n\
-%6d Planetary Nebulae (P)\n\
 %6d Nebulae (N,F,K)\n\
+%6d Planetary Nebulae (P)\n\
 %6d Pulsars (L)\n\
 %6d Quasars (Q)\n\
 %6d Radio sources (J)\n\
+%6d Supernovae (Y)\n\
 %6d Supernova Remnants (R)\n\
 %6d Stars (S,V,D,B,M,T)\n\
+%6d Binary systems\n\
 %6d Undefined\
 ",
-	ne, nh, np, nes, nc, ng, npn, nn, nl, nq, nj, nr, ns, no);
+	npm, ne, nh, np, nes, nc, ng, nn, npn, nl, nq, nj, ny, nr, ns, nbs, no);
 
+	/* set report and count in label */
 	set_something (report_w, XmNvalue, (XtArgVal)report);
-
-	/* set count in lable */
-	sprintf (report, "%d Loaded Objects", t);
+	sprintf (report, "%d Loaded Objects", n);
 	set_xmstring (nobjs_w, XmNlabelString, report);
 }
 
-/* callback from the Ok button along the bottom of the FSB.
- * we take this to mean load the current selection.
- */
-/* ARGSUSED */
-static void
-db_load_cb (w, client, data)
-Widget w;
-XtPointer client;
-XtPointer data;
-{
-	XmFileSelectionBoxCallbackStruct *s =
-				    (XmFileSelectionBoxCallbackStruct *)data;
-	DBCat *dbcp;
-	char *fn;
-
-	if (s->reason != XmCR_OK) {
-	    printf ("%s: Unknown reason = 0x%x\n", "db_load_cb()", s->reason);
-	    exit (1);
-	}
-
-	watch_cursor(1);
-	XmStringGetLtoR (s->value, XmSTRING_DEFAULT_CHARSET, &fn);
-	dbcp = db_catfind (fn);
-	if (dbcp)
-	    db_catdel (dbcp);
-	db_read (fn, XmToggleButtonGetState (nodups_w));
-	all_newdb(1);
-	XtFree (fn);
-	watch_cursor(0);
-}
-
 static void
 dbdelall()
 {
@@ -629,6 +507,18 @@ dbdelall()
 	all_newdb(0);
 }
 
+/* callback when user wants to bring up the Index window
+ */
+/* ARGSUSED */
+static void
+db_index_cb (w, client, data)
+Widget w;
+XtPointer client;
+XtPointer data;
+{
+	obj_manage();
+}
+
 /* callback when user wants to delete all files
  */
 /* ARGSUSED */
@@ -639,13 +529,39 @@ XtPointer client;
 XtPointer data;
 {
 	if (confirm())
-	    query (toplevel_w, "Delete all files from memory?",
+	    query (dbshell_w, "Delete all files from memory?",
 		   "Yes .. delete all", "No .. no change", NULL,
 		   dbdelall, NULL, NULL);
 	else 
 	    dbdelall();
 }
 
+static void
+dbrelall()
+{
+	watch_cursor(1);
+	db_rel_all();
+	all_newdb(0);
+	watch_cursor(0);
+}
+
+/* callback when user wants to reload all files
+ */
+/* ARGSUSED */
+static void
+db_relall_cb (w, client, data)
+Widget w;
+XtPointer client;
+XtPointer data;
+{
+	if (confirm())
+	    query (dbshell_w, "Reload all files in memory?",
+		   "Yes -- reload all", "No -- no change", NULL,
+		   dbrelall, NULL, NULL);
+	else 
+	    dbrelall();
+}
+
 /* callback from the open fifo button */
 /* ARGSUSED */
 static void
@@ -669,7 +585,7 @@ XtPointer data;
 "Database files may be read in to add to this list or the list may be deleted."
 };
 
-	hlp_dialog ("DataBase", msg, XtNumber(msg));
+	hlp_dialog ("Database", msg, XtNumber(msg));
 }
 
 /* callback from a specific Help button.
@@ -685,6 +601,17 @@ XtPointer call;
 	hlp_dialog ((char *)client, NULL, 0);
 }
 
+/* ARGSUSED */
+static void
+db_popdown_cb (w, client, data)
+Widget w;
+XtPointer client;
+XtPointer data;
+{
+	if (dbf_w)
+	    XtUnmanageChild (dbf_w);
+}
+
 /* ARGSUSED */
 static void
 db_close_cb (w, client, data)
@@ -711,6 +638,21 @@ XtPointer call;
 	watch_cursor(0);
 }
 
+/* call from any of the Header catalog PBs.
+ * client is a DBCat *.
+ */
+/* ARGSUSED */
+static void
+cathdr_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	watch_cursor(1);
+	db_showhdr (((DBCat *)client)->name);
+	watch_cursor(0);
+}
+
 /* build one new catalog entry */
 static void
 db_1catrow (rc_w, dbcp)
@@ -744,23 +686,35 @@ DBCat *dbcp;
 	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
 	XtSetArg (args[n], XmNleftWidget, pb_w); n++;
-	XtSetArg (args[n], XmNleftOffset, 4); n++;
-	XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
-	XtSetArg (args[n], XmNrightPosition, 45); n++;
+	XtSetArg (args[n], XmNleftOffset, 3); n++;
+	XtSetArg (args[n], XmNshadowThickness, 1); n++;
+	XtSetArg (args[n], XmNmarginHeight, 0); n++;
+	pb_w = XmCreatePushButton (f_w, "Header", args, n);
+	wtip (pb_w, "Display header info from this catalog");
+	XtAddCallback (pb_w, XmNactivateCallback, cathdr_cb, (XtPointer)dbcp);
+	XtManageChild (pb_w);
+
+	n = 0;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
+	XtSetArg (args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNleftWidget, pb_w); n++;
+	XtSetArg (args[n], XmNleftOffset, 3); n++;
 	XtSetArg (args[n], XmNalignment, XmALIGNMENT_END); n++;
 	nl_w = XmCreateLabel (f_w, "NObjs", args, n);
 	XtManageChild (nl_w);
 
 	for (nobjs = i = 0; i < NOBJTYPES; i++)
-	    nobjs += dbcp->n[i];
+	    nobjs += dbcp->tmem[i].nuse;
 	(void) sprintf (buf, "%6d", nobjs);
 	set_xmstring (nl_w, XmNlabelString, buf);
 
 	n = 0;
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
-	XtSetArg (args[n], XmNleftPosition, 47); n++;
+	XtSetArg (args[n], XmNleftAttachment, XmATTACH_WIDGET); n++;
+	XtSetArg (args[n], XmNleftWidget, nl_w); n++;
+	XtSetArg (args[n], XmNleftOffset, 3); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
 	cl_w = XmCreateLabel (f_w, "Name", args, n);
@@ -768,5 +722,78 @@ DBCat *dbcp;
 	set_xmstring (cl_w, XmNlabelString, dbcp->name);
 }
 
+/* display the header of the given catalog file */
+static void
+db_showhdr(char *fn)
+{
+	char buf[1024];
+	FILE *fp;
+	char *hdr;
+	int n, l;
+
+	/* create the text window first time */
+	if (!dbf_w) {
+	    Arg args[20];
+	    Widget w;
+
+	    n = 0;
+	    XtSetArg (args[n], XmNcolormap, xe_cm); n++;
+	    XtSetArg (args[n], XmNmarginWidth, 10);  n++;
+	    XtSetArg (args[n], XmNmarginHeight, 10);  n++;
+	    dbf_w = XmCreateFormDialog (toplevel_w, "DBHeader", args, n);
+	    set_something (dbf_w, XmNcolormap, (XtArgVal)xe_cm);
+	    set_xmstring (dbf_w, XmNdialogTitle, "xephem Catalog header");
+
+	    n = 0;
+	    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_FORM); n++;
+	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
+	    XtSetArg (args[n], XmNleftPosition, 10); n++;
+	    w = XmCreatePushButton (dbf_w, "Close", args, n);
+	    XtAddCallback (w, XmNactivateCallback, dbhdr_close_cb, NULL);
+	    XtManageChild (w);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
+	    XtSetArg (args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
+	    XtSetArg (args[n], XmNbottomWidget, w); n++;
+	    XtSetArg (args[n], XmNbottomOffset, 10); n++;
+	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
+	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+	    XtSetArg (args[n], XmNrows, 24); n++;
+	    XtSetArg (args[n], XmNcolumns, 82); n++;
+	    XtSetArg (args[n], XmNeditable, False); n++;
+	    XtSetArg (args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
+	    XtSetArg (args[n], XmNcursorPositionVisible, False); n++;
+	    XtSetArg (args[n], XmNblinkRate, 0); n++;
+	    dbt_w = XmCreateScrolledText (dbf_w, "DBH", args, n);
+	    XtManageChild (dbt_w);
+	}
+
+	fp = fopend (fn, "catalogs", "r");
+	if (!fp)
+	    return;
+
+	hdr = XtMalloc(1);
+	hdr[0] = '\0';
+	for (n = l = 0; fgets (buf, sizeof(buf), fp) != NULL; l += n) {
+	    if (dbline_candidate (buf) == 0)
+		continue;
+	    n = strlen (buf);
+	    hdr = XtRealloc (hdr, n+l+1);
+	    strcpy (hdr+l, buf);
+	}
+	XmTextSetString (dbt_w, hdr);
+	XtFree (hdr);
+
+	XtManageChild (dbf_w);
+}
+
+static void
+dbhdr_close_cb (Widget w, XtPointer client, XtPointer call)
+{
+	if (dbf_w)
+	    XtUnmanageChild (dbf_w);
+}
+
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: dbmenu.c,v $ $Date: 2000/09/12 04:53:13 $ $Revision: 1.20 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: dbmenu.c,v $ $Date: 2005/04/27 17:04:59 $ $Revision: 1.48 $ $Name:  $"};
