@@ -91,24 +91,27 @@ fav_manage()
 void
 fav_add (Obj *op)
 {
-	Favorite *newfp;
+	Favorite *addfp;
+	int i;
 
 	watch_cursor(1);
 
 	if (!favshell_w)
 	    fav_create();
 
-	/* check for dup */
-	if (fav_already (op->o_name) >= 0) {
-	    watch_cursor(0);
-	    return;
+	/* replace if name is dup, else add */
+	i = fav_already (op->o_name);
+	if (i >= 0) {
+	    addfp = &favs[i];
+	    memcpy (&addfp->o, op, sizeof(Obj));
+	    addfp->op = NULL;	/* local copy now for sure */
+	} else {
+	    addfp = favGrow();
+	    memcpy (&addfp->o, op, sizeof(Obj));
+	    addfp->op = (op->o_flags & FLDSTAR) ? NULL : op;
 	}
 
-	/* add, assume in DB unless field star */
-	newfp = favGrow();
-	memcpy (&newfp->o, op, sizeof(Obj));
-	newfp->op = (op->o_flags & FLDSTAR) ? NULL : op;
-	newfp->on = 1;
+	addfp->on = 1;
 
 	/* display */
 	bldExport();
@@ -161,6 +164,7 @@ fav_newdb()
 	    favs[i].op = srchRealDB (favs[i].o.o_name);
 
 	bldExport();
+	showFavorites();
 }
 
 
@@ -457,7 +461,7 @@ int i;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
 	lbl_w = XmCreateLabel (f_w, "FL", args, n);
-	db_write_line (&favs[i].o, buf);
+	db_write_line (favs[i].op ? favs[i].op : &favs[i].o, buf);
 	set_xmstring (lbl_w, XmNlabelString, buf);
 	XtManageChild (lbl_w);
 }
@@ -821,7 +825,7 @@ saveFavs(char *fn)
 	/* write */
 	fprintf (fp, "<Favorites>\n");
 	for (i = 0; i < nfavs; i++) {
-	    db_write_line (&favs[i].o, line);
+	    db_write_line (favs[i].op ? favs[i].op : &favs[i].o, line);
 	    fprintf (fp, "  <favorite on='%s'>%s</favorite>\n",
 					favs[i].on ? "true" : "false", line);
 	}
@@ -853,5 +857,4 @@ srchRealDB (char *name)
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: favmenu.c,v $ $Date: 2010/06/14 02:30:31 $ $Revision: 1.26 $ $Name:  $"};
-
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: favmenu.c,v $ $Date: 2012/10/06 02:19:35 $ $Revision: 1.29 $ $Name:  $"};
