@@ -4,121 +4,74 @@
 #include <math.h>
 #include <signal.h>
 #include <errno.h>
-
-#ifdef __STDC__
 #include <stdlib.h>
 #include <string.h>
-typedef const void *qsort_arg; 
-#else
-extern char *getenv();
-extern void *memset();
-typedef void *qsort_arg;
-#endif
-
-#if defined(_POSIX_SOURCE)
 #include <unistd.h>
-#else
-extern int close();
-#endif
 
 #include <Xm/Form.h>
 #include <Xm/Label.h>
 #include <Xm/MessageB.h>
 #include <Xm/PushB.h>
 #include <Xm/Separator.h>
+#include <Xm/RowColumn.h>
 #include <Xm/TextF.h>
 #include <Xm/ToggleB.h>
 
-#include "P_.h"
-#include "astro.h"
-#include "circum.h"
-#include "db.h"
-#include "net.h"
+#include "xephem.h"
 
-extern Widget toplevel_w;
-#define XtD     XtDisplay(toplevel_w)
-extern XtAppContext xe_app;
-extern Colormap xe_cm;
+#define	FOVLIM		degrad(30.0)	/* max fov we'll fetch */
 
-#define	DUPSEP		degrad(4/3600.)	/* dup if < this close, rads */
-#define	DUPMAG		(3.0)		/* dup if mags differ < this much */
-#define	FOVLIM		degrad(15.0)	/* max fov we'll fetch */
-
-extern Now *mm_get_now P_((void));
-extern char *getShareDir P_((void));
-extern double delra P_((double dra));
-extern int GSCFetch P_((double ra0, double dec0, double fov, double fmag,
-    ObjF **opp, int nopp, char msg[]));
-extern int GSCSetup P_((char *cdp, char *chp, char msg[]));
-extern int USNOFetch P_((double r0, double d0, double fov, double fmag,
-    ObjF **opp, int nopp, char msg[]));
-extern int USNOSetup P_((char *cdp, int wantgsc, char msg[]));
-extern int existsh P_((char *name));
-extern int gscesofetch P_((Now *np, double ra, double dec, double fov,
-    double mag, ObjF **opp, int nop, char msg[]));
-extern int gscnetfetch P_((char *host, Now *np, double ra, double dec,
-    double fov, double mag, ObjF **opp, int nop, char msg[]));
-extern int isUp P_((Widget w));
-extern int obj_cir P_((Now *np, Obj *op));
-extern int xe2chkfile P_((char *file, char *msg));
-extern int xe2fetch P_((char *file, Now *np, double ra, double dec,
-    double fov, double mag, ObjF **opp, char *msg));
-extern Obj *db_scan P_((DBScan *sp));  
-extern void XCheck P_((XtAppContext app));
-extern void defaultTextFN P_((Widget w, int setcols, char *x, char *y));
-extern void db_scaninit P_((DBScan *sp, int mask, ObjF *op, int nop));
-extern void db_update P_((Obj *op));
-extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
-extern void set_something P_((Widget w, char *resource, XtArgVal value));
-extern void set_xmstring P_((Widget w, char *resource, char *txt));
-extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
-extern void sv_loadfs P_((int force));
-extern void watch_cursor P_((int want));
-extern void wtip P_((Widget w, char *tip));
-extern void xe_msg P_((char *msg, int app_modal));
-
-static int objfd_cmp P_((qsort_arg v1, qsort_arg v2));
-static int fs_nodbdups P_((ObjF *fop, int nfop, double ra, double dec,
-    double fov));
-static int scanchk P_((Obj *mop, Obj *op, double cdec));
-static void fs_create_fsshell P_((void));
-static int fs_save P_((void));
-static void fs_setup P_((void));
-static void apply_cb P_((Widget w, XtPointer client, XtPointer call));
-static void ok_cb P_((Widget w, XtPointer client, XtPointer call));
-static void gsctb_cb P_((Widget w, XtPointer client, XtPointer call));
-static void pmtb_cb P_((Widget w, XtPointer client, XtPointer call));
-static void cancel_cb P_((Widget w, XtPointer client, XtPointer call));
-static void help_cb P_((Widget w, XtPointer client, XtPointer call));
+static int objfd_cmp (const void *v1, const void *v2);
+static int fs_nodbdups (ObjF *fop, int nfop, double ra, double dec,
+    double fov);
+static int scanchk (Obj *mop, double dupsep, double dupmag, Obj *op,
+    double cdec);
+static void fs_create_fsshell (void);
+static int fs_save (void);
+static void fs_setup (void);
+static void apply_cb (Widget w, XtPointer client, XtPointer call);
+static void ok_cb (Widget w, XtPointer client, XtPointer call);
+static void gsctb_cb (Widget w, XtPointer client, XtPointer call);
+static void pmtb_cb (Widget w, XtPointer client, XtPointer call);
+static void cancel_cb (Widget w, XtPointer client, XtPointer call);
+static void help_cb (Widget w, XtPointer client, XtPointer call);
 
 static Widget fsshell_w;	/* the main shell */
 static Widget cdtf_w, cdtb_w;	/* GSC CD-ROM text field and toggle button */
-static Widget nettf_w, nettb_w;	/* GSC network text field and toggle button */
+static Widget gsc23tb_w;	/* GSC 2.3 network toggle button */
+static Widget gsc23tf_w;	/* GSC 2.3 network text field */
 static Widget chtf_w, chtb_w;	/* GSC Cache text field and toggle button */
 static Widget notf_w, notb_w;	/* USNO text field and toggle button */
-static Widget etb_w;		/* ESO toggle button */
+static Widget gsc22tb_w;	/* GSC 2.2 toggle button */
+static Widget gsc22tf_w;	/* GSC 2.2 text field */
 static Widget ppmtf_w, ppmtb_w;	/* PMM text field and toggle button */
 static Widget tyctf_w, tyctb_w;	/* Tycho text field and toggle button */
+static Widget ucactf_w,ucactb_w;/* UCAC text field and toggle button */
 static Widget nodupt_w;		/* no dups toggle button */
+static Widget nodupsep_w;	/* no dups sep TF */
+static Widget nodupmag_w;	/* no dups mag TF */
 
-/* TB codes */
+/* GSC TB codes */
 typedef enum {
-    CDROMTB, CACHETB, NETTB, ESOTB
+    CDROMTB, CACHETB, NETTB, GSC22TB
 } TB;
 
 static int cd_on;		/* whether GSC CDROM is currently on */
 static char *cd_dn;		/* GSC CDROM dir */
 static int ch_on;		/* whether GSC Cache is currently on */
 static char *ch_dn;		/* GSC Cache dir */
-static int net_on;		/* whether GSC network connection is on */
-static char *net_host;		/* GSC network host name */
-static int e_on;		/* whether ESO network connection is on */
+static int gsc23_on;		/* whether GSC network connection is on */
+static char *gsc23_host;		/* GSC network host name */
+static int gsc22_on;		/* whether GSC 2.2 STScI connection is on */
+static char *gsc22_dn;		/* GSC22 directory */
 static int usno_on;		/* whether USNO is currently on */
 static char *usno_fn;		/* USNO filename */
 static int ppm_on;		/* whether ppm is currently on */
 static char *ppm_fn;		/* ppm filename */
 static int tyc_on;		/* whether tycho is currently on */
 static char *tyc_fn;		/* tycho filename */
+static int ucac_on;		/* whether UCAC is currently on */
+static char *ucac_dir;		/* UCAC dirname */
 static int nodup_on;		/* whether dup checking is on */
 
 static char fscategory[] = "Field stars";
@@ -144,6 +97,16 @@ fs_manage()
 	set_something (fsshell_w, XmNiconic, (XtArgVal)False);
 }
 
+/* set new state for nodups, return current state */
+int
+fs_setnodups (int new)
+{
+	int old = nodup_on;
+
+	XmToggleButtonSetState (nodupt_w, nodup_on = new, False);
+	return (old);
+}
+
 /* call to fetch a set of ObjF from the field star sources.
  * don't complain just if none are turned on at the moment.
  * we may eliminate any that seem to be dups of existing FIXED db objects.
@@ -153,21 +116,18 @@ fs_manage()
  * N.B. we only malloc *opp if we return > 0.
  */
 int
-fs_fetch (np, ra, dec, fov, mag, opp)
-Now *np;	/* now */
-double ra, dec;	/* at np->n_epoch */
-double fov;	/* field of view, rads */
-double mag;	/* limiting mag */
-ObjF **opp;	/* we set *opp to a malloced list of ObjF */
+fs_fetch (
+Now *np,		/* now */
+double ra, double dec,	/* at np->n_epoch */
+double fov,		/* field of view, rads */
+double mag,		/* limiting mag */
+ObjF **opp) 		/* we set *opp to a malloced list of ObjF */
 {
 	ObjF *op = NULL;
 	int nop = 0;
 	int s = 0;
 	char msg[256];
-	int i;
-
-	if(!cd_on &&!ch_on &&!net_on &&!e_on &&!usno_on &&!ppm_on &&!tyc_on)
-	    return (-1);
+	int i, n;
 
 	watch_cursor (1);
 
@@ -183,43 +143,72 @@ ObjF **opp;	/* we set *opp to a malloced list of ObjF */
 
 	/* get the PM catalogs, if desired */
 	if (ppm_on) {
-	    nop = xe2fetch (ppm_fn, np, ra, dec, fov, mag, &op, msg);
-	    if (nop < 0)
+	    n = xe2fetch (ppm_fn, np, ra, dec, fov, mag, &op, msg);
+	    if (n < 0) {
+		xe_msg (1, "PPM: %s", msg);
 		ppm_on = 0;
+		fs_setup();
+	    } else
+		nop += n;
 	}
 	if (tyc_on) {
-	    nop = xe2fetch (tyc_fn, np, ra, dec, fov, mag, &op, msg);
-	    if (nop < 0)
+	    n = xe2fetch (tyc_fn, np, ra, dec, fov, mag, &op, msg);
+	    if (n < 0) {
+		xe_msg (1, "Tycho: %s", msg);
 		tyc_on = 0;
+		fs_setup();
+	    } else
+		nop += n;
+	}
+	if (ucac_on) {
+	    n = UCACFetch (ra, dec, fov, mag, &op, nop, msg);
+	    if (n < 0) {
+		xe_msg (1, "UCAC: %s", msg);
+		ucac_on = 0;
+		fs_setup();
+	    } else
+		nop = n;
 	}
 
-	/* now add the GSC, if desired -- enforce max fov */
-	if (net_on) {
-	    nop = gscnetfetch (net_host, np, ra, dec, fov, mag, &op,nop,msg);
-	    if (nop < 0)
-		net_on = 0;
+	/* now add the GSC 2.3, if desired -- enforce max fov */
+	if (gsc23_on) {
+	    n = gsc23fetch (gsc23_host, np, ra, dec, fov, mag, &op, nop, msg);
+	    if (n < 0) {
+		xe_msg (1, "NET: %s", msg);
+		gsc23_on = 0;
+		fs_setup();
+	    } else
+		nop = n;
 	}
-	if (e_on) {
-	    nop = gscesofetch (np, ra, dec, fov, mag, &op, nop, msg);
-	    if (nop < 0)
-		e_on = 0;
+	if (gsc22_on) {
+	    n = xe3fetch (gsc22_dn, ra, dec, fov, mag, &op, nop, msg);
+	    if (n < 0) {
+		xe_msg (1, "GSC 2.2: %s", msg);
+		gsc22_on = 0;
+		fs_setup();
+	    } else
+		nop = n;
 	}
 	if (cd_on || ch_on) {
-	    nop = GSCFetch (ra, dec, fov, mag, &op, nop, msg);
-	    if (nop < 0)
+	    n = GSCFetch (ra, dec, fov, mag, &op, nop, msg);
+	    if (n < 0) {
+		xe_msg (1, "GSC: %s", msg);
 		cd_on = 0;
+		fs_setup();
+	    } else
+		nop = n;
 	}
-	if (nop < 0)
-	    xe_msg (msg, 1);
 
 	/* now add USNO if desired -- enforce max fov */
 	if (usno_on) {
-	    nop = USNOFetch (ra, dec, fov, mag, &op, nop, msg);
-	    if (nop < 0)
+	    n = USNOFetch (ra, dec, fov, mag, &op, nop, msg);
+	    if (n < 0) {
+		xe_msg (1, "USNO: %s", msg);
 		usno_on = 0;
+		fs_setup();
+	    } else
+		nop = n;
 	}
-	if (nop < 0)
-	    xe_msg (msg, 1);
 
 	/* set the s_fields to np */
 	for (i = 0; i < nop; i++)
@@ -264,13 +253,12 @@ ObjF **opp;	/* we set *opp to a malloced list of ObjF */
 int
 fs_pmon()
 {
-	return (ppm_on || tyc_on);
+	return (ppm_on || tyc_on || ucac_on);
 }
 
 /* called to put up or remove the watch cursor.  */
 void
-fs_cursor (c)
-Cursor c;
+fs_cursor (Cursor c)
 {
 	Window win;
 
@@ -284,8 +272,7 @@ Cursor c;
 
 /* qsort-style comparison of two ObjF's by dec */
 static int
-objfd_cmp (v1, v2)
-qsort_arg v1, v2;
+objfd_cmp (const void* v1, const void* v2)
 {
 	Obj *op1 = (Obj *)v1;
 	Obj *op2 = (Obj *)v2;
@@ -298,8 +285,8 @@ qsort_arg v1, v2;
 	return (0);
 }
 
-/* squeeze out all entries in fop[] which are located within DUPSEP and have a
- *   magnitude within DUPMAG of any FIXED object currently in the database. We
+/* squeeze out all entries in fop[] which are located within dupsep and have a
+ *   magnitude within dupmag of any FIXED object currently in the database. We
  *   also squeeze out any GSC or USNO stars which meet those same criteria
  *   for any PM star.
  * when finished, all remaining entries will be contiguous at the front of the
@@ -310,18 +297,25 @@ qsort_arg v1, v2;
  * N.B. we assume all entries in fop[] are within fov of ra/dec.
  */
 static int
-fs_nodbdups (fop, nfop, ra, dec, fov)
-ObjF *fop;
-int nfop;
-double ra, dec, fov;
+fs_nodbdups (ObjF *fop, int nfop, double ra, double dec, double fov)
 {
 	double rov = fov/2;
+	double dupsep, dupmag;
 	Obj *op;
 	DBScan dbs;
 	int l, u, m;
 	Obj *mop;
 	double diff;
 	double cdec;
+	char *txt;
+
+	/* get sep and mag */
+	txt = XmTextFieldGetString (nodupsep_w);
+	dupsep = degrad(atod(txt)/3600.);
+	XtFree (txt);
+	txt = XmTextFieldGetString (nodupmag_w);
+	dupmag = atod(txt);
+	XtFree (txt);
 
 	/* sort fop by increasing dec */
 	qsort ((void *)fop, nfop, sizeof(ObjF), objfd_cmp);
@@ -340,20 +334,20 @@ double ra, dec, fov;
 	    cdec = cos(mop->s_dec);
 	    for (u = m+1; u < nfop; u++) {
 		op = (Obj *)&fop[u];
-		if (fabs(mop->s_dec - op->s_dec) >= DUPSEP)
+		if (fabs(mop->s_dec - op->s_dec) >= dupsep)
 		    break;
 		if (op->o_name[0] == 'G'
-			&& cdec*delra(mop->s_ra - op->s_ra) < DUPSEP
-			&& fabs(get_mag(mop) - get_mag(op)) < DUPMAG)
+			&& cdec*delra(mop->s_ra - op->s_ra) < dupsep
+			&& fabs(get_mag(mop) - get_mag(op)) < dupmag)
 		    op->o_flags |= FLDSTAR;
 	    }
 	    for (l = m-1; l >= 0; l--) {
 		op = (Obj *)&fop[l];
-		if (fabs(mop->s_dec - op->s_dec) >= DUPSEP)
+		if (fabs(mop->s_dec - op->s_dec) >= dupsep)
 		    break;
 		if (op->o_name[0] == 'G'
-			&& cdec*delra(mop->s_ra - op->s_ra) < DUPSEP
-			&& fabs(get_mag(mop) - get_mag(op)) < DUPMAG)
+			&& cdec*delra(mop->s_ra - op->s_ra) < dupsep
+			&& fabs(get_mag(mop) - get_mag(op)) < dupmag)
 		    op->o_flags |= FLDSTAR;
 	    }
 	}
@@ -388,10 +382,10 @@ double ra, dec, fov;
 	     * N.B. remember, u and l have crossed each other by now.
 	     */
 	    for (; l < nfop; l++)
-		if (scanchk ((Obj *)(&fop[l]), op, cdec) < 0)
+		if (scanchk ((Obj *)(&fop[l]), dupsep, dupmag, op, cdec) < 0)
 		    break;
 	    for (; u >= 0; --u)
-		if (scanchk ((Obj *)(&fop[u]), op, cdec) < 0)
+		if (scanchk ((Obj *)(&fop[u]), dupsep, dupmag, op, cdec) < 0)
 		    break;
 	}
 
@@ -412,20 +406,18 @@ double ra, dec, fov;
 	return (l);
 }
 
-/* if mop is further than DUPSEP from op in dec, return -1.
- * then if mop is within DUPSEP in ra and within DUPMAG in mag too set FLDSTAR
+/* if mop is further than dupsep from op in dec, return -1.
+ * then if mop is within dupsep in ra and within dupmag in mag too set FLDSTAR
  *   in mop->o_flags.
  * return 0.
  */
 static int
-scanchk (mop, op, cdec)
-Obj *mop, *op;
-double cdec;
+scanchk (Obj *mop, double dupsep, double dupmag, Obj *op, double cdec)
 {
-	if (fabs(mop->s_dec - op->s_dec) >= DUPSEP)
+	if (fabs(mop->s_dec - op->s_dec) >= dupsep)
 	    return (-1);
-	if (cdec*delra(mop->s_ra - op->s_ra) < DUPSEP
-		&& fabs(get_mag(mop) - get_mag(op)) < DUPMAG)
+	if (cdec*delra(mop->s_ra - op->s_ra) < dupsep
+		&& fabs(get_mag(mop) - get_mag(op)) < dupmag)
 	    mop->o_flags |= FLDSTAR;
 	return (0);
 }
@@ -434,6 +426,7 @@ static void
 fs_create_fsshell()
 {
 	Widget fsform_w, f, w;
+	Widget l_w, rc_w;
 	Arg args[20];
 	int n;
 
@@ -446,6 +439,7 @@ fs_create_fsshell()
 	XtSetArg (args[n], XmNdeleteResponse, XmUNMAP); n++;
 	fsshell_w = XtCreatePopupShell ("FieldStars", topLevelShellWidgetClass,
 							toplevel_w, args, n);
+	setup_icon (fsshell_w);
 	set_something (fsshell_w, XmNcolormap, (XtArgVal)xe_cm);
 	sr_reg (fsshell_w, "XEphem*FieldStars.x", fscategory, 0);
 	sr_reg (fsshell_w, "XEphem*FieldStars.y", fscategory, 0);
@@ -476,6 +470,7 @@ fs_create_fsshell()
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNcolumns, 60); n++;
 	    cdtf_w = XmCreateTextField (fsform_w, "GSCCDDirectory", args, n);
+	    defaultTextFN (cdtf_w, 0, "/mnt/cdrom", NULL);
 	    wtip (cdtf_w, "Pathname to the root of the CDROM when mounted");
 	    XtManageChild (cdtf_w);
 	    sr_reg (cdtf_w, NULL, fscategory, 1);
@@ -527,16 +522,18 @@ fs_create_fsshell()
 	    XtManageChild (chtb_w);
 	    sr_reg (chtb_w, NULL, fscategory, 1);
 
-	    /* make the GSC network toggle/text */
+	    /* make the GSC 2.3 network toggle/text */
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
 	    XtSetArg (args[n], XmNtopWidget, chtf_w); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNcolumns, 60); n++;
-	    nettf_w = XmCreateTextField (fsform_w, "XEphemdbdURL", args, n);
-	    wtip (nettf_w, "URL from which to fetch GSC stars");
-	    sr_reg (nettf_w, NULL, fscategory, 1);
+	    gsc23tf_w = XmCreateTextField (fsform_w, "GSC23URL", args, n);
+	    defaultTextFN (gsc23tf_w, 0,
+	    	"http://gsss.stsci.edu/webservices/vo/ConeSearch.aspx", NULL);
+	    wtip (gsc23tf_w, "URL from which to fetch GSC 2.3 stars");
+	    sr_reg (gsc23tf_w, NULL, fscategory, 1);
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
@@ -544,39 +541,53 @@ fs_create_fsshell()
 	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNleftOffset, 15); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_WIDGET); n++;
-	    XtSetArg (args[n], XmNrightWidget, nettf_w); n++;
+	    XtSetArg (args[n], XmNrightWidget, gsc23tf_w); n++;
 	    XtSetArg (args[n], XmNrightOffset, 10); n++;
 	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
-	    nettb_w = XmCreateToggleButton (fsform_w, "XEphemdbdNet", args, n);
-	    XtAddCallback (nettb_w, XmNvalueChangedCallback, gsctb_cb,
+	    gsc23tb_w = XmCreateToggleButton (fsform_w, "GSC23Net", args, n);
+	    XtAddCallback (gsc23tb_w, XmNvalueChangedCallback, gsctb_cb,
 							    (XtPointer)NETTB);
-	    set_xmstring (nettb_w, XmNlabelString, "Internet to xephemdbd:");
-	    wtip (nettb_w,"Whether to use an Internet XEphem server for stars");
-	    sr_reg (nettb_w, NULL, fscategory, 1);
-	    XtManageChild (nettb_w);
-	    XtManageChild (nettf_w);
+	    set_xmstring (gsc23tb_w, XmNlabelString, "Internet to GSC 2.3:");
+	    wtip (gsc23tb_w,"Whether to use Internet to fetch GSC 2.3 stars");
+	    sr_reg (gsc23tb_w, NULL, fscategory, 1);
+	    XtManageChild (gsc23tb_w);
+	    XtManageChild (gsc23tf_w);
 
-	    /* make the ESO network toggle/text */
+	    /* make the GSC 2.2 toggle/text */
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
-	    XtSetArg (args[n], XmNtopWidget, nettf_w); n++;
+	    XtSetArg (args[n], XmNtopWidget, gsc23tf_w); n++;
+	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+	    XtSetArg (args[n], XmNcolumns, 60); n++;
+	    gsc22tf_w = XmCreateTextField (fsform_w, "GSC22Dir", args, n);
+	    wtip (gsc22tf_w, "Directory to GSC 2.2 catalog");
+	    defaultTextFN (gsc22tf_w, 0, getShareDir(), "catalogs/GSC2201");
+	    sr_reg (gsc22tf_w, NULL, fscategory, 1);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	    XtSetArg (args[n], XmNtopWidget, gsc23tf_w); n++;
 	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	    XtSetArg (args[n], XmNleftOffset, 15); n++;
+	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_WIDGET); n++;
+	    XtSetArg (args[n], XmNrightWidget, gsc22tf_w); n++;
+	    XtSetArg (args[n], XmNrightOffset, 10); n++;
 	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
-	    etb_w = XmCreateToggleButton (fsform_w, "ESONet", args, n);
-	    XtAddCallback (etb_w, XmNvalueChangedCallback, gsctb_cb,
-							    (XtPointer)ESOTB);
-	    set_xmstring (etb_w, XmNlabelString, "Internet to ESO");
-	    wtip(etb_w,"Whether to use the Internet to ESO to fetch GSC stars");
-	    XtManageChild (etb_w);
-	    sr_reg (etb_w, NULL, fscategory, 1);
+	    gsc22tb_w = XmCreateToggleButton (fsform_w, "GSC22", args, n);
+	    XtAddCallback (gsc22tb_w, XmNvalueChangedCallback, gsctb_cb,
+							    (XtPointer)GSC22TB);
+	    set_xmstring (gsc22tb_w, XmNlabelString, "GSC 2.2 Directory:");
+	    wtip (gsc22tb_w,"Whether to use local GSC 2.2 and one PM catalog");
+	    sr_reg (gsc22tb_w, NULL, fscategory, 1);
+	    XtManageChild (gsc22tb_w);
+	    XtManageChild (gsc22tf_w);
 
 	/* make the USNO section title */
 
 	n = 0;
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNtopWidget, etb_w); n++;
+	XtSetArg (args[n], XmNtopWidget, gsc22tf_w); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
 	w = XmCreateLabel (fsform_w, "NOMT", args, n);
@@ -646,6 +657,7 @@ fs_create_fsshell()
 	    XtSetArg (args[n], XmNrightOffset, 10); n++;
 	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
 	    ppmtb_w = XmCreateToggleButton (fsform_w, "PPM", args, n);
+	    XtAddCallback (ppmtb_w, XmNvalueChangedCallback, pmtb_cb, NULL);
 	    set_xmstring (ppmtb_w, XmNlabelString, "PPM catalog: ");
 	    wtip (ppmtb_w, "Whether to access the PPM catalog, ppm.xe2");
 	    XtManageChild (ppmtb_w);
@@ -674,41 +686,110 @@ fs_create_fsshell()
 	    XtSetArg (args[n], XmNrightOffset, 10); n++;
 	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
 	    tyctb_w = XmCreateToggleButton (fsform_w, "Tycho", args, n);
+	    XtAddCallback (tyctb_w, XmNvalueChangedCallback, pmtb_cb, NULL);
 	    set_xmstring (tyctb_w, XmNlabelString, "Hipparcos + Tycho2: ");
 	    wtip (tyctb_w, "Whether to access the combined Hipparcos and Tycho2 catalog");
 	    XtManageChild (tyctb_w);
 	    sr_reg (tyctb_w, NULL, fscategory, 1);
 
-	    /* enforce and tie together as 1-of-2 pair */
-	    if(XmToggleButtonGetState(ppmtb_w) &&
-	    				XmToggleButtonGetState(tyctb_w)) {
-		xe_msg ("PPM and Tycho are both on -- turning off Tycho", 0);
-		XmToggleButtonSetState (tyctb_w, False, False);
-	    }
-	    XtAddCallback (ppmtb_w, XmNvalueChangedCallback, pmtb_cb,
-							    (XtPointer)tyctb_w);
-	    XtAddCallback (tyctb_w, XmNvalueChangedCallback, pmtb_cb,
-							    (XtPointer)ppmtb_w);
-
-	    /* make the No dups toggle */
+	    /* make the UCAC toggle/text */
 
 	    n = 0;
 	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
 	    XtSetArg (args[n], XmNtopWidget, tyctf_w); n++;
+	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
+	    XtSetArg (args[n], XmNcolumns, 60); n++;
+	    ucactf_w = XmCreateTextField (fsform_w, "UCACDirectory", args, n);
+	    wtip (ucactf_w, "Full path to directory of UCAC files");
+	    defaultTextFN (ucactf_w, 0, getShareDir(), "catalogs/UCAC");
+	    XtManageChild (ucactf_w);
+	    sr_reg (ucactf_w, NULL, fscategory, 1);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	    XtSetArg (args[n], XmNtopWidget, tyctf_w); n++;
+	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
+	    XtSetArg (args[n], XmNleftOffset, 15); n++;
+	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_WIDGET); n++;
+	    XtSetArg (args[n], XmNrightWidget, ucactf_w); n++;
+	    XtSetArg (args[n], XmNrightOffset, 10); n++;
+	    XtSetArg (args[n], XmNalignment, XmALIGNMENT_BEGINNING); n++;
+	    ucactb_w = XmCreateToggleButton (fsform_w, "UCAC", args, n);
+	    XtAddCallback (ucactb_w, XmNvalueChangedCallback, pmtb_cb, NULL);
+	    set_xmstring (ucactb_w, XmNlabelString, "UCAC: ");
+	    wtip (ucactb_w,"Whether to access the USNO CCD Astrograph Catalog");
+	    XtManageChild (ucactb_w);
+	    sr_reg (ucactb_w, NULL, fscategory, 1);
+
+	    /* enforce and tie together as 1-of-3 pair */
+	    if (XmToggleButtonGetState(ppmtb_w) +
+					XmToggleButtonGetState(tyctb_w) +
+					XmToggleButtonGetState(ucactb_w) > 1) {
+		xe_msg (0, "Only one PM catalog -- using Tycho");
+		XmToggleButtonSetState (ppmtb_w, False, False);
+		XmToggleButtonSetState (tyctb_w, True, False);
+		XmToggleButtonSetState (ucactb_w, False, False);
+	    }
+
+	    /* make the No dups toggle and stats fields */
+
+	    n = 0;
+	    XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
+	    XtSetArg (args[n], XmNtopWidget, ucactf_w); n++;
 	    XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
-	    XtSetArg (args[n], XmNleftPosition, 40); n++;
-	    nodupt_w = XmCreateToggleButton (fsform_w, "NoDups", args, n);
-	    set_xmstring (nodupt_w,XmNlabelString,"Skip likely duplicates");
+	    XtSetArg (args[n], XmNleftPosition, 10); n++;
+	    XtSetArg (args[n], XmNorientation, XmHORIZONTAL); n++;
+	    rc_w = XmCreateRowColumn (fsform_w, "FSDRC", args, n);
+	    XtManageChild (rc_w);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNmarginHeight, 0); n++;
+	    nodupt_w = XmCreateToggleButton (rc_w, "NoDups", args, n);
+	    set_xmstring (nodupt_w, XmNlabelString, "Skip likely duplicates");
 	    wtip (nodupt_w,
-	      "When loading, skip objects which are very similar to ones already in memory");
+	      "When loading, skip objects similar to ones already in memory");
 	    XtManageChild (nodupt_w);
 	    sr_reg (nodupt_w, NULL, fscategory, 1);
 
+	    n = 0;
+	    XtSetArg (args[n], XmNmarginHeight, 0); n++;
+	    l_w = XmCreateLabel (rc_w, "FSDL1", args, n);
+	    set_xmstring (l_w, XmNlabelString, "within");
+	    XtManageChild (l_w);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNcolumns, 4); n++;
+	    nodupsep_w = XmCreateTextField (rc_w, "DupSep", args, n);
+	    fixTextCursor (nodupsep_w);
+	    wtip (nodupsep_w, "skip if closer than this");
+	    sr_reg (nodupsep_w, NULL, fscategory, 1);
+	    XtManageChild (nodupsep_w);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNmarginHeight, 0); n++;
+	    l_w = XmCreateLabel (rc_w, "FSDL1", args, n);
+	    set_xmstring (l_w, XmNlabelString, "arc secs and");
+	    XtManageChild (l_w);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNcolumns, 4); n++;
+	    nodupmag_w = XmCreateTextField (rc_w, "DupMag", args, n);
+	    fixTextCursor (nodupmag_w);
+	    sr_reg (nodupmag_w, NULL, fscategory, 1);
+	    wtip (nodupmag_w, "skip if brightness differs less than this");
+	    XtManageChild (nodupmag_w);
+
+	    n = 0;
+	    XtSetArg (args[n], XmNmarginHeight, 0); n++;
+	    l_w = XmCreateLabel (rc_w, "FSDL1", args, n);
+	    set_xmstring (l_w, XmNlabelString, "magnitudes");
+	    XtManageChild (l_w);
+
 	/* make the controls across the bottom under a separator */
 
 	n = 0;
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNtopWidget, nodupt_w); n++;
+	XtSetArg (args[n], XmNtopWidget, rc_w); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
 	w = XmCreateSeparator (fsform_w, "Sep", args, n);
@@ -778,11 +859,13 @@ fs_setup ()
 	if (ch_dn)
 	    XmTextFieldSetString (chtf_w, ch_dn);
 
-	XmToggleButtonSetState (nettb_w, net_on, False);
-	if (net_host)
-	    XmTextFieldSetString (nettf_w, net_host);
+	XmToggleButtonSetState (gsc23tb_w, gsc23_on, False);
+	if (gsc23_host)
+	    XmTextFieldSetString (gsc23tf_w, gsc23_host);
 
-	XmToggleButtonSetState (etb_w, e_on, False);
+	XmToggleButtonSetState (gsc22tb_w, gsc22_on, False);
+	if (gsc22_dn)
+	    XmTextFieldSetString (gsc22tf_w, gsc22_dn);
 
 	/* USNO */
 	XmToggleButtonSetState (notb_w, usno_on, False);
@@ -796,6 +879,10 @@ fs_setup ()
 	XmToggleButtonSetState (tyctb_w, tyc_on, False);
 	if (tyc_fn)
 	    XmTextFieldSetString (tyctf_w, tyc_fn);
+	XmToggleButtonSetState (ucactb_w, ucac_on, False);
+	if (ucac_dir)
+	    XmTextFieldSetString (ucactf_w, ucac_dir);
+
 	XmToggleButtonSetState (nodupt_w, nodup_on, False);
 }
 
@@ -823,61 +910,51 @@ fs_save ()
 	ch_dn = XmTextFieldGetString (chtf_w);
 	if ((cd_on || ch_on) && GSCSetup (cd_on?cd_dn:NULL, ch_on?ch_dn:NULL,
 								    msg) < 0) {
-	    xe_msg (msg, 1);
+	    xe_msg (1, "Cache: %s", msg);
 	    cd_on = 0;
 	    ch_on = 0;
 	    fs_setup ();
 	    allok = 0;
 	}
 
-	/* xephemdbd network connection */
-	net_on = XmToggleButtonGetState (nettb_w);
-	if (net_host)
-	    XtFree (net_host);
-	net_host = XmTextFieldGetString (nettf_w);
-	if (net_on) {
+	/* GSC 2.3 network connection */
+	gsc23_on = XmToggleButtonGetState (gsc23tb_w);
+	if (gsc23_host)
+	    XtFree (gsc23_host);
+	gsc23_host = XmTextFieldGetString (gsc23tf_w);
+	if (gsc23_on) {
 	    /* test by actually fetching a small patch */
 	    Now *np = mm_get_now();
-	    double fov = degrad(1);
-	    double mag = 13.0;
 	    int nop;
 	    ObjF *op;
 
 	    op = NULL;
-	    nop = gscnetfetch (net_host, np, 0.0, 0.0, fov, mag, &op, 0, msg);
+	    nop = gsc23fetch (gsc23_host, np, 1.0, 1.0, degrad(.01), 20.0, &op, 0, msg);
 	    if (nop <= 0) {
 		if (nop == 0)
-		    (void)strcpy(msg,
-		    		"xephemdbd connected but returned no objects");
-		xe_msg (msg, 1);
-		net_on = 0;
+		    xe_msg (1, "GSC 2.3 connected but returned no objects");
+		else
+		    xe_msg (1, "GSC 2.3 fetch: %s", msg);
+		gsc23_on = 0;
 		fs_setup ();
 		allok = 0;
 	    } else
 		free ((void *)op);
 	}
 
-	/* ESO connection */
-	e_on = XmToggleButtonGetState (etb_w);
-	if (e_on) {
-	    /* test by actually fetching a small patch */
-	    Now *np = mm_get_now();
-	    double fov = degrad(1);
-	    double mag = 13.0;
-	    int nop;
-	    ObjF *op;
-
-	    op = NULL;
-	    nop = gscesofetch (np, 0.0, 0.0, fov, mag, &op, 0, msg);
-	    if (nop <= 0) {
-		if (nop == 0)
-		    (void) strcpy(msg,"ESO Connected but returned no objects");
-		xe_msg (msg, 1);
-		e_on = 0;
+	/* GSC22 */
+	gsc22_on = XmToggleButtonGetState (gsc22tb_w);
+	if (gsc22_dn)
+	    XtFree (gsc22_dn);
+	gsc22_dn = XmTextFieldGetString (gsc22tf_w);
+	if (gsc22_on) {
+	    /* test directory */
+	    if (xe3chkdir (gsc22_dn, msg) < 0) {
+		xe_msg (1, msg);
+		gsc22_on = 0;
 		fs_setup ();
 		allok = 0;
-	    } else if (nop > 0)
-		free ((void *)op);
+	    }
 	}
 
 	/* USNO */
@@ -886,7 +963,7 @@ fs_save ()
 	    XtFree (usno_fn);
 	usno_fn = XmTextFieldGetString (notf_w);
 	if (usno_on && USNOSetup (usno_fn, 1, msg) < 0) {
-	    xe_msg (msg, 1);
+	    xe_msg (1, "USNO: %s", msg);
 	    usno_on = 0;
 	    fs_setup();
 	    allok = 0;
@@ -898,7 +975,7 @@ fs_save ()
 	    XtFree (ppm_fn);
 	ppm_fn = XmTextFieldGetString (ppmtf_w);
 	if (ppm_on && xe2chkfile (ppm_fn, msg) < 0) {
-	    xe_msg (msg, 1);
+	    xe_msg (1, "PPM: %s", msg);
 	    ppm_on = 0;
 	    fs_setup();
 	    allok = 0;
@@ -910,12 +987,24 @@ fs_save ()
 	    XtFree (tyc_fn);
 	tyc_fn = XmTextFieldGetString (tyctf_w);
 	if (tyc_on && xe2chkfile (tyc_fn, msg) < 0) {
-	    xe_msg (msg, 1);
+	    xe_msg (1, "Tycho: %s", msg);
 	    tyc_on = 0;
 	    fs_setup();
 	    allok = 0;
 	}
 
+	/* UCAC */
+	ucac_on = XmToggleButtonGetState (ucactb_w);
+	if (ucac_dir)
+	    XtFree (ucac_dir);
+	ucac_dir = XmTextFieldGetString (ucactf_w);
+	if (ucac_on && UCACSetup (ucac_dir, msg) < 0) {
+	    xe_msg (1, "UCAC: %s", msg);
+	    ucac_on = 0;
+	    fs_setup();
+	    allok = 0;
+	}
+
 	/* options */
 	nodup_on = XmToggleButtonGetState (nodupt_w);
 
@@ -927,10 +1016,7 @@ fs_save ()
 /* called from Apply */
 /* ARGSUSED */
 static void
-apply_cb (w, client, call)
-Widget w;
-XtPointer client;
-XtPointer call;
+apply_cb (Widget w, XtPointer client, XtPointer call)
 {
 	if (fs_save() == 0)
 	    sv_loadfs (1);
@@ -939,10 +1025,7 @@ XtPointer call;
 /* called from Ok */
 /* ARGSUSED */
 static void
-ok_cb (w, client, call)
-Widget w;
-XtPointer client;
-XtPointer call;
+ok_cb (Widget w, XtPointer client, XtPointer call)
 {
 	if (fs_save() == 0) {
 	    sv_loadfs (1);
@@ -953,74 +1036,73 @@ XtPointer call;
 /* called from Ok */
 /* ARGSUSED */
 static void
-cancel_cb (w, client, call)
-Widget w;
-XtPointer client;
-XtPointer call;
+cancel_cb (Widget w, XtPointer client, XtPointer call)
 {
 	/* outta here */
 	XtPopdown (fsshell_w);
 }
 
-/* called from any of the CDROM, network or Cache toggle buttons.
+/* called from any of the GSC CDROM, network or Cache toggle buttons.
  * client is one of the TB enums to tell us which.
  */
 /* ARGSUSED */
 static void
-gsctb_cb (w, client, call)
-Widget w;
-XtPointer client;
-XtPointer call;
+gsctb_cb (Widget w, XtPointer client, XtPointer call)
 {
 	if (XmToggleButtonGetState(w))
-	    switch ((int)client) {
+	    switch ((long int)client) {
 	    case CDROMTB:	/* FALLTHRU */
 	    case CACHETB:
-		/* turn off net and ESO if turning on CDROM or Cache */
-		XmToggleButtonSetState (nettb_w, False, False);
-		XmToggleButtonSetState (etb_w, False, False);
+		/* turn off net and gsc22 if turning on CDROM or Cache */
+		XmToggleButtonSetState (gsc23tb_w, False, False);
+		XmToggleButtonSetState (gsc22tb_w, False, False);
 		break;
 	    case NETTB:
-		/* turn off CDROM and Cache and ESO if turning on network */
+		/* turn off CDROM and Cache and gsc22 if turning on network */
 		XmToggleButtonSetState (cdtb_w, False, False);
 		XmToggleButtonSetState (chtb_w, False, False);
-		XmToggleButtonSetState (etb_w, False, False);
+		XmToggleButtonSetState (gsc22tb_w, False, False);
+		/* turn on at least one PM catalog */
+		if (!XmToggleButtonGetState(ppmtb_w)
+				    && !XmToggleButtonGetState(tyctb_w) 
+				    && !XmToggleButtonGetState(ucactb_w))
+		    XmToggleButtonSetState (tyctb_w, True, False);
 		break;
-	    case ESOTB:
-		/* turn off CDROM and Cache and net if turning on ESO */
+	    case GSC22TB:
+		/* turn off CDROM and Cache and net if turning on GSC22 */
 		XmToggleButtonSetState (cdtb_w, False, False);
 		XmToggleButtonSetState (chtb_w, False, False);
-		XmToggleButtonSetState (nettb_w, False, False);
+		XmToggleButtonSetState (gsc23tb_w, False, False);
+		/* turn on at least one PM catalog */
+		if (!XmToggleButtonGetState(ppmtb_w)
+				    && !XmToggleButtonGetState(tyctb_w) 
+				    && !XmToggleButtonGetState(ucactb_w))
+		    XmToggleButtonSetState (tyctb_w, True, False);
 		break;
 	    default:
-		printf ("FS: bad client: %d\n", (int)client);
-		exit(1);
+		printf ("FS: bad client: %d\n", (int)(long int)client);
+		abort();
 	    }
 }
 
-/* used to maintain 1-of-2 ppm vs. tycho databases.
- * client is the other widget.
+/* used to maintain 1-of-3 ppm vs tycho vs ucac databases.
+ * client coming on turns off the other 2.
  */
 /* ARGSUSED */
 static void
-pmtb_cb (w, client, call)
-Widget w;
-XtPointer client;
-XtPointer call;
+pmtb_cb (Widget w, XtPointer client, XtPointer call)
 {
-	Widget otherw = (Widget)client;
-
-	if (XmToggleButtonGetState(w))
-	    XmToggleButtonSetState (otherw, False, False);
+	if (XmToggleButtonGetState(w)) {
+	    if (w != ppmtb_w) XmToggleButtonSetState (ppmtb_w, False, False);
+	    if (w != tyctb_w) XmToggleButtonSetState (tyctb_w, False, False);
+	    if (w != ucactb_w) XmToggleButtonSetState (ucactb_w, False, False);
+	}
 }
 
 /* called from Ok */
 /* ARGSUSED */
 static void
-help_cb (w, client, call)
-Widget w;
-XtPointer client;
-XtPointer call;
+help_cb (Widget w, XtPointer client, XtPointer call)
 {
         static char *msg[] = {
 "Set up which field star sources to use and where they are located."
@@ -1031,4 +1113,4 @@ XtPointer call;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: fsmenu.c,v $ $Date: 2001/10/04 20:30:32 $ $Revision: 1.16 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: fsmenu.c,v $ $Date: 2009/01/05 20:55:54 $ $Revision: 1.33 $ $Name:  $"};
