@@ -240,7 +240,7 @@ or enter any time step as follows:\n\
 	"Precession epoch: EOD for full apparent, fixed year for astrometric",
 	"Precession Epoch (decimal year): ",		"Equinox:", NULL},
     {"MagDec", MAG_FID, 0,
-	"Magnetic bearing = true az + mag declination.",
+	"True az = magnetic az + magnetic declination.",
 	NULL, "Mag decl:", "MagDec"},
 };
 
@@ -2731,4 +2731,4 @@ static char *hlp[] = {
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: mainmenu.c,v $ $Date: 2009/01/05 20:55:54 $ $Revision: 1.90 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: mainmenu.c,v $ $Date: 2014/07/15 02:11:33 $ $Revision: 1.91 $ $Name:  $"};
