@@ -915,7 +915,7 @@ ol_writeLB()
 	fprintf (fp, "    </%s>\n", lbetag);
 
 	/* finished */
-	fprintf (fp, cltag);
+	fprintf (fp, "%s", cltag);
 	xe_msg (1, "Added new log entry");
 	fclose (fp);
 }
@@ -1245,4 +1245,4 @@ oll_info_eh (Widget w, XtPointer client, XEvent *ev, Boolean *c_to_dispatch)
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: obslog.c,v $ $Date: 2006/08/21 20:04:30 $ $Revision: 1.23 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: obslog.c,v $ $Date: 2015/08/09 21:47:00 $ $Revision: 1.24 $ $Name:  $"};
