@@ -18,6 +18,10 @@
 
 #include "xephem.h"
 
+#define	TL	5	/* tick mark length, in pixels */
+#define	RB	30	/* right border */
+#define	NT	10	/* rough number of tick marks on each axis */
+
 static void mk_gcs (Widget w);
 static double binsplit (int i);
 static int x_ticks (int asdate, double minx, double maxx, int maxticks,
@@ -29,24 +33,17 @@ static GC plt_gc;		/* the GC to use for everything */
 static Pixel plt_p[MAXPLTLINES];/* colors, one per category */
 static XFontStruct *plt_fs;	/* handy font metrix for placing text */
 static Pixel da_p;		/* the foreground color of the drawing area */
+static Pixel dabg_p;		/* the background color of the drawing area */
+static int top_border;		/* space for coords values */
 
-/* plot the given file in the drawing area in cartesian coords.
- * drawing space is from 0..(nx-1) and 0..(ny-1).
+/* plot based on the given info in the drawing area in cartesian coords.
+ * update di with graph boundaries.
  * TODO: add z tags somehow
  * return 0 if ok, else give a message about trouble and return -1.
  */
 int
-plot_cartesian (
-FILE *pfp,
-Widget widget,
-unsigned int nx, unsigned int ny,
-int flipx, int flipy,	/* respectively true to flip their axis */
-int grid,		/* whether to include a grid with the tick marks */
-int xasdate)		/* whether to show x axis as dates */
+plot_cartesian (DrawInfo *di, Widget widget, Dimension nx, Dimension ny)
 {
-#define	TL	5	/* tick mark length, in pixels */
-#define	RB	30	/* right border */
-#define	NT	10	/* rough number of tick marks on each axis */
 	Display *dsp = XtDisplay(widget);
 	Window win = XtWindow(widget);
 	double xticks[NT+2];
@@ -69,14 +66,16 @@ int xasdate)		/* whether to show x axis as dates */
 	int asc, desc;	/* font ascent and descent, in pixels */
 	int maxlblw;	/* width of longest y axis label, in pixels */
 	int i;
-#define	XCORD(x)	(x0 + (int)((flipx?maxx-(x):(x)-minx)*xscale + 0.5))
-#define	YCORD(y)	(y0 - (int)((flipy?maxy-(y):(y)-miny)*yscale + 0.5))
+#define	XCORD(x)	(x0 + (int)((di->flipx?maxx-(x):(x)-minx)*xscale + 0.5))
+#define	YCORD(y)	(y0 - (int)((di->flipy?maxy-(y):(y)-miny)*yscale + 0.5))
 
 	/* find ranges and number of and tags for each unique line */
 	nlines = 0;
-	while (fgets (buf, sizeof(buf), pfp)) {
+	while (fgets (buf, sizeof(buf), di->fp)) {
 	    if (sscanf (buf, fmt, tag, &x, &y) != 3)
 		continue;
+	    if (di->xjd_asdate)
+		mjd_year (x-MJD0, &x);	/* work in years */
 	    if (nlines == 0) {
 		maxx = minx = x;
 		maxy = miny = y;
@@ -115,12 +114,12 @@ int xasdate)		/* whether to show x axis as dates */
 	XSetFont (dsp, plt_gc, plt_fs->fid);
 
 	/* decide tickmarks */
-	nxt = x_ticks (xasdate, minx, maxx, NT, xticks);
-	minx = xticks[0];
-	maxx = xticks[nxt-1];
+	nxt = x_ticks (di->xjd_asdate || di->xyr_asdate, minx, maxx, NT, xticks);
+	di->data_minx = minx = xticks[0];
+	di->data_maxx = maxx = xticks[nxt-1];
 	nyt = tickmarks (miny, maxy, NT, yticks);
-	miny = yticks[0];
-	maxy = yticks[nyt-1];
+	di->data_miny = miny = yticks[0];
+	di->data_maxy = maxy = yticks[nyt-1];
 
 	/* compute length of longest y-axis label and other char stuff.
 	 */
@@ -132,13 +131,17 @@ int xasdate)		/* whether to show x axis as dates */
 	    XTextExtents (plt_fs, buf, l, &dir, &asc, &desc, &overall);
 	    if (overall.width > maxlblw)
 		maxlblw = overall.width;
+	    top_border = 2*(asc+desc) + 5;
 	}
 
 	/* compute border sizes and the scaling factors */
-	x0 = maxlblw+TL+10;
+	di->win_minx = x0 = maxlblw+TL+10;
 	w = nx - x0 - RB;
+	di->win_maxx = di->win_minx + w;
 	y0 = ny - (asc+desc+2+2*TL);
-	h = y0 - (asc+desc+2);
+	h = y0 - top_border;
+	di->win_miny = y0 - h;
+	di->win_maxy = di->win_miny + h;
 	xscale = w/(maxx-minx);
 	yscale = h/(maxy-miny);
 
@@ -150,7 +153,7 @@ int xasdate)		/* whether to show x axis as dates */
 	    iy = YCORD(yticks[i]);
 	    XPSDrawLine (dsp, win, plt_gc, x0-TL, iy, x0, iy);
 	    XPSDrawString (dsp, win, plt_gc, 1, iy+(asc-desc)/2, buf, l);
-	    if (grid)
+	    if (di->grid)
 		XPSDrawLine (dsp, win, plt_gc, x0, iy, x0+w-1, iy);
 	}
 
@@ -161,14 +164,13 @@ int xasdate)		/* whether to show x axis as dates */
 	ylblw = 0;
 	for (i = 0; i < nxt; i++) {
 	    ix = XCORD(xticks[i]);
-	    XPSDrawLine (dsp, win, plt_gc, ix, y0+TL, ix, y0);
-	    if (grid)
-		XPSDrawLine (dsp, win, plt_gc, ix, y0, ix, y0-h-1);
+	    if (di->grid)
+		XPSDrawLine (dsp, win, plt_gc, ix, y0, ix, y0-h);
 	}
-	ylblw += draw_x_label (dsp, win, xasdate, minx, XCORD(minx),y0,w+x0+RB);
-	ylblw += draw_x_label (dsp, win, xasdate, maxx, XCORD(maxx),y0,w+x0+RB);
+	ylblw += draw_x_label (dsp, win, di->xjd_asdate || di->xyr_asdate, minx, XCORD(minx),y0,w+x0+RB);
+	ylblw += draw_x_label (dsp, win, di->xjd_asdate || di->xyr_asdate, maxx, XCORD(maxx),y0,w+x0+RB);
 	if (ylblw < w/2)
-	    (void) draw_x_label (dsp, win, xasdate, xticks[nxt/2],
+	    (void) draw_x_label (dsp, win, di->xjd_asdate || di->xyr_asdate, xticks[nxt/2],
 					    XCORD(xticks[nxt/2]), y0, w+x0+RB);
 
 	/* draw border of actual plotting area */
@@ -182,10 +184,10 @@ int xasdate)		/* whether to show x axis as dates */
 	 * is put up as a title line (minus its first two char and trailing \n).
 	 */
 	sawtitle = 0;
-	rewind (pfp);
+	rewind (di->fp);
 	for (i = 0; i < nlines; i++)
 	    one[i] = 0;
-	while (fgets (buf, sizeof(buf), pfp)) {
+	while (fgets (buf, sizeof(buf), di->fp)) {
 	    if (sscanf (buf, fmt, tag, &x, &y) != 3) {
 		/* a title line ? */
 		int l;
@@ -201,6 +203,8 @@ int xasdate)		/* whether to show x axis as dates */
 		}
 		continue;
 	    }
+	    if (di->xjd_asdate)
+		mjd_year (x-MJD0, &x);	/* work in years */
 	    for (i = 0; i < nlines; i++)
 		if (strcmp(tag, tags[i]) == 0)
 		    break;
@@ -220,20 +224,60 @@ int xasdate)		/* whether to show x axis as dates */
 	return (0);
 }
 
+/* given a cursor location in the plot described by di, draw the 
+ * corresponding data values
+ */
+void
+plot_coords (Widget da_w, DrawInfo *di, int window_x, int window_y)
+{
+        Display *dsp = XtDisplay(da_w);
+	Window win = XtWindow(da_w);
+	int graphx, graphy;
+	int wid, hei;
+	double datax, datay;
+	char buf[1024];
+	int bufl;
+
+	/* interpolate data */
+	if (window_x < di->win_minx || window_x > di->win_maxx || window_y < di->win_miny
+		|| window_y > di->win_maxy)
+	    return;
+	wid = di->win_maxx - di->win_minx;
+	hei = di->win_maxy - di->win_miny;
+	graphx = window_x - di->win_minx;
+	if (di->flipx)
+	    graphx = wid - graphx;
+	graphy = window_y - di->win_miny;
+	if (di->flipy)
+	    graphy = hei - graphy;
+	datax = di->data_minx + graphx*(di->data_maxx - di->data_minx)/wid;
+	datay = di->data_maxy - graphy*(di->data_maxy - di->data_miny)/hei; /* window 0 is on top */
+
+	/* erase */
+	XSetForeground (dsp, plt_gc, dabg_p);
+	XFillRectangle (dsp, win, plt_gc, 30, top_border/2, 200, top_border/2);
+
+	/* draw */
+	XSetForeground (dsp, plt_gc, da_p);
+	bufl = sprintf (buf, "%.6g", datax);
+	XDrawString (dsp, win, plt_gc, 30, top_border-2, buf, bufl);
+	bufl = sprintf (buf, "%.6g", datay);
+	XDrawString (dsp, win, plt_gc, 120, top_border-2, buf, bufl);
+}
+
 static void
 mk_gcs (w)
 Widget w;
 {
 	Display *dsp = XtDisplay(w);
 	Window win = XtWindow(w);
-	Pixel da_bgp;	/* the background color of the drawing area */
 	int i;
 
 	/* create the annotation and default plot color gc,
 	 * using the foreground color of the PlotDA.
 	 */
 	get_something (w, XmNforeground, (XtArgVal)&da_p);
-	get_something (w, XmNbackground, (XtArgVal)&da_bgp);
+	get_something (w, XmNbackground, (XtArgVal)&dabg_p);
 	plt_gc = XCreateGC (dsp, win, 0L, NULL);
 	XSetForeground (dsp, plt_gc, da_p);
 	get_views_font (dsp, &plt_fs);
@@ -292,7 +336,7 @@ double ticks[];
 	if (!asdate)
 	    return (tickmarks (minx, maxx, maxticks, ticks));
 
-	/* find spanning period in whole months */
+	/* find spanning period in mjds */
 	year_mjd (minx, &minjd);
 	mjd_cal (minjd, &m0, &d0, &y0);
 	year_mjd (maxx, &maxjd);
@@ -327,7 +371,7 @@ double ticks[];
 	    nd = (int)(maxjd - minjd + 1);	/* period */
 	    dt = nd/maxticks + 1;		/* round up */
 	    nt = (nd+(dt-1))/dt + 1;		/* n ticks */
-	    jd = mjd_day (minjd);
+	    jd = mjd_day (minjd);		/* whole day */
 	    for (i = 0; i < nt; i++) {
 		mjd_year (jd, &ticks[i]);
 		jd += dt;
@@ -359,7 +403,7 @@ int w;
 
 	    year_mjd (v, &jd);
 	    fs_date (buf, pref_get(PREF_DATE_FORMAT), jd);
-	} else
+	} else 
 	    (void) sprintf (buf, "%g", v);
 	l = strlen(buf);
 	XTextExtents (plt_fs, buf, l, &dir, &asc, &des, &ovl);
@@ -377,4 +421,4 @@ int w;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: plot_aux.c,v $ $Date: 2006/04/10 09:00:06 $ $Revision: 1.11 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: plot_aux.c,v $ $Date: 2012/12/30 17:01:02 $ $Revision: 1.12 $ $Name:  $"};
