@@ -57,8 +57,10 @@ static void plt_da_unmap_cb (Widget w, XtPointer client, XtPointer call);
 static void plt_da_flipx_cb (Widget w, XtPointer client, XtPointer call);
 static void plt_da_flipy_cb (Widget w, XtPointer client, XtPointer call);
 static void plt_da_grid_cb (Widget w, XtPointer client, XtPointer call);
-static void plt_da_xasdate_cb (Widget w, XtPointer client, XtPointer call);
+static void plt_da_xyr_asdate_cb (Widget w, XtPointer client, XtPointer call);
+static void plt_da_xjd_asdate_cb (Widget w, XtPointer client, XtPointer call);
 static void plt_da_exp_cb (Widget da_w, XtPointer client, XtPointer call);
+static void plt_da_motion_eh (Widget da_w, XtPointer client, XEvent *ev, Boolean *continue_to_dispatch);
 static void plt_drawall (Widget da_w);
 static int plt_ano (double *sx, double *sy, int *xp, int *yp, int w2x,int arg);
 
@@ -105,22 +107,6 @@ typedef struct {
 static PltLine pltlines[MAXPLTLINES];
 static int npltlines;		/* number of completely defined plots */
 
-/* one of these gets malloced and passed to the drawing area expose callback via
- * its client parameter. be sure to free it when the parent FormDialog goes
- * away too.
- * by doing this, we can have lots of different plots up at once and yet we
- * don't have to keep track of them - they clean up after themselves.
- */
-typedef struct {
-    char *filename;	/* name of file being plotted (also malloced) */
-    FILE *fp;		/* FILE pointer for the file */
-    int flipx, flipy;	/* flip state for this instance */
-    Widget fx_w, fy_w;	/* widgets for sr_unreg() */
-    int grid;		/* whether to include a grid */
-    int xasdate;	/* whether to show x axis as dates */
-    Widget as_w, g_w;	/* widgets for sr_unreg() */
-} DrawInfo;
-
 /* called when the plot menu is activated via the main menu pulldown.
  * if never called before, create and manage all the widgets as a child of a
  * form. otherwise, just get going.
@@ -929,6 +915,7 @@ plt_da_manage(char *fn)
 	Widget da_w, w;
 	Widget mb_w, pd_w, cb_w;
 	XmString str;
+	EventMask mask;
 	Arg args[20];
 	char titlebuf[64];
 	int n;
@@ -995,6 +982,8 @@ plt_da_manage(char *fn)
 	XtSetArg (args[n], XmNmarginHeight, 0); n++;
 	da_w = XmCreateDrawingArea (daform_w, "PlotMap", args, n);
 	XtAddCallback (da_w, XmNexposeCallback, plt_da_exp_cb, NULL);
+	mask = Button1MotionMask | PointerMotionMask | PointerMotionHintMask;
+	XtAddEventHandler (da_w, mask, False, plt_da_motion_eh, 0);
 	XtManageChild (da_w);
 
 	/* when unmap the form, pass the da to the unmap callback */
@@ -1109,23 +1098,40 @@ plt_da_manage(char *fn)
 	     * sr_reg (w, NULL, plotcategory, 1);
 	     */
 
-	    str = XmStringCreate("Show X-Axis as Dates",
+	    str = XmStringCreate("Show X-Axis Years as Dates",
 						    XmSTRING_DEFAULT_CHARSET);
 	    n = 0;
 	    XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
 	    XtSetArg (args[n], XmNmarginHeight, 0); n++;
 	    XtSetArg (args[n], XmNlabelString, str); n++;
-	    w = XmCreateToggleButton(pd_w, "XAsDate", args, n);
-	    di->xasdate = XmToggleButtonGetState (w);
-	    di->as_w = w;
+	    w = XmCreateToggleButton(pd_w, "XYrAsDate", args, n);
+	    di->xyr_asdate = XmToggleButtonGetState (w);
+	    di->yas_w = w;
 	    XmStringFree (str);
-	    XtAddCallback (w, XmNvalueChangedCallback, plt_da_xasdate_cb, da_w);
+	    XtAddCallback (w, XmNvalueChangedCallback, plt_da_xyr_asdate_cb, da_w);
 	    wtip (w, "Assume ordinate values are decimal years and display as dates");
 	    XtManageChild (w);
 	    /* TODO - can not use until fix findWFB()
 	     * sr_reg (w, NULL, plotcategory, 1);
 	     */
 
+	    str = XmStringCreate("Show X-Axis JDs as Dates",
+						    XmSTRING_DEFAULT_CHARSET);
+	    n = 0;
+	    XtSetArg (args[n], XmNvisibleWhenOff, True); n++;
+	    XtSetArg (args[n], XmNmarginHeight, 0); n++;
+	    XtSetArg (args[n], XmNlabelString, str); n++;
+	    w = XmCreateToggleButton(pd_w, "XJDAsDate", args, n);
+	    di->xjd_asdate = XmToggleButtonGetState (w);
+	    di->jas_w = w;
+	    XmStringFree (str);
+	    XtAddCallback (w, XmNvalueChangedCallback, plt_da_xjd_asdate_cb, da_w);
+	    wtip (w, "Assume ordinate values are Julian dates and display as dates");
+	    XtManageChild (w);
+	    /* TODO - can not use until fix findWFB()
+	     * sr_reg (w, NULL, plotcategory, 1);
+	     */
+
 	/* go. the expose will do the actual plotting */
 	XtManageChild (daform_w);
 }
@@ -1371,13 +1377,13 @@ XtPointer call;
 	XSendEvent (dsp, win, /*propagate*/False, ExposureMask, (XEvent *)&ev);
 }
 
-/* callback from the X-is-date toggle button within the drawing FormDiag itself.
+/* callback from the X-as-Year-is-date toggle button within the drawing FormDiag itself.
  * toggle the xisdate flag in the parent's DrawInfo structure and fake an
  * expose. client is the DrawingArea widget.
  */
 /* ARGSUSED */
 static void
-plt_da_xasdate_cb (w, client, call)
+plt_da_xyr_asdate_cb (w, client, call)
 Widget w;
 XtPointer client;
 XtPointer call;
@@ -1393,8 +1399,61 @@ XtPointer call;
 	int x, y;
 	unsigned int nx, ny, bw, d;
 
+	/* honor this, and toggle off the other if on */
 	get_something (daform_w, XmNuserData, (XtArgVal)&di);
-	di->xasdate = t->set;
+	di->xyr_asdate = t->set;
+	if (!di->xyr_asdate && di->xjd_asdate)
+	    return;	/* we were toggled off */
+	if (di->xyr_asdate && di->xjd_asdate) {
+	    di->xjd_asdate = 0;
+	    XmToggleButtonSetState (di->jas_w, False, True);
+	}
+
+	XGetGeometry(dsp, win, &root, &x, &y, &nx, &ny, &bw, &d);
+
+	ev.type = Expose;
+	ev.send_event = 1;	/* gets set anyways */
+	ev.display = dsp;
+	ev.window = win;
+	ev.x = ev.y = 0;
+	ev.width = nx;
+	ev.height = ny;
+	ev.count = 0;
+
+	XSendEvent (dsp, win, /*propagate*/False, ExposureMask, (XEvent *)&ev);
+}
+
+/* callback from the X-as-JD-is-date toggle button within the drawing FormDiag itself.
+ * toggle the xisdate flag in the parent's DrawInfo structure and fake an
+ * expose. client is the DrawingArea widget.
+ */
+/* ARGSUSED */
+static void
+plt_da_xjd_asdate_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	XmToggleButtonCallbackStruct *t = (XmToggleButtonCallbackStruct *) call;
+	Widget da_w = (Widget) client;
+	Widget daform_w = XtParent(da_w);
+	Display *dsp = XtDisplay(da_w);
+	Window win = XtWindow(da_w);
+	DrawInfo *di;
+	XExposeEvent ev;
+	Window root;
+	int x, y;
+	unsigned int nx, ny, bw, d;
+
+	/* honor this, and toggle off the other if on */
+	get_something (daform_w, XmNuserData, (XtArgVal)&di);
+	di->xjd_asdate = t->set;
+	if (!di->xjd_asdate && di->xyr_asdate)
+	    return;	/* we were toggled off */
+	if (di->xjd_asdate && di->xyr_asdate) {
+	    di->xyr_asdate = 0;
+	    XmToggleButtonSetState (di->yas_w, False, True);
+	}
 
 	XGetGeometry(dsp, win, &root, &x, &y, &nx, &ny, &bw, &d);
 
@@ -1451,6 +1510,35 @@ XtPointer call;
 	plt_drawall (da_w);
 }
 
+/* plot drawing area's motion handler.
+ * called when pointer moves over a plot.
+ * get a DrawInfo from our parent's userData.
+ * report data at that location
+ */
+static void
+plt_da_motion_eh (da_w, client, ev, continue_to_dispatch)
+Widget da_w;
+XtPointer client;
+XEvent *ev;
+Boolean *continue_to_dispatch;
+{
+	Display *dsp = XtDisplay(da_w);
+	Window win = XtWindow(da_w);
+	Window root, child;
+	int rx, ry, wx, wy;
+	unsigned mask;
+	Widget daform_w;
+	DrawInfo *di;
+
+	daform_w = XtParent(da_w);
+	get_something (daform_w, XmNuserData, (XtArgVal)&di);
+
+	XQueryPointer (dsp, win, &root, &child, &rx, &ry, &wx, &wy, &mask);
+
+	plot_coords (da_w, di, wx, wy);
+}
+
+
 /* given a drawing area, draw the plot.
  * remember: the parent of the da is the form and it's userData is the di info.
  */
@@ -1471,8 +1559,7 @@ Widget da_w;
 	get_something (daform_w, XmNuserData, (XtArgVal)&di);
 	XClearWindow (XtDisplay(da_w), XtWindow(da_w));
 	rewind (di->fp);
-	if (plot_cartesian (di->fp, da_w, w, h, di->flipx, di->flipy, di->grid,
-							    di->xasdate) < 0) {
+	if (plot_cartesian (di, da_w, w, h) < 0) {
 	    /* had trouble, so done with this FormDialog.
 	     */
 	    xe_msg (0, "Error plotting `%s'", di->filename);
@@ -1502,4 +1589,4 @@ plt_ano (double *sx, double *sy, int *xp, int *yp, int w2x, int arg)
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: plotmenu.c,v $ $Date: 2005/11/14 03:55:44 $ $Revision: 1.37 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: plotmenu.c,v $ $Date: 2012/12/30 17:01:02 $ $Revision: 1.38 $ $Name:  $"};
