@@ -149,12 +149,6 @@ Obj *op;
 	    abort();
 	}
 
-	/* check whether above horizon */
-	if (op->s_alt < 0) {
-	    xe_msg (1, "%s is below the horizon", op->o_name);
-	    return;
-	}
-
 	/* send as edb if on */
 	if (XmToggleButtonGetState (edbon_w)) {
 	    char *dev, *prop, *ele;
@@ -871,4 +865,4 @@ fromINDIValue (double indiv, Widget m_w, Widget b_w)
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: scope.c,v $ $Date: 2007/07/26 21:03:08 $ $Revision: 1.27 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: scope.c,v $ $Date: 2011/09/23 01:52:39 $ $Revision: 1.28 $ $Name:  $"};
