@@ -41,7 +41,6 @@ typedef struct {
     int inuse;			/* whether this entry is in use */
     char name[MAXEPNM];		/* use's name */
     double w, h;		/* size, rads */
-    double pa;			/* position angle, rads E of N */
     int isE;			/* whether Elliptical or Rectangular */
     int isS;			/* whether Solid or Outline */
     Widget row_w;		/* widget with GUI controls for this entry */
@@ -53,17 +52,17 @@ static Widget favrc_w;		/* RC for listing favorites */
 static Widget saved_w;		/* TF for name of file to save definitions */
 static Widget savep_w;		/* TF for name of file to save positions */
 
-static char defeyepfn[] = "eyepieces.eyp";	/* default defs file name */
-static char eyep_suffix[] = ".eyp";		/* def file name suffix */
-static char defeyelfn[] = "eyepieces.epl";	/* default loc file name */
-static char eyel_suffix[] = ".epl";		/* loc file name suffix */
+static char defeyepfn[] = "eyepieces.epd";	/* default defs file name */
+static char eyep_suffix[] = ".epd";		/* def file name suffix */
+static char defeyelfn[] = "eyepieces.epp";	/* default positions file name*/
+static char eyel_suffix[] = ".epp";		/* positions file name suffix */
 
 static Widget eyep_w;		/* overall eyepiece dialog */
 static Widget eyepws_w;		/* eyepiece width scale */
 static Widget eyephs_w;		/* eyepiece height scale */
 static Widget eyepwl_w;		/* eyepiece width label */
-static Widget eyeppa_w;		/* eyepiece position angle, scale */
-static Widget eyeppl_w;		/* eyepiece position angle label */
+static Widget eyepa_w;		/* eyepiece rotation angle, scale */
+static Widget eyepal_w;		/* eyepiece rotation angle label */
 static Widget eyephl_w;		/* eyepiece height label */
 static Widget eyer_w;		/* eyepiece Round TB */
 static Widget eyes_w;		/* eyepiece Square TB */
@@ -109,7 +108,6 @@ static EyePiece *se_addeyep (void);
 static double se_getScale (Widget w);
 static void se_setScale (Widget w, double a);
 static double se_getPScale (Widget w);
-static void se_setPScale (Widget w, double a);
 static void se_addFavEyeP (FavEyeP *newfp);
 static void se_loadEyep (char *fn);
 static void se_saveFav (char *fn);
@@ -154,8 +152,7 @@ Cursor c;
 
 /* add one eyepiece with the current definition for the given location */
 void
-se_add (ra, dec, alt, az)
-double ra, dec, alt, az;
+se_add (int aamode, double azra, double altdec)
 {
 	int telrad;
 	int nnew;
@@ -171,10 +168,9 @@ double ra, dec, alt, az;
 	/* fill in the details */
 	while (--nnew >= 0) {
 	    EyePiece *new = se_addeyep();
-	    new->ra = ra;
-	    new->dec = dec;
-	    new->alt = alt;
-	    new->az = az;
+	    new->azra = azra;
+	    new->altdec = altdec;
+	    new->aamode = aamode;
 	    if (telrad) {
 		new->eyepw = degrad(telrad_sz[nnew]);
 		new->eyeph = degrad(telrad_sz[nnew]);
@@ -190,10 +186,10 @@ double ra, dec, alt, az;
 }
 
 /* return whether there are any eyepieces that cover the given location,
- * isradec determining the interpretation of the coords.
+ * aamode determining the interpretation of the coords.
  */
 int
-se_isOneHere (double azra, double altdec, int isradec)
+se_isOneHere (int aamode, double azra, double altdec)
 {
 	double caltdec = cos(altdec);
 	double saltdec = sin(altdec);
@@ -203,13 +199,13 @@ se_isOneHere (double azra, double altdec, int isradec)
 	    EyePiece *ep = &eyep[i];
 	    double L, l, csep, maxr;
 
-	    if (isradec) {
-		L = ep->ra;
-		l = ep->dec;
+	    if (aamode == ep->aamode) {
+		L = ep->azra;
+		l = ep->altdec;
 	    } else {
-		L = ep->az;
-		l = ep->alt;
+		sv_other (ep->altdec, ep->azra, ep->aamode, &l, &L);
 	    }
+
 	    solve_sphere (azra-L, PI/2-l, saltdec, caltdec, &csep, NULL);
 	    maxr = (ep->eyepw < ep->eyeph ? ep->eyepw : ep->eyeph)/2;
 	    if (acos(csep) < maxr)
@@ -221,10 +217,10 @@ se_isOneHere (double azra, double altdec, int isradec)
 }
 
 /* delete eyepiece that most closely covers the given location.
- * isradec determines how to interpret the coords.
+ * aamode determines how to interpret the coords.
  */
 void
-se_del (double azra, double altdec, int isradec)
+se_del (int aamode, double azra, double altdec)
 {
 	double caltdec, saltdec;
 	double r, maxcsep;
@@ -239,13 +235,13 @@ se_del (double azra, double altdec, int isradec)
 	for (ep = eyep; ep < endep; ep++) {
 	    double L, l, csep;
 
-	    if (isradec) {
-		L = ep->ra;
-		l = ep->dec;
+	    if (aamode == ep->aamode) {
+		L = ep->azra;
+		l = ep->altdec;
 	    } else {
-		L = ep->az;
-		l = ep->alt;
+		sv_other (ep->altdec, ep->azra, ep->aamode, &l, &L);
 	    }
+
 	    solve_sphere (azra-L, PI/2-l, saltdec, caltdec, &csep, NULL);
 	    if (csep > maxcsep) {
 		maxcsep = csep;
@@ -282,21 +278,18 @@ se_getlist (EyePiece **ep)
 	return (neyep);
 }
 
-/* fetch the current eyepiece diameter, in rads, pos angle, rads E of N,
+/* fetch the current eyepiece size and rotation angle, all in rads,
  * whether it is round, and whether it is filled from the dialog.
  */
 static void
-se_eyepsz(wp, hp, pp, rp, fp)
-double *wp, *hp, *pp;
-int *rp;
-int *fp;
+se_eyepsz(double *wp, double *hp, double *ap, int *rp, int *fp)
 {
 	if (!eyep_w)
 	    se_create_eyep_w();
 
 	*wp = se_getScale (eyepws_w);
 	*hp = se_getScale (eyephs_w);
-	*pp = se_getPScale (eyeppa_w);
+	*ap = se_getPScale (eyepa_w);
 
 	*rp = XmToggleButtonGetState (eyer_w);
 	*fp = XmToggleButtonGetState (eyef_w);
@@ -348,7 +341,7 @@ se_create_eyep_w()
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	w = XmCreateLabel (eyep_w, "L", args, n);
-	set_xmstring (w, XmNlabelString, "Set next eyepiece size, shape and style:");
+	set_xmstring (w, XmNlabelString, "Set next eyepiece size, angle, shape and style:");
 	XtManageChild (w);
 
 	/* w scale and its labels */
@@ -407,14 +400,14 @@ se_create_eyep_w()
 	se_scale_fmt (eyephs_w, eyephl_w);
 	XtManageChild (eyephs_w);
 
-	/* ps scale and its label */
+	/* angle scale and its label */
 
 	n = 0;
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
 	XtSetArg (args[n], XmNtopWidget, eyephs_w); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
-	eyeppl_w = XmCreateLabel (eyep_w, "EyepPL", args, n);
-	XtManageChild (eyeppl_w);
+	eyepal_w = XmCreateLabel (eyep_w, "EyepPL", args, n);
+	XtManageChild (eyepal_w);
 
 	n = 0;
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
@@ -426,19 +419,19 @@ se_create_eyep_w()
 	XtSetArg (args[n], XmNshowValue, False); n++;
 	XtSetArg (args[n], XmNscaleMultiple, 1); n++;
 	XtSetArg (args[n], XmNmaximum, 359); n++;
-	eyeppa_w = XmCreateScale (eyep_w, "EyepPA", args, n);
-	XtAddCallback (eyeppa_w, XmNdragCallback, se_pscale_cb, 0);
-	XtAddCallback (eyeppa_w, XmNvalueChangedCallback, se_pscale_cb, 0);
-	wtip (eyeppa_w, "Slide to desired position angle of eyepiece, E of N");
-	sr_reg (eyeppa_w, NULL, skyepcategory, 0);
-	se_pascale_fmt (eyeppa_w, eyeppl_w);
-	XtManageChild (eyeppa_w);
+	eyepa_w = XmCreateScale (eyep_w, "EyepA", args, n);
+	XtAddCallback (eyepa_w, XmNdragCallback, se_pscale_cb, 0);
+	XtAddCallback (eyepa_w, XmNvalueChangedCallback, se_pscale_cb, 0);
+	wtip (eyepa_w, "Rotation angle of eyepiece, from Z if Alt/Az else from NCP");
+	sr_reg (eyepa_w, NULL, skyepcategory, 0);
+	se_pascale_fmt (eyepa_w, eyepal_w);
+	XtManageChild (eyepa_w);
 
 	/* lock TB */
 
 	n = 0;
 	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNtopWidget, eyeppa_w); n++;
+	XtSetArg (args[n], XmNtopWidget, eyepa_w); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
 	lock_w = XmCreateToggleButton (eyep_w, "Lock", args, n);
 	set_xmstring (lock_w, XmNlabelString, "Lock W and H together");
@@ -869,7 +862,7 @@ se_create_eyep_w()
 	    XtSetArg (args[n], XmNleftOffset, 10); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	    XtSetArg (args[n], XmNrightPosition, 85); n++;
-	    saved_w = XmCreateTextField (eyep_w, "DefFile", args, n);
+	    saved_w = XmCreateTextField (eyep_w, "EyePDefFile", args, n);
 	    XtAddCallback (saved_w, XmNactivateCallback, se_saved_cb, NULL);
 	    sr_reg (saved_w, NULL, skyepcategory, 0);
 	    defaultTextFN (saved_w, 0, defeyepfn, NULL);
@@ -927,7 +920,7 @@ se_create_eyep_w()
 	    XtSetArg (args[n], XmNleftOffset, 10); n++;
 	    XtSetArg (args[n], XmNrightAttachment, XmATTACH_POSITION); n++;
 	    XtSetArg (args[n], XmNrightPosition, 85); n++;
-	    savep_w = XmCreateTextField (eyep_w, "PosFile", args, n);
+	    savep_w = XmCreateTextField (eyep_w, "EyePPosFile", args, n);
 	    XtAddCallback (savep_w, XmNactivateCallback, se_savep_cb, NULL);
 	    sr_reg (savep_w, NULL, skyepcategory, 0);
 	    defaultTextFN (savep_w, 0, defeyelfn, NULL);
@@ -1024,7 +1017,7 @@ Widget s_w, l_w;
 {
 	char buf[64];
 
-	sprintf (buf, "PA %4.0f", raddeg(se_getPScale(s_w)));
+	sprintf (buf, "A  %4.0f", raddeg(se_getPScale(s_w)));
 	set_xmstring (l_w, XmNlabelString, buf);
 }
 
@@ -1045,7 +1038,7 @@ XtPointer call;
 	    XtSetSensitive (lock_w, False);
 	    XtSetSensitive (eyephs_w, False);
 	    XtSetSensitive (eyepws_w, False);
-	    XtSetSensitive (eyeppa_w, False);
+	    XtSetSensitive (eyepa_w, False);
 	} else {
 	    XtSetSensitive (eyef_w, True);
 	    XtSetSensitive (eyeb_w, True);
@@ -1054,7 +1047,7 @@ XtPointer call;
 	    XtSetSensitive (lock_w, True);
 	    XtSetSensitive (eyephs_w, True);
 	    XtSetSensitive (eyepws_w, True);
-	    XtSetSensitive (eyeppa_w, True);
+	    XtSetSensitive (eyepa_w, True);
 	}
 }
 
@@ -1099,7 +1092,7 @@ Widget w;
 XtPointer client;
 XtPointer call;
 {
-	se_pascale_fmt (eyeppa_w, eyeppl_w);
+	se_pascale_fmt (eyepa_w, eyepal_w);
 }
 
 /* callback from the delete-all eyepieces control.
@@ -1365,15 +1358,6 @@ Widget w;
 	return (degrad(v));
 }
 
-/* set the given Scale widget to the given pos angle, in rads */
-static void
-se_setPScale (w, a)
-Widget w;
-double a;
-{
-	XmScaleSetValue (w, (int)raddeg(a));
-}
-
 /* called to install "use" a FavEyeP.
  * client is index into favs[]
  */
@@ -1390,8 +1374,6 @@ XtPointer call;
 	se_scale_fmt (eyepws_w, eyepwl_w);
 	se_setScale (eyephs_w, fp->h);
 	se_scale_fmt (eyephs_w, eyephl_w);
-	se_setPScale (eyeppa_w, fp->pa);
-	se_pascale_fmt (eyeppa_w, eyeppl_w);
 	XmToggleButtonSetState (fp->isE ? eyer_w : eyes_w, True, True);
 	XmToggleButtonSetState (fp->isS ? eyef_w : eyeb_w, True, True);
 	XmToggleButtonSetState (telrad_w, False, True);
@@ -1422,9 +1404,10 @@ XtPointer client;
 XtPointer call;
 {
 	FavEyeP newfav;
+	double aunused;
 
 	memset (&newfav, 0, sizeof(newfav));
-	se_eyepsz (&newfav.w, &newfav.h, &newfav.pa, &newfav.isE, &newfav.isS);
+	se_eyepsz (&newfav.w, &newfav.h, &aunused, &newfav.isE, &newfav.isS);
 	sprintf (newfav.name, "My eyepiece #%d", nfavs+1);
 	se_addFavEyeP (&newfav);
 }
@@ -1468,10 +1451,6 @@ XtPointer call;
 	/* save */
 	se_saveFav(fn);
 
-	/* confirm */
-	if (confirm())
-	    xe_msg (1, "%s saved", fn);
-
 	/* clean up */
 	XtFree (txt);
 }
@@ -1515,10 +1494,6 @@ XtPointer call;
 	/* save */
 	se_savePos(fn);
 
-	/* confirm */
-	if (confirm())
-	    xe_msg (1, "%s saved", fn);
-
 	/* clean up */
 	XtFree (txt);
 }
@@ -1580,7 +1555,7 @@ FavEyeP *newfp;
 	    w = XmCreateLabel (fp->row_w, "FL", args, n);
 	    fs_sexa (wstr, raddeg(fp->w), 2, 60);
 	    fs_sexa (hstr, raddeg(fp->h), 2, 60);
-	    sprintf (buf, "%s %s %3.0f %c %c", wstr, hstr, raddeg(fp->pa),
+	    sprintf (buf, "%s %s  %c %c", wstr, hstr, 
 				    fp->isE ? 'E' : 'R', fp->isS ? 'S' : 'O');
 	    set_xmstring (w, XmNlabelString, buf);
 	    wtip (w, "Coded definition for this eyepiece: W H Shape Style");
@@ -1600,7 +1575,7 @@ se_saveFav(char *fn)
 	    return;			/* already informed user */
 
 	/* write */
-	fprintf (fp, "<Eyepieces>\n");
+	fprintf (fp, "<EyepieceDefinitions>\n");
 	for (i = 0; i < nfavs; i++) {
 	    FavEyeP *fvp = &favs[i];
 	    char *name;
@@ -1608,15 +1583,17 @@ se_saveFav(char *fn)
 	    if (!fvp->inuse)
 		continue;
 	    name = XmTextFieldGetString (fvp->name_w);
-	    fprintf (fp, "  <eyepiece width='%g' height='%g' pa='%g' isElliptical='%d' isSolid='%d'>%s</eyepiece>\n",
-					raddeg(fvp->w), raddeg(fvp->h),
-				    raddeg(fvp->pa), fvp->isE, fvp->isS, name);
+	    fprintf (fp, "  <eyepiece width='%g' height='%g' isElliptical='%d' isSolid='%d'>%s</eyepiece>\n",
+		    raddeg(fvp->w), raddeg(fvp->h), fvp->isE, fvp->isS, name);
 	    XtFree (name);
 	}
-	fprintf (fp, "</Eyepieces>\n");
+	fprintf (fp, "</EyepieceDefinitions>\n");
 
 	/* finished */
 	fclose (fp);
+
+	if (confirm())
+	    xe_msg (1, "Wrote %d definitions to %s", nfavs, fn);
 }
 
 /* read and replace the favorites list from the given file, or file named in
@@ -1657,7 +1634,7 @@ se_loadEyep(char *fn)
 	    xe_msg (1, "%s: %s", fn, msg[0] ? msg : "bad format");
 	    return;
 	}
-	if (strcmp (tagXMLEle(root), "Eyepieces")) {
+	if (strcmp (tagXMLEle(root), "EyepieceDefinitions")) {
 	    xe_msg (1, "%s: not an Eyepieces file", fn);
 	    delXMLEle (root);
 	    return;
@@ -1684,10 +1661,12 @@ se_loadEyep(char *fn)
 	    newf.isS = atoi(findXMLAttValu(ep,"isSolid"));
 	    newf.w = degrad(atof(findXMLAttValu(ep,"width")));
 	    newf.h = degrad(atof(findXMLAttValu(ep,"height")));
-	    newf.pa = degrad(atof(findXMLAttValu(ep,"pa")));
 	    se_addFavEyeP (&newf);
 	}
 
+	if (confirm())
+	    xe_msg (1, "Read %d definitions from %s", nfavs, fn);
+
 	/* finished */
 	delXMLEle (root);
 }
@@ -1705,17 +1684,20 @@ se_savePos(char *fn)
 	    return;			/* already informed user */
 
 	/* write */
-	fprintf (fp, "<EyepiecePositions>\n");
+	fprintf (fp, "<EyepiecePlacements>\n");
 	for (i = 0; i < neyep; i++) {
 	    EyePiece *ep = &eyep[i];
 
-	    fprintf (fp, "  <position ra='%g' dec='%g' alt='%g' az='%g'",
-					    ep->ra, ep->dec, ep->alt, ep->az);
-	    fprintf (fp, " width='%g' height='%g' pa='%g'",
+	    fprintf (fp, "  <placement altdec='%g' azra='%g' aamode='%d'",
+					    ep->altdec, ep->azra, ep->aamode);
+	    fprintf (fp, " width='%g' height='%g' angle='%g'",
 					    ep->eyepw, ep->eyeph, ep->eyepa);
 	    fprintf (fp, " isRound='%d' isSolid='%d' />\n",ep->round,ep->solid);
 	}
-	fprintf (fp, "</EyepiecePositions>\n");
+	fprintf (fp, "</EyepiecePlacements>\n");
+
+	if (confirm())
+	    xe_msg (1, "Wrote %d eyepiece placements to %s", neyep, fn);
 
 	/* finished */
 	fclose (fp);
@@ -1758,8 +1740,8 @@ se_loadPos(char *fn)
 	    xe_msg (1, "%s: %s", fn, msg[0] ? msg : "bad format");
 	    return;
 	}
-	if (strcmp (tagXMLEle(root), "EyepiecePositions")) {
-	    xe_msg (1, "%s: not an Eyepiece Positions file", fn);
+	if (strcmp (tagXMLEle(root), "EyepiecePlacements")) {
+	    xe_msg (1, "%s: not an Eyepiece Placements file", fn);
 	    delXMLEle (root);
 	    return;
 	}
@@ -1773,21 +1755,23 @@ se_loadPos(char *fn)
 	for (ep = nextXMLEle(root,1); ep != NULL; ep = nextXMLEle(root,0)) {
 	    EyePiece *nep;
 
-	    if (strcmp (tagXMLEle(ep), "position"))
+	    if (strcmp (tagXMLEle(ep), "placement"))
 		continue;
 
 	    nep = se_addeyep ();
-	    nep->ra =    atof(findXMLAttValu(ep,"ra"));
-	    nep->dec =   atof(findXMLAttValu(ep,"dec"));
-	    nep->alt =   atof(findXMLAttValu(ep,"alt"));
-	    nep->az =    atof(findXMLAttValu(ep,"az"));
-	    nep->eyepw = atof(findXMLAttValu(ep,"width"));
-	    nep->eyeph = atof(findXMLAttValu(ep,"height"));
-	    nep->eyepa = atof(findXMLAttValu(ep,"pa"));
-	    nep->round = atoi(findXMLAttValu(ep,"isRound"));
-	    nep->solid = atoi(findXMLAttValu(ep,"isSolid"));
+	    nep->altdec = atof(findXMLAttValu(ep,"altdec"));
+	    nep->azra   = atof(findXMLAttValu(ep,"azra"));
+	    nep->aamode = atof(findXMLAttValu(ep,"aamode"));
+	    nep->eyepw  = atof(findXMLAttValu(ep,"width"));
+	    nep->eyeph  = atof(findXMLAttValu(ep,"height"));
+	    nep->eyepa  = atof(findXMLAttValu(ep,"angle"));
+	    nep->round  = atoi(findXMLAttValu(ep,"isRound"));
+	    nep->solid  = atoi(findXMLAttValu(ep,"isSolid"));
 	}
 
+	if (confirm())
+	    xe_msg (1, "Read %d eyepiece placements from %s", neyep, fn);
+
 	/* finished */
 	delXMLEle (root);
 
@@ -1796,4 +1780,4 @@ se_loadPos(char *fn)
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: skyeyep.c,v $ $Date: 2009/01/05 20:55:54 $ $Revision: 1.31 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: skyeyep.c,v $ $Date: 2011/05/16 02:32:23 $ $Revision: 1.32 $ $Name:  $"};
