@@ -47,6 +47,16 @@ telIsOn()
 void
 telGoto (Obj *op)
 {
+	/* check horizons */
+	if (op->s_alt < 0) {
+	    xe_msg (1, "%s is below the horizon", op->o_name);
+	    return;
+	} else if (!sv_hznOpOk(op)) {
+	    xe_msg (1, "%s is below Sky View user horizon map", op->o_name);
+	    return;
+	}
+
+	/* ok */
 	sfifo_sendout (op);
 	(void) sc_goto (op);
 }
@@ -177,4 +187,4 @@ ififo_cb (XtPointer client, int *fdp, XtInputId *idp)
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: skyfifos.c,v $ $Date: 2010/01/18 01:28:09 $ $Revision: 1.7 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: skyfifos.c,v $ $Date: 2011/09/23 01:52:39 $ $Revision: 1.8 $ $Name:  $"};
