@@ -4,13 +4,7 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <math.h>
-
-#if defined(__STDC__)
 #include <stdlib.h>
-typedef const void * qsort_arg;
-#else
-typedef char * qsort_arg;
-#endif
 
 #include <X11/Xlib.h>
 #include <Xm/Xm.h>
@@ -24,31 +18,7 @@ typedef char * qsort_arg;
 #include <Xm/Text.h>
 #include <Xm/Scale.h>
 
-#include "P_.h"
-#include "astro.h"
-#include "circum.h"
-#include "skytoolbar.h"
-
-extern Colormap xe_cm;
-
-extern Now *mm_get_now P_((void));
-extern char *getXRes P_((char *name, char *def));
-extern int is_deepsky P_((Obj *op));
-extern void get_something P_((Widget w, char *resource, XtArgVal value));
-extern void gk_mag P_((double g, double k, double rp, double rho, double *mp));
-extern void hlp_dialog P_((char *tag, char *deflt[], int ndeflt));
-extern void set_something P_((Widget w, char *resource, XtArgVal value));
-extern void set_xmstring P_((Widget w, char *resource, char *txt));
-extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
-extern void sv_all P_((Now *np));
-extern void sv_amagoff P_((void));
-extern void sv_draw_obj P_((Display *dsp, Drawable win, GC gc, Obj *op, int x,
-    int y, int diam, int dotsonly));
-extern void sv_getcenter P_((int *aamode, double *fov,
-    double *altp, double *azp, double *rap, double *decp));
-extern void wtip P_((Widget w, char *tip));
-extern void xe_msg P_((char *msg, int app_modal));
-extern void zero_mem P_((void *loc, unsigned len));
+#include "xephem.h"
 
 
 /* info for each filter option toggle */
@@ -79,22 +49,22 @@ typedef struct {
     int s;		/* mag step per dot size */
 } AutoMag;
 
-static void svf_create_filter P_((Widget mainrc, FilterCat *fcp));
-static void svf_reset P_((void));
-static void svf_apply P_((void));
-static int svf_bright_ok P_((Obj *op));
-static void svf_da_cb P_((Widget w, XtPointer client, XtPointer call));
-static void svf_draw_symbol P_((FilterTB *ftbp, Widget daw));
-static void svf_magdrag_cb P_((Widget w, XtPointer client, XtPointer call));
-static void svf_apply_cb P_((Widget w, XtPointer client, XtPointer call));
-static void svf_ok_cb P_((Widget w, XtPointer client, XtPointer call));
-static void svf_close_cb P_((Widget w, XtPointer client, XtPointer call));
-static void svf_help_cb P_((Widget w, XtPointer client, XtPointer call));
-static void svf_toggle_cb P_((Widget w, XtPointer client, XtPointer call));
-static void svf_all_cb P_((Widget w, XtPointer client, XtPointer call));
-static void svf_reset_cb P_((Widget w, XtPointer client, XtPointer call));
-static void svf_cat_toggle_cb P_((Widget w, XtPointer client, XtPointer call));
-static int svf_initautomag P_((AutoMag **ampp));
+static void svf_create_filter (Widget mainrc, FilterCat *fcp);
+static void svf_reset (void);
+static void svf_apply (void);
+static int svf_bright_ok (Obj *op);
+static void svf_da_cb (Widget w, XtPointer client, XtPointer call);
+static void svf_draw_symbol (FilterTB *ftbp, Widget daw);
+static void svf_magdrag_cb (Widget w, XtPointer client, XtPointer call);
+static void svf_apply_cb (Widget w, XtPointer client, XtPointer call);
+static void svf_ok_cb (Widget w, XtPointer client, XtPointer call);
+static void svf_close_cb (Widget w, XtPointer client, XtPointer call);
+static void svf_help_cb (Widget w, XtPointer client, XtPointer call);
+static void svf_toggle_cb (Widget w, XtPointer client, XtPointer call);
+static void svf_all_cb (Widget w, XtPointer client, XtPointer call);
+static void svf_reset_cb (Widget w, XtPointer client, XtPointer call);
+static void svf_cat_toggle_cb (Widget w, XtPointer client, XtPointer call);
+static int svf_initautomag (AutoMag **ampp);
 
 
 static FilterTB solsys_f[] = {
@@ -109,13 +79,14 @@ static FilterTB other_f[] = {
     {"Q Quasars", 	"Quasars",	FIXED,		'Q'},
     {"L Pulsars", 	"Pulsars",	FIXED,		'L'},
     {"J Radio", 	"Radio",	FIXED,		'J'},
+    {"Y Supernova", 	"Supernova",	FIXED,		'Y'},
     {"R S/N Rem", 	"SNRemnants",	FIXED,		'R'},
-    {"Undefined",	"Undefined",	FIXED,		'\0'}
 };
 
 static FilterTB stars_f[] = {
     {"S Single",	"Stars",	FIXED,		'S'},
-    {"B Binary",	"Binary",	FIXED,		'B'},
+    {"B Binary",	"Binary",	BINARYSTAR,	 0},
+       /* B as a subfield of FIXED merged into D as of BINARYSTAR */
     {"D Double",	"Double",	FIXED,		'D'},
     {"M Multiple",	"Multiple",	FIXED,		'M'},
     {"V Variable",	"Variable",	FIXED,		'V'},
@@ -218,8 +189,12 @@ Widget shell_w;
 	XtSetArg (args[n], XmNmarginWidth, 8); n++;
 	XtSetArg (args[n], XmNmarginHeight, 8); n++;
 	XtSetArg (args[n], XmNcolormap, xe_cm); n++;
+	XtSetArg (args[n], XmNdefaultPosition, False); n++;
 	filter_w = XmCreateFormDialog (shell_w, "SkyFilter", args, n);
 	set_something (filter_w, XmNcolormap, (XtArgVal)xe_cm);
+	XtAddCallback (filter_w, XmNhelpCallback, svf_help_cb, NULL);
+	sr_reg (XtParent(filter_w), "XEphem*SkyFilter.x", skyfcategory, 0);
+	sr_reg (XtParent(filter_w), "XEphem*SkyFilter.y", skyfcategory, 0);
 
 	/* set some stuff in the parent DialogShell.
 	 * setting XmNdialogTitle in the Form didn't work..
@@ -441,11 +416,7 @@ char ct[NCLASSES];
 
 /* return the current mag limits and mag step */
 void
-svf_getmaglimits (stmagp, ssmagp, dsmagp, magstpp)
-int *stmagp;
-int *ssmagp;
-int *dsmagp;
-int *magstpp;
+svf_getmaglimits (int *stmagp, int *ssmagp, int *dsmagp, int *magstpp)
 {
 	*stmagp = stmag;
 	*ssmagp = ssmag;
@@ -658,14 +629,14 @@ svf_apply()
 		int t = ftbp->type;
 		if (t < 0 || t >= XtNumber(type_table)) {
 		    printf ("svf_apply: type out of range: %d\n", t);
-		    exit (1);
+		    abort();
 		}
 		ftbp->state = XmToggleButtonGetState (ftbp->tbw);
 		if (t == FIXED) {
 		    int c = (int)ftbp->class;
 		    if (c < 0 || c >= XtNumber(fclass_table)) {
 			printf ("svf_apply: FIXED class out of range: %d\n", c);
-			exit (1);
+			abort();
 		    }
 		    fclass_table[c] = ftbp->state;
 		} else
@@ -712,7 +683,8 @@ Obj *op;
 	    return (mag <= ssmag);
 	    /* break; */
 	    }
-	case FIXED:
+	case FIXED:		/* FALLTHRU */
+	case BINARYSTAR:
 	    return (get_mag(op) <= (is_deepsky(op) ? dsmag : stmag));
 	    /* break; */
 	case EARTHSAT:
@@ -720,7 +692,7 @@ Obj *op;
 	    return (1);
 	default: 
 	    printf ("sv_bright_ok(): bad type: %d\n", op->o_type);
-	    exit (1);
+	    abort();
 	    return (0);	/* for lint */
 	}
 }
@@ -774,6 +746,7 @@ FilterTB *ftbp;
 Widget daw;
 {
 	static GC gc;
+	static Pixel fg_p, bg_p;
 	Display *dsp = XtDisplay(daw);
 	Window win = XtWindow(daw);
 	Obj o;
@@ -782,10 +755,18 @@ Widget daw;
 
 	zero_mem ((void *)&o, sizeof(Obj));
 	o.o_type = ftbp->type;
-	if (ftbp->type == FIXED) {
+	switch (ftbp->type) {
+	case BINARYSTAR:
+	    o.b_nbp = 1;	/* just to avoid orb ele calc */
+	    break;
+	case FIXED:
 	    o.f_class = ftbp->class;
 	    set_ratio (&o, 2, 1);
 	    set_pa (&o, PI/2);
+	    break;
+	case PLANET:
+	    o.s_phase = 100;
+	    break;
 	}
 
 	get_something (daw, XmNwidth, (XtArgVal)&w);
@@ -793,16 +774,14 @@ Widget daw;
 	diam = (w > h ? h : w) - 2; /* allow for a small border */
 
 	if (!gc) {
-	    XGCValues gcv;
-	    unsigned long gcm;
-	    Pixel p;
-
-	    gcm = GCForeground;
-	    get_something (daw, XmNforeground, (XtArgVal)&p);
-	    gcv.foreground = p;
-	    gc = XCreateGC (dsp, win, gcm, &gcv);
+	    get_something (daw, XmNforeground, (XtArgVal)&fg_p);
+	    get_something (daw, XmNbackground, (XtArgVal)&bg_p);
+	    gc = XCreateGC (dsp, win, 0L, NULL);
 	}
 
+	XSetForeground (dsp, gc, bg_p);
+	XFillRectangle (dsp, win, gc, 0, 0, w, h);
+	XSetForeground (dsp, gc, fg_p);
 	sv_draw_obj (dsp, win, gc, &o, (int)w/2-1, (int)h/2-1, diam, 0);
 	sv_draw_obj (dsp, win, gc, NULL, 0, 0, 0, 0);	/* flush */
 }
@@ -867,7 +846,7 @@ XtPointer call;
 "This allows settings the types of objects to display."
 };
 
-	hlp_dialog ("Sky View - filter", msg, sizeof(msg)/sizeof(msg[0]));
+	hlp_dialog ("SkyView_filter", msg, sizeof(msg)/sizeof(msg[0]));
 }
 
 /* called when Toggle is pushed on the filter dialog */
@@ -943,9 +922,7 @@ XtPointer call;
 /* given two pointers to MagAuto, return how they sort by r in qsort-style.
  */
 static int
-am_cf (v1, v2)
-qsort_arg v1;
-qsort_arg v2;
+am_cf (const void * v1, const void * v2)
 {
 	AutoMag *am1 = (AutoMag *)v1;
 	AutoMag *am2 = (AutoMag *)v2;
@@ -971,7 +948,6 @@ svf_initautomag(ampp)
 AutoMag **ampp;
 {
 	static char AMResource[] = "AutoMag";
-	char buf[128];
 	double r;
 	int st, ss, ds, s;
 	AutoMag *amp;
@@ -983,8 +959,7 @@ AutoMag **ampp;
 	/* get the automag resource */
 	am = getXRes (AMResource, NULL);
 	if (!am) {
-	    (void) sprintf (buf, "No %s resource found", AMResource);
-	    xe_msg (buf, 0);
+	    xe_msg (0, "No %s resource found", AMResource);
 	    return (0);
 	}
 
@@ -999,7 +974,7 @@ AutoMag **ampp;
 	    AutoMag *tamp;
 
 	    if (!new) {
-		xe_msg ("Insufficient memory for AutoMag table", 0);
+		xe_msg (0, "Insufficient memory for AutoMag table");
 		break;
 	    }
 	    amp = (AutoMag *)new;
@@ -1019,7 +994,7 @@ AutoMag **ampp;
 	}
 
 	if (namp == 0) {
-	    xe_msg ("No valid entries in AutoMag resource", 0);
+	    xe_msg (0, "No valid entries in AutoMag resource");
 	    return (0);
 	}
 
@@ -1027,14 +1002,11 @@ AutoMag **ampp;
 	qsort ((void *)amp, namp, sizeof(AutoMag), am_cf);
 
 	/* report results to message log because the format is a bit obscure */
-	(void)sprintf(buf,"Sky View found %d AutoMag table entries as follows:",
-									namp);
-	xe_msg (buf, 0);
-	xe_msg ("     FOV  Stars    SolSys   DeepSky   DotSiz", 0);
+	xe_msg(0, "Sky View found %d AutoMag table entries as follows:", namp);
+	xe_msg(0, "     FOV  Stars    SolSys   DeepSky   DotSiz");
 	for (i = 0; i < namp; i++) {
-	    (void) sprintf (buf, "  %6.2f    %3d      %3d    %3d       %3d",
+	    xe_msg (0, "  %6.2f    %3d      %3d    %3d       %3d",
 		raddeg(amp[i].r), amp[i].st, amp[i].ss, amp[i].ds, amp[i].s);
-	    xe_msg (buf, 0);
 	}
 
 	*ampp = amp;
@@ -1042,4 +1014,4 @@ AutoMag **ampp;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: skyfiltmenu.c,v $ $Date: 2000/12/01 08:52:35 $ $Revision: 1.10 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: skyfiltmenu.c,v $ $Date: 2004/05/05 17:43:32 $ $Revision: 1.24 $ $Name:  $"};
