@@ -20,7 +20,7 @@
 extern char skycategory[];
 
 static char skyhfn[]= "xephem_skyhist";	/* file name to store history */
-#define	VERSIONN		4	/* version number */
+#define	VERSIONN		5	/* version number */
 
 /* info to manage any number of history entries and their controls.
  * we destroy and recreate all the entries in the Dialog each time the
@@ -707,7 +707,7 @@ XtPointer call;
 	    fprintf(histfp, "%*s\n", GRIDLEN, hp->vgrid);
 	    fprintf(histfp, "%*s\n", GRIDLEN, hp->hgrid);
 	    fprintf(histfp,
-		"%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n",
+		"%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d\n",
 		hp->winw, hp->winh, hp->cyl_proj, hp->aa_mode,
 		hp->flip_lr, hp->flip_tb,
 		hp->justd, hp->eclip, hp->galac, hp->eq,
@@ -715,7 +715,7 @@ XtPointer call;
 		hp->eyep, hp->magscale, hp->automag,
 		hp->lbl_lst, hp->lbl_lfs, hp->lbl_lss, hp->lbl_lds,
 		hp->lbl_bst, hp->lbl_bfs, hp->lbl_bss, hp->lbl_bds,
-		hp->conr);
+		hp->conr, hp->hznclipping);
 	    for (j=0; j < (int)NOBJTYPES; j++) 
 		fprintf(histfp, "%c", hp->type_table[j]?'1':'0');
 	    fprintf(histfp, "\n");
@@ -777,7 +777,7 @@ char *msg;
 	if (sscanf(buf, "Version %d", &v) != 1) {
 	    sprintf (msg, "%s:\nNo Version line:\n%s\n", fn, buf);
 	    goto err;
-	} else if (!(v == VERSIONN || v == VERSIONN-1)) {
+	} else if (!(v <= VERSIONN && v >= VERSIONN-2)) {
 	    sprintf (msg, "%s:\nWrong version, must be %d.", fn, VERSIONN);
 	    goto err;
 	}
@@ -820,7 +820,7 @@ char *msg;
 	    if (!fgets(buf, sizeof(buf), histfp))
 		goto err;
 	    n = sscanf(buf,
-		"%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
+		"%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
 		    &hp.winw, &hp.winh, &hp.cyl_proj, &hp.aa_mode,
 		    &hp.flip_lr, &hp.flip_tb,
 		    &hp.justd, &hp.eclip, &hp.galac, &hp.eq,
@@ -828,8 +828,10 @@ char *msg;
 		    &hp.eyep, &hp.magscale, &hp.automag,
 		    &hp.lbl_lst, &hp.lbl_lfs, &hp.lbl_lss, &hp.lbl_lds,
 		    &hp.lbl_bst, &hp.lbl_bfs, &hp.lbl_bss, &hp.lbl_bds,
-		    &hp.conr);
-	    if (v == 3 && n == 24) {
+		    &hp.conr, &hp.hznclipping );
+	    if (v == 4 && n == 27) {
+		hp.hznclipping = 1;
+	    } else if (v == 3 && n == 24) {
 		hp.lbl_bds = hp.lbl_bfs;
 		hp.lbl_bss = hp.lbl_bst;
 		hp.lbl_bst = hp.lbl_lds;
@@ -839,7 +841,8 @@ char *msg;
 		hp.lbl_bss = 10;
 		hp.lbl_bds = 10;
 		hp.conr = 0;
-	    } else if (!(v == VERSIONN && n == 27)) {
+		hp.hznclipping = 1;
+	    } else if (!(v == VERSIONN && n == 28)) {
 		sprintf (msg, "%s:\nBogus switches line .. \n%s", fn, buf);
 		goto err;
 	    }
@@ -912,4 +915,4 @@ char *how;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: skyhist.c,v $ $Date: 2009/01/05 20:55:54 $ $Revision: 1.39 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: skyhist.c,v $ $Date: 2012/07/07 18:04:42 $ $Revision: 1.40 $ $Name:  $"};
