@@ -1301,7 +1301,7 @@ XtPointer call;
 		int d = dx*dx + (y - hp->sy)*(y - hp->sy);
 		if (d < mind) {
 		    mind = d;
-		    op = (j == 0) ? favs[i] : &hp->o;
+		    op = &hp->o;
 		    Mjd = hp->trts.t;
 		    fav = 1;
 		}
@@ -2377,4 +2377,4 @@ mk_gcs()
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: solsysmenu.c,v $ $Date: 2009/01/05 20:55:54 $ $Revision: 1.46 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: solsysmenu.c,v $ $Date: 2009/08/04 03:04:42 $ $Revision: 1.47 $ $Name:  $"};
