@@ -10,6 +10,7 @@
 #include <math.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <time.h>
 
 #include <Xm/Xm.h>
 #include <Xm/Form.h>
@@ -41,10 +42,9 @@ static void sun_print_cb (Widget w, XtPointer client, XtPointer data);
 static void sun_mloop_cb (Widget w, XtPointer client, XtPointer data);
 static void sun_motion_eh (Widget w, XtPointer client, XEvent *ev,
     Boolean *continue_to_dispatch);
-static int getLatestFN (char fn[]);
-static int readSOHOImage (char fn[]);
-static int displayGif (char fn[], unsigned char *gif, int gifl);
-static void storeGifFN (char fn[], int w);
+static int readSOHOImage(void);
+static int displayPic (char fn[], unsigned char *pic, int picl);
+static void storePicFN (char fn[], int w);
 static int getTS (int *tp, int *sp);
 static int scanDir (char *dir, char ***files, int nfiles);
 static void sun_refresh(void);
@@ -74,9 +74,9 @@ static Widget save_w;		/* save PB */
 static Widget coords_w;		/* coordinates label */
 static Widget crn_w;		/* Carrington rotation number label */
 static Pixmap sun_pm;		/* staging area */
-static unsigned char *rawgif;	/* current raw gif file, if any */
-static long rawgifl;		/* length of rawgif */
-static char *giffn;		/* malloced name of current gif */
+static unsigned char *rawpic;	/* current raw picture file, if any */
+static long rawpicl;		/* length of rawpic */
+static char *picfn;		/* malloced name of current picture */
 
 /* info about each image format choice */
 typedef struct {
@@ -93,8 +93,8 @@ static SOHOType stype[] = {
     {"EIT195",	"EIT 195",         "eit_195",	  0,  0, degrad(0.00075)},
     {"EIT284",	"EIT 284",         "eit_284",	  0,  0, degrad(0.00075)},
     {"EIT304",	"EIT 304",         "eit_304",	  0,  0, degrad(0.00075)},
-    {"MDICon",	"MDI Continuum",   "mdi_igr",	  0,  0, degrad(0.00055)},
-    {"MDIMag",	"MDI Magnetogram", "mdi_mag",	  0,  0, degrad(0.00055)},
+    {"HMICon",	"HMI Continuum",   "hmi_igr",	  0,  0, degrad(0.00055)},
+    {"HMIMag",	"HMI Magnetogram", "hmi_mag",	  0,  0, degrad(0.00055)},
     {"LASC2",	"LASCO C2",        "c2",	-16, 20, degrad(0.00365)},
     {"LASC3",	"LASCO C3",        "c3",	-16, 20, degrad(0.00775)},
 };
@@ -108,7 +108,6 @@ typedef struct {
     Widget w;			/* controlling TB */
 } SOHOSize;
 static SOHOSize ssize[] = {
-    {"256x256",		"256x256",         "256", 	4},
     {"512x512",		"512x512",         "512",	2},
     {"1024x1024",	"1024x1024",       "1024",	1},
 };
@@ -677,20 +676,20 @@ sun_gather (char ***files)
 	return (nfiles);
 }
 
-/* display the given gif file stream */
+/* display the given picture file stream */
 static void
 sun_display (FILE *fp, char *name)
 {
-	unsigned char *gif;
-	int gifl;
+	unsigned char *pic;
+	int picl;
 
 	fseek (fp, 0L, SEEK_END);
-	gifl = (int) ftell(fp);
-	gif = (unsigned char *) XtMalloc (gifl);
+	picl = (int) ftell(fp);
+	pic = (unsigned char *) XtMalloc (picl);
 	fseek (fp, 0L, SEEK_SET);
-	fread (gif, gifl, 1, fp);
-	if (displayGif (name, gif, gifl) < 0)
-	    XtFree ((char *)gif);		/* already told user why */
+	fread (pic, picl, 1, fp);
+	if (displayPic (name, pic, picl) < 0)
+	    XtFree ((char *)pic);		/* already told user why */
 }
 
 /* called when the Files pulldown is about to come up */
@@ -780,20 +779,10 @@ sun_load_cb (Widget w, XtPointer client, XtPointer data)
 static void
 sun_download_cb (Widget w, XtPointer client, XtPointer data)
 {
-	char fn[1024];
-
 	watch_cursor (1);
 
-	/* get name of file with latest image */
-	stopd_up();
-	if (getLatestFN (fn) < 0) {
-	    stopd_down();
-	    watch_cursor (0);
-	    return;
-	}
-
 	/* read image */
-	if (readSOHOImage (fn) < 0) {
+	if (readSOHOImage() < 0) {
 	    stopd_down();
 	    watch_cursor (0);
 	    return;
@@ -818,29 +807,29 @@ sun_save_cb (Widget w, XtPointer client, XtPointer data)
 	XtSetSensitive (save_w, False);
 
 	/* sanity check */
-	if (!giffn || !rawgif) {
+	if (!picfn || !rawpic) {
 	    xe_msg (1, "No image to save");
 	    return;
 	}
 
 	/* create file */
-	sprintf (fn, "%s/%s", getPrivateDir(), giffn);
+	sprintf (fn, "%s/%s", getPrivateDir(), picfn);
 	fp = fopenh (fn, "w");
 	if (!fp) {
-	    xe_msg (1, "%s:\n%s", giffn, syserrstr());
+	    xe_msg (1, "%s:\n%s", fn, syserrstr());
 	    return;
 	}
 
 	/* write */
-	if (fwrite (rawgif, rawgifl, 1, fp) != 1) {
-	    xe_msg (1, "%s:\n%s", giffn, syserrstr());
+	if (fwrite (rawpic, rawpicl, 1, fp) != 1) {
+	    xe_msg (1, "%s:\n%s", picfn, syserrstr());
 	    fclose (fp);
 	    return;
 	}
 
 	/* ok */
 	if (confirm())
-	    xe_msg (1, "%s:\nWritten successfully", giffn);
+	    xe_msg (1, "%s:\nWritten successfully", picfn);
 	fclose (fp);
 }
 
@@ -861,7 +850,7 @@ sun_motion_eh (Widget w, XtPointer client, XEvent *ev, Boolean *dispatch)
 	double r, d;
 	char cbuf[64], rbuf[32], dbuf[32];
 
-	if (!giffn || !rawgif)
+	if (!picfn || !rawpic)
 	    return;
 
 	XGetGeometry (dsp, win, &root, &rx, &ry, &wid, &hei, &bw, &dep);
@@ -879,89 +868,37 @@ sun_motion_eh (Widget w, XtPointer client, XEvent *ev, Boolean *dispatch)
 	set_xmstring (coords_w, XmNlabelString, cbuf);
 }
 
-/* get the file name of the latest image with the current type and size.
+/* read picture from sohohost and display.
+ * store picture in rawpic[] and its filename in *picfn.
  * return 0 if ok else xe_msg and -1
  */
 static int
-getLatestFN(char fn[])
+readSOHOImage()
 {
-	static char start_string[] = "individual images";
+	char fn[1024];
 	int t, s;
-	char *page, *sp;
 	char get[1024];
-	char msg[1024];
-	int npage;
-	int socket;
-	int n;
+	char buf[1024];
+	char filetime[100];
+	char filetype[100];
+	int isjpeg, jpegl;
+	int njpeg;
+	unsigned char *jpeg;
+	int fd, nr;
+	struct tm tm;
+
+	memset(&tm, 0, sizeof(struct tm));
 
 	/* get desired type and size */
 	if (getTS (&t, &s) < 0)
 	    return (-1);
 
-	/* build http command for page that lists latest fn */
-	sprintf (get, "GET http://%s/data/realtime/realtime-%s-%s.html HTTP/1.0\r\nUser-Agent: xephem/%s\r\n\r\n", sohohost, stype[t].file, ssize[s].file, PATCHLEVEL);
-
-	/* open */
-	socket = httpGET (sohohost, get, msg);
-	if (socket < 0) {
-	    xe_msg (1, "http get: %s", msg);
-	    return (-1);
-	}
-
-	/* read entire page */
-	npage = 0;
-	page = NULL;
-	n = 0;
-	do {
-	    npage += n;
-	    page = XtRealloc (page, npage+NSREAD);
-	} while ((n = readbytes (socket, (unsigned char *) (page+npage),
-								NSREAD)) > 0);
-	close (socket);
-	if (n < 0) {
-	    xe_msg (1, "%s", syserrstr());
-	    XtFree (page);
-	    return (-1);
-	}
-
-	/* dig where the latest URL is */
-	sp = strstr (page, start_string);
-	if (!sp) {
-	    xe_msg (1, "Can not find '%s'", start_string);
-	    xe_msg (0, page);
-	    XtFree (page);
-	    return (-1);
-	}
-	sp = strchr (sp, '"');
-	if (!sp) {
-	    xe_msg (1, "No double-quote after '%s'", start_string);
-	    xe_msg (0, page);
-	    XtFree (page);
-	    return (-1);
-	}
-	sp++;			/* skip " */
-	sprintf (fn, "%.*s", (int)(strchr(sp,'"')-sp), sp);
-	XtFree (page);
-	return (0);
-}
-
-/* given the name of a gif file, read from sohohost and display.
- * store gif in rawgif[] and its filename in *giffn.
- * return 0 if ok else xe_msg and -1
- */
-static int
-readSOHOImage (char fn[])
-{
-	char get[1024];
-	char buf[1024];
-	int isgif, gifl;
-	int ngif;
-	unsigned char *gif;
-	int fd, nr;
+	/* build latest fn */
+	sprintf (filetype, "%s", stype[t].file);
+	sprintf (fn, "/data/realtime/%s/%s/latest.jpg", filetype, ssize[s].file);
 
 	/* build GET command */
-	sprintf (get, "GET http://%s%s HTTP/1.0\r\nUser-Agent: xephem/%s\r\n\r\n",
-						    sohohost, fn, PATCHLEVEL);
+	sprintf (get, "GET http://%s%s HTTP/1.0\r\nUser-Agent: xephem/%s\r\n\r\n", sohohost, fn, PATCHLEVEL);
 
 	/* query server */
 	fd = httpGET (sohohost, get, buf);
@@ -970,36 +907,41 @@ readSOHOImage (char fn[])
 	    return (-1);
 	}
 
-	/* read header (everything to first blank line), looking for gif */
-	isgif = 0;
-	gifl = 0;
+	/* read header (everything to first blank line), looking for jpeg */
+	isjpeg = 0;
+	jpegl = 0;
 	while (recvline (fd, buf, sizeof(buf)) > 1) {
 	    xe_msg (0, "Rcv: %s", buf);
-	    if (strstr (buf, "Content-Type:") && strstr (buf, "image/gif"))
-		isgif = 1;
+	    if (strstr (buf, "Content-Type:") && strstr (buf, "image/jpeg"))
+		isjpeg = 1;
 	    if (strstr (buf, "Content-Length:"))
-		gifl = atoi (buf+15);
+		jpegl = atoi (buf+15);
+	    if (strstr (buf, "Last-Modified:")) {
+		strptime(buf+20, "%d %b %Y %H:%M:%S %Z", &tm);
+		strftime(filetime, sizeof(filetime), "%Y%m%d_%H%M%S", &tm);
+		xe_msg (0, "Rcv: Filetime: %s", filetime);
+	    }
 	}
-	if (!isgif) {
+	if (!isjpeg) {
 	    while (recvline (fd, buf, sizeof(buf)) > 0)
 		xe_msg (0, "Rcv: %s", buf);
 	    xe_msg (1, "Error talking to SOHO .. see File->System log\n");
 	    close (fd);
 	    return (-1);
 	}
-	if (gifl == 0) {
+	if (jpegl == 0) {
 	    xe_msg (1, "No Content-Length in header");
 	    close (fd);
 	    return (-1);
 	}
 
-	/* remainder should be a gif file, read into gif[] */
+	/* remainder should be a jpeg file, read into jpeg[] */
 	pm_up();
-	gif = NULL;
-	for (ngif = 0; ngif < gifl; ngif += nr) {
-	    pm_set (100*ngif/gifl);
-	    gif = (unsigned char *) XtRealloc ((char*)gif, ngif+NSREAD);
-	    nr = readbytes (fd, gif+ngif, NSREAD);
+	jpeg = NULL;
+	for (njpeg = 0; njpeg < jpegl; njpeg += nr) {
+	    pm_set (100*njpeg/jpegl);
+	    jpeg = (unsigned char *) XtRealloc ((char*)jpeg, njpeg+NSREAD);
+	    nr = readbytes (fd, jpeg+njpeg, NSREAD);
 	    if (nr < 0) {
 		xe_msg (1, "%s:\n%s", sohohost, syserrstr());
 		pm_down();
@@ -1012,28 +954,71 @@ readSOHOImage (char fn[])
 	pm_down();
 	close (fd);
 
-	/* display gif */
-	if (displayGif (fn, gif, ngif) < 0)
+        sprintf (fn, "/%s_%s.jpg", filetime, filetype);
+	/* display jpeg */
+	if (displayPic (fn, jpeg, njpeg) < 0)
 	    return (-1);
 
 	/* clean up */
 	return (0);
 }
 
-/* display the gif file named fn and install as the "current" file.
+/* display the picture file named fn and install as the "current" file.
  * return 0 if ok, else -1 and why xe_msg()
- * N.B. gif must be malloced memory and user should /not/ free, let us manage it
+ * N.B. picture must be malloced memory and user should /not/ free, let us manage it
  */
 static int
-displayGif (char fn[], unsigned char *gif, int gifl)
+displayPic (char fn[], unsigned char *pic, int picl)
 {
+	static XColor xcol[256];
+	static int xcolset;
 	Pixmap pm;
 	char buf[1024];
 	int w, h;
+	int ok = 0;
 
 	/* explode into pm */
-	if (gif2pm (XtD, xe_cm, gif, gifl, &w, &h, &pm, buf) < 0) {
-	    xe_msg (1, "%s:\n%s", fn, buf);
+	if (strstr (fn, ".gif")) {
+	    ok = 1;
+	    if (gif2pm (XtD, xe_cm, pic, picl, &w, &h, &pm, buf) < 0) {
+		xe_msg (1, "Display GIF: %s:\n%s", fn, buf);
+		return (-1);
+	    }
+	}
+	if (strstr (fn, ".jpg")) {
+	    char tmpfn[2048];
+	    FILE *tmpfp;
+
+	    ok = 1;
+
+	    /* free last batch of colors */
+	    if (xcolset) {
+		freeXColors (XtD, xe_cm, xcol, XtNumber(xcol));
+		xcolset = 0;
+	    }
+
+	    /* convert JPEG to pixmap */
+	    (void) tempfilename (tmpfn, "suntmp", ".jpg");
+	    tmpfp = fopen (tmpfn, "wb+");
+	    if (!tmpfp) {
+		xe_msg (1, "Can not create tmp file: %s", tmpfn);
+		return (-1);
+	    }
+	    fwrite(pic, picl, 1, tmpfp);
+	    rewind(tmpfp);
+	    if (jpeg2pm (XtD, xe_cm, tmpfp, &w, &h, &pm, xcol, buf) < 0) {
+		xe_msg (1, "Display JPEG: %s:\n%s", fn, buf);
+		fclose (tmpfp);
+		unlink (tmpfn);
+		return (-1);
+	    }
+	    xcolset = 1;
+	    fclose (tmpfp);
+	    unlink (tmpfn);
+	}
+
+	if (ok == 0) {
+	    xe_msg (1, "Unknown file type: %s", fn);
 	    return (-1);
 	}
 
@@ -1045,18 +1030,18 @@ displayGif (char fn[], unsigned char *gif, int gifl)
 	set_something (sunda_w, XmNheight, (XtArgVal)(Dimension)h);
 	centerScrollBars(ssw_w);
 	sun_refresh();
-	if (rawgif)
-	    XtFree ((char *)rawgif);
-	rawgif = gif;
-	rawgifl = gifl;
-	storeGifFN (fn, w);
+	if (rawpic)
+	    XtFree ((char *)rawpic);
+	rawpic = pic;
+	rawpicl = picl;
+	storePicFN (fn, w);
 
 	return (0);
 }
 
-/* store base of fn[] in giffn[], include width w in name somewhere */
+/* store base of fn[] in picfn[], include width w in name somewhere */
 static void
-storeGifFN (char fn[], int w)
+storePicFN (char fn[], int w)
 {
 	char buf[1024];
 	char *sp, *lsp;
@@ -1071,9 +1056,9 @@ storeGifFN (char fn[], int w)
 	}
 
 	sprintf (buf, "SOHO_%d_%s", w, lsp+1);
-	if (giffn)
-	    XtFree (giffn);
-	giffn = XtNewString (buf);
+	if (picfn)
+	    XtFree (picfn);
+	picfn = XtNewString (buf);
 }
 
 /* get indices into user's current type and size settings.
@@ -1224,5 +1209,5 @@ carrington(Now *np)
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: sunmenu.c,v $ $Date: 2010/10/06 21:12:16 $ $Revision: 1.22 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: sunmenu.c,v $ $Date: 2012/03/07 01:10:15 $ $Revision: 1.23 $ $Name:  $"};
 
