@@ -1,65 +1,38 @@
+/* code to manage what the outside world sees as the db_ interface.
+ */
+
 #include <stdio.h>
 #include <ctype.h>
 #include <math.h>
-
-
-#if defined(__STDC__)
 #include <stdlib.h>
 #include <string.h>
-typedef const void * qsort_arg;
-#else
-typedef void * qsort_arg;
-extern void *malloc(), *realloc();
-#endif
-
-#if defined(_POSIX_SOURCE)
 #include <unistd.h>
-#else
-extern int read();
-extern int close();
-#endif
 
-#ifndef SEEK_SET
-#define	SEEK_SET 0
-#define	SEEK_END 2
-#endif
+#include <X11/Intrinsic.h>
 
-#include "P_.h"
-#include "astro.h"
-#include "circum.h"
+#include "xephem.h"
 
-extern FILE *fopenh P_((char *name, char *how));
-extern Now *mm_get_now P_((void));
-extern char *syserrstr P_((void));
-extern int db_read P_((FILE *fp, int nodups));
-extern double delra P_((double dra));
-extern int openh P_((char *name, int flags, ...));
-extern void all_newdb P_((int appended));
-extern void db_update P_((Obj *op));
-extern void pm_set P_((int p));
-extern void watch_cursor P_((int want));
-extern void xe_msg P_((char *msg, int app_modal));
-
-static int db_add P_((Obj *newop));
-static void db_init P_((void));
+static int db_objadd (Obj *newop);
+static void db_init (void);
 
 #define	MAXDBLINE	256	/* longest allowed db line */
 
-#define	DBFIFO_MSG	'!'	/* introduces a message line from the DBFIFO */
-
 /* This counter is incremented when we want to mark all the Obj derived entries
  * as being out-of-date. This works because then each of the age's will be !=
  * db_age. This is to eliminate ever calling obj_cir() under the same
  * circumstances for a given db object.
- * N.B. For this to work, call db_update() before using a Obj *.
+ * N.B. For this to work, call db_update() not obj_cir().
  */
-static ObjAge_t db_age;
+static ObjAge_t db_age = 1;	/* insure all objects are initially o-o-d */
 
 /* the "database".
  * one such struct per object type. the space for objects is malloced in
  *   seperate chunks of DBCHUNK to avoid needing big contiguous memory blocks.
+ *   the space is never freed, the total is just reduced and can be reused.
+ * N.B. deleting by catalog depends on all objects from a given catalog being
+ *   appended contiguously within their respective ObjType list.
  * N.B. because the number is fixed and known, we use static storage for the
- *   NOBJ Objects for the PLANET type; see db_init().
+ *   NOBJ Objects for the PLANET type, not malloced storage; see db_init().
  */
 #define	DBCHUNK	256	/* number we malloc more of at once; a power of two
 			 * might help the compiler optimize the divides and
@@ -69,19 +42,21 @@ typedef struct {
     char **dblist;	/* malloced list of malloced DBCHUNKS arrays */
     int nobj;		/* number of objects actually in use */
     int nmem;		/* total number of objects for which we have room */
-    int ncp;		/* nobj at time of last checkpoint */
     int size;		/* bytes per object */
 } DBMem;
 static DBMem db[NOBJTYPES];	/* this is the head for each object */
+
+/* return true if the database has been initialized */
 #define	DBINITED	(db[PLANET].dblist)	/* anything setup by db_init()*/
 
 /* macro that returns the address of an object given its type and index */
 #define	OBJP(t,n)	\
 		((Obj *)(db[t].dblist[(n)/DBCHUNK] + ((n)%DBCHUNK)*db[t].size))
 
-static void db_free P_((DBMem *dmp));
+#define DB_SIZE_ROUND(s)        \
+                          (((s) + sizeof(double) - 1) & ~(sizeof(double) - 1))
 
-typedef char MaxNm[MAXNM];	/* eases readability of an array of these */
+static void db_free (DBMem *dmp);
 
 /* return number of objects in the database.
  * this includes the NOBJ basic objects.
@@ -101,25 +76,8 @@ db_n()
 	return (n);
 }
 
-/* return the total current number of objects as of the last checkpoint.
- * this includes the NOBJ basic objects.
- */
-int
-db_n_cp()
-{
-	DBMem *dmp;
-	int n;
-
-	if (!DBINITED)
-	    db_init();
-
-	for (n = 0, dmp = db; dmp < &db[NOBJTYPES]; dmp++)
-	    n += dmp->ncp;
-	return (n);
-}
-
-/* given one of the basic ids in astro.h return pointer to its updated Obj in
- * the database.
+/* given one of the basic ids in astro.h or circum.h return pointer to its
+ * updated Obj in the database.
  */
 Obj *
 db_basic(id)
@@ -130,7 +88,7 @@ int id;
 	if (!DBINITED)
 	    db_init();
 
-	if (id < 0 || id >= NOBJ) {
+	if (id < 0 || id >= db[PLANET].nobj) {
 	    printf ("db_basic(): bad id: %d\n", id);
 	    exit (1);
 	}
@@ -141,22 +99,7 @@ int id;
 	return (op);
 }
 
-/* set our official copy of the given user-defined object to *op */
-void
-db_setuserobj(id, op)
-int id;	/* OBJXYZ */
-Obj *op;
-{
-	if (id == OBJX || id == OBJY || id == OBJZ) {
-	    Obj *bop = OBJP(PLANET,id);
-	    *bop = *op;
-	} else {
-	    printf ("db_setuserobj(): bad id: %d\n", id);
-	    exit (1);
-	}
-}
-
-/* mark all db objects as old
+/* mark all db objects as out-of-date
  */
 void
 db_invalidate()
@@ -185,7 +128,7 @@ int nop;
 	if (!DBINITED)
 	    db_init();
 
-	sp->t = 0;
+	sp->t = UNDEFOBJ;
 	sp->n = 0;
 	sp->m = mask;
 	sp->op = op;
@@ -199,38 +142,30 @@ Obj *
 db_scan (sp)
 DBScan *sp;
 {
-	Obj *op;
-
 	if (!DBINITED)
 	    db_init();
 
-    	/* check op/nop only after everything else is complete */
-    doop:
-	if (sp->t == NOBJTYPES) {
-	    if (sp->op && sp->n < sp->nop)
-		return ((Obj*)&sp->op[sp->n++]);
-	    else
-		return (NULL);
-	}
-
-	/* outter loop is just to skip over undefined user objects, if any. */
-	do {
-	    /* advance to the next type if we have scanned all the ones of
-	     * the current type or if the type is not desired.
-	     */
-	    while (sp->n >= db[sp->t].nobj || !((1<<sp->t) & sp->m)) {
-		sp->n = 0;
-		if (++sp->t == NOBJTYPES) {
-		    /* no more in the real db -- try op */
-		    goto doop;
+	/* find next object, splicing in ObjF list with FIXED */
+	while (sp->t < NOBJTYPES) {
+	    if ((1<<sp->t) & sp->m) {
+		if (sp->t == FIXED && sp->op) {
+		    if (sp->n < sp->nop)
+			return ((Obj*)&sp->op[sp->n++]);
+		    sp->op = NULL;		/* flag to turn off op list */
+		    sp->n = 0;
+		}
+		/* return next for this type, skipping any UNDEF user objects */
+		while (sp->n < db[sp->t].nobj) {
+		    Obj *op = OBJP(sp->t, sp->n);	/* MACRO 2nd arg twice*/
+		    sp->n++;
+		    if (op->o_type != UNDEFOBJ)
+			return (op);
 		}
 	    }
-
-	    op = OBJP(sp->t, sp->n);
-	    sp->n++;
-	} while (op->o_type == UNDEFOBJ);
-
-	return (op);
+	    sp->t++;
+	    sp->n = 0;
+	}
+	return (NULL);
 }
 
 /* see to it that all the s_* fields in the given db object are up to date.
@@ -257,13 +192,9 @@ Obj *op;
 	    exit (1);
 	} 
 
-	if (op->o_age != db_age ||
-		    op == OBJP(PLANET,OBJX) || op == OBJP(PLANET,OBJY ||
-					       op == OBJP(PLANET,OBJZ))) {
-	    if (obj_cir (mm_get_now(), op) < 0) {
-		printf ("%s: bad object\n", me);
-		exit(1);
-	    }
+	if (op->o_age != db_age) {
+	    if (obj_cir (mm_get_now(), op) < 0)
+		xe_msg (0, "%s: no longer valid", op->o_name);
 	    op->o_age = db_age;
 	}
 }
@@ -278,184 +209,96 @@ db_del_all()
 	if (!DBINITED)
 	    db_init();
 
+	/* free memory for each type */
 	for (dmp = db; dmp < &db[NOBJTYPES]; dmp++) {
 	    /* N.B. PLANET entries are fixed -- not malloced */
 	    if (dmp == &db[PLANET])
-		continue;
+		continue;	/* N.B. except planets! */
 	    db_free (dmp);
 	}
 }
 
-/* delete all objects back to the last checkpoint.
- * we don't actually free memory.
- */
-void
-db_del_cp ()
-{
-	DBMem *dmp;
-
-	if (!DBINITED)
-	    db_init();
-
-	for (dmp = db; dmp < &db[NOBJTYPES]; dmp++)
-	    dmp->nobj = dmp->ncp;
-}
-
-/* set the checkpoint to the current set of objects in the db.
- */
-void
-db_set_cp()
-{
-	DBMem *dmp;
-
-	if (!DBINITED)
-	    db_init();
-
-	for (dmp = db; dmp < &db[NOBJTYPES]; dmp++)
-	    dmp->ncp = dmp->nobj;
-}
-
-/* set the checkpoints back to 0.
- * N.B. never fiddle with PLANET's count.
- */
-void
-db_clr_cp()
-{
-	DBMem *dmp;
-
-	if (!DBINITED)
-	    db_init();
-
-	for (dmp = db; dmp < &db[NOBJTYPES]; dmp++) {
-	    /* N.B. PLANET entries are fixed -- not malloced */
-	    if (dmp != &db[PLANET])
-		dmp->ncp = 0;
-	}
-}
-
-/* compare 2 pointers to MaxNm's in qsort fashion */
-static int
-name_cmpf (s1, s2)
-qsort_arg s1, s2;
-{
-	return (strcmp (*(MaxNm *)s1, *(MaxNm *)s2));
-}
-
-/* see whether name is in names[nnames] (which is sorted in ascending order).
- * if so, return 0, else -1
- */
-static int
-chk_name (name, names, nnames)
-char *name;
-MaxNm *names;
-int nnames;
-{
-	int t, b, m, s;
-
-	/* check for no names at all */
-	if (nnames <= 0)
-	    return (-1);
-
-	/* binary search */
-	t = nnames - 1;
-	b = 0;
-	while (b <= t) {
-	    m = (t+b)/2;
-	    s = strcmp (name, names[m]);
-	    if (s == 0)
-		return (0);
-	    if (s < 0)
-		t = m-1;
-	    else
-		b = m+1;
-	}
-
-	return (-1);
-}
-
-/* read the given database file into memory.
- * add to the existing list.
- * if nodups, skip objects whose name already appears in memory.
+/* read the given .edb or .tle file into memory.
+ * add a new catalog entry, sorted by catalog name, update GUI.
  * stop gracefully if we run out of memory.
- * return 0 if all ok, else -1.
+ * keep operator informed.
+ * look in several places for file.
+ * if enabled and only one object in whole file, preload into an ObjXYZ.
  */
-int
-db_read (fp, nodups)
-FILE *fp;
-int nodups;
+void
+db_read (fn)
+char *fn;
 {
-	char buf[MAXDBLINE];
-	MaxNm *names = 0;
-	int nnames = 0;
+	char bufs[3][MAXDBLINE];
+	char *brot, *b0 = bufs[0], *b1 = bufs[1], *b2 = bufs[2];
+	int nobjs;
+	Obj o;
+	char *base;
+	FILE *fp;
 	long len;
+	int fok;
 
 	if (!DBINITED)
 	    db_init();
 
+	/* skip any leading blanks */
+	while (*fn == ' ')
+	    fn++;
+
+	/* open the file.
+	 * try looking for fn in several places.
+	 */
+	fp = fopenh (fn, "r");
+	if (fp)
+	    goto ok;
+	xe_msg (1, "%s:\n%s", fn, syserrstr());
+	return;
+
+	/* need pure base of fn */
+    ok:
+	for (base = fn+strlen(fn);
+		    base > fn && base[-1] != '/' && base[-1] != '\\'; --base)
+	    continue;
+
 	/* set up to run the progress meter based on file position */
 	(void) fseek (fp, 0L, SEEK_END);
 	len = ftell (fp);
 	(void) fseek (fp, 0L, SEEK_SET);
-	pm_set (0);
-
-	if (nodups) {
-	    /* build sorted list of existing names. 
-	     * N.B. actually copy the names in case the real db moves.
-	     */
-	    DBScan dbs;
-	    int mask = ALLM;
-	    Obj *op;
-
-	    nnames = 0;
-	    names = (MaxNm *) malloc (db_n() * sizeof(MaxNm));
-	    if (!names) {
-		xe_msg ("No memory to check dups", 1);
-		return (-1);
-	    }
-	    for (db_scaninit(&dbs,mask,NULL,0); (op = db_scan(&dbs)) != NULL; )
-		(void) strcpy (names[nnames++], op->o_name);
-	    if (nnames > db_n()) {
-		printf ("db_read: too many objects! %d %d\n", db_n(), nnames);
-		exit (1);
-	    }
-
-	    qsort ((void *)names, nnames, sizeof(MaxNm), name_cmpf);
-	}
 
 	/* read each line from the file and add good ones to the db */
-	while (fgets (buf, sizeof(buf), fp)) {
-	    char whynot[256];
-	    Obj o;
-
-	    pm_set ((int)(ftell(fp)*100/len)); /* update progress meter */
-
-	    if (db_crack_line (buf, &o, whynot) < 0) {
-		if (whynot[0] != '\0')
-		    printf ("Bad db: %s: %s\n", buf, whynot);
-		continue;
+	nobjs = 0;
+	while (fgets (b2, MAXDBLINE, fp)) {
+	    if (db_crack_line (b2,&o,0,0,0) > 0 || !db_tle (b0,b1,b2,&o)) {
+		if (db_objadd (&o) < 0) {
+		    xe_msg (1, "No more memory");
+		    fclose(fp);
+		    return;
+		}
+		nobjs++;
 	    }
 
-	    if (nodups) {
-		char c, nm[MAXNM];
-		int i;
-
-		for (i = 0; i < (MAXNM-1) && (c = buf[i]) != ','; i++)
-		    nm[i] = c;
-		nm[i] = '\0';
-		if (chk_name (nm, names, nnames) == 0)
-		    continue;
-	    }
-
-	    if (db_add (&o) < 0)
-		break;
+	    /* rotate for possible TLE */
+	    brot = b0;
+	    b0 = b1;
+	    b1 = b2;
+	    b2 = brot;
 	}
 
-	/* finished with names */
-	if (names)
-	    free ((void *)names);
+	/* clean up */
+	fok = !ferror(fp);
+	fclose(fp);
 
-	/* should have hit eof -- if not, other trouble */
-	return (feof(fp) ? 0 : -1);
+	/* check for trouble */
+	if (!fok) {
+	    xe_msg (1, "%s:\n%s", base, syserrstr());
+	    return;
+	}
+
+	/* record result */
+	if (nobjs == 0)
+	    xe_msg (1, "%s contains no data", base);
+	else
+	    xe_msg (0, "%s: contained %d objects", base, nobjs);
 }
 
 /* allocate *newop to the appropriate list, growing if necessary.
@@ -463,7 +306,7 @@ int nodups;
  * N.B we do *not* validate newop in any way.
  */
 static int
-db_add (newop)
+db_objadd (newop)
 Obj *newop;
 {
 	int t = newop->o_type;
@@ -477,17 +320,18 @@ Obj *newop;
 	    char **newdbl;
 	    char *newchk;
 
+	    /* grow list of chunks */
 	    newdbl = dmp->dblist ? (char **) realloc (dmp->dblist, newdblsz)
 				 : (char **) malloc (newdblsz);
 	    if (!newdbl)
 		return (-1);
 
+	    /* add 1 chunk */
 	    newchk = malloc (dmp->size * DBCHUNK);
 	    if (!newchk) {
 		free ((char *)newdbl);
 		return (-1);
 	    }
-
 	    newdbl[ndbl] = newchk;
 
 	    dmp->dblist = newdbl;
@@ -506,40 +350,24 @@ Obj *newop;
 static void
 db_init()
 {
-	/* these must match the order in astro.h */
-	static char *planet_names[] = {
-	    "Mercury", "Venus", "Mars", "Jupiter", "Saturn",
-	    "Uranus", "Neptune", "Pluto", "Sun", "Moon",
-	};
-	static Obj plan_objs[NOBJ];	/* OBJXYZ can be anything so use Obj */
-	static char *plan_dblist[1] = {(char *)plan_objs};
-	int i;
+	static Obj *plan_dblist[1];
 
 	/* init the object sizes.
 	 * N.B. must do this before using the OBJP macro
 	 */
 	db[UNDEFOBJ].size	= 0;
-	db[FIXED].size		= sizeof(ObjF);
-	db[ELLIPTICAL].size	= sizeof(ObjE);
-	db[HYPERBOLIC].size	= sizeof(ObjH);
-	db[PARABOLIC].size	= sizeof(ObjP);
-	db[EARTHSAT].size	= sizeof(ObjES);
-	db[PLANET].size		= sizeof(Obj); /* *NOT* ObjPl */
+	db[FIXED].size		= DB_SIZE_ROUND(sizeof(ObjF));
+	db[BINARYSTAR].size	= DB_SIZE_ROUND(sizeof(ObjB));
+	db[ELLIPTICAL].size	= DB_SIZE_ROUND(sizeof(ObjE));
+	db[HYPERBOLIC].size	= DB_SIZE_ROUND(sizeof(ObjH));
+	db[PARABOLIC].size	= DB_SIZE_ROUND(sizeof(ObjP));
+	db[EARTHSAT].size	= DB_SIZE_ROUND(sizeof(ObjES));
 
-	/* init the planets.
-	 * because the number is fixed, we use static storage for the NOBJ
-	 * Objects for the PLANET type.
+	/* init access to the built-in objects
 	 */
-	db[PLANET].dblist = plan_dblist;
-	db[PLANET].nmem = NOBJ;
-	db[PLANET].nobj = NOBJ;
-	db[PLANET].ncp = NOBJ;
-	for (i = MERCURY; i <= MOON; i++) {
-	    Obj *op = OBJP (PLANET, i);
-	    op->o_type = PLANET;
-	    (void) strncpy (op->o_name, planet_names[i], sizeof(op->o_name)-1);
-	    op->pl.pl_code = i;
-	}
+	db[PLANET].nmem = db[PLANET].nobj = getBuiltInObjs (&plan_dblist[0]);
+	db[PLANET].dblist = (char **) plan_dblist;
+	db[PLANET].size		= sizeof(Obj); /* *NOT* ObjPl */
 }
 
 /* free all the memory associated with the given DBMem */
@@ -557,8 +385,7 @@ DBMem *dmp;
 
 	dmp->nobj = 0;
 	dmp->nmem = 0;
-	dmp->ncp = 0;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: db.c,v $ $Date: 1998/10/16 04:36:17 $ $Revision: 1.2 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: db.c,v $ $Date: 2004/06/21 02:48:15 $ $Revision: 1.5 $ $Name:  $"};
