@@ -1,7 +1,10 @@
-/* read the UCAC2 or UCAC3 catalog, depending on which index file is found.
+/* read the UCAC2, 3 or 4 catalog, depending on which index file is found.
  * UCACSetup(): call to change options and base directories.
  * UCACFetch(): return an array of ObjF matching the given criteria.
- * stars are in 360 files each .5 deg hi, within 240 bins each .1 hour RA wide.
+ * UCAC2 and 3 stars are in 360 files each .5 deg hi, within 240 bins each
+    .1 hour RA wide.
+ * UCAC4 stars are in 900 files each .2 deg hi, within 1440 bins each
+    1 minute of RA wide.
  */
 
 #include <stdio.h>
@@ -21,7 +24,7 @@ typedef unsigned int UI;		/* unsigned integer */
 /* access an I*2 or I*4 at offset i in UC array a in little-endian byte order.
  * a bit slow but ultra portable.
  */
-#define	I2(a,i)		((int)((((UI)(a)[i]) | (((UI)(a)[i+1])<<8))))
+#define	I2(a,i)		((int)(short)((((UI)(a)[i]) | (((UI)(a)[i+1])<<8))))
 #define	I4(a,i)		((int)((((UI)(a)[i]) | (((UI)(a)[i+1])<<8) | \
 				(((UI)(a)[i+2])<<16) | (((UI)(a)[i+3])<<24))))
 
@@ -31,20 +34,32 @@ typedef struct {
     int n;				/* number actually in use */
 } ObjFArray;
 
+/* zone sizes for UCAC2 and 3 */
 #define	NZW	240			/* number ra zones wide */
 #define	ZW	(24./NZW)		/* zone width, hours */
 #define	NZH	360			/* number dec zones hi */
 #define	ZH	(180./NZH)		/* zone height, degrees */
 
+/* zones sizes for UCAC4 */
+#define	NZW4	1440			/* number ra zones wide */
+#define	ZW4	(24./NZW4)		/* zone width, hours */
+#define	NZH4	900			/* number dec zones hi */
+#define	ZH4	(180./NZH4)		/* zone height, degrees */
+
 #define	DPMAS	(1.0/3600000.0)		/* degrees per milliarcsecond */
 
-static int ucacv;			/* UCAC catalog version: 2 or 3 */
 typedef UC U2Star[44];			/* UCAC2 record */
 typedef UC U3Star[84];			/* UCAC3 record */
+typedef UC U4Star[78];			/* UCAC4 record */
 static char *basedir;			/* full dir with zone files and index */
 static FILE *indexfp;			/* index file handle */
 
-static FILE *openIndex (char *path, char *msg);
+static FILE *openIndex (char dir[], char msg[], int *ucacvp);
+static int add4Bin (ObjFArray *oap, int rz, int dz);
+static int read4Index (int rz, int dz, int *nskip, int *nnew);
+static int read4Raw (U4Star u[], int dz, int nskip, int nraw);
+static void crack4 (U4Star u, int dz, int znm, Obj *op);
+static int readu4hpm (int rnm, int *pmra, int *pmdec);
 static int add3Bin (ObjFArray *oap, int rz, int dz);
 static int read3Index (int rz, int dz, int *nskip, int *nnew);
 static int read3Raw (U3Star u[], int dz, int nskip, int nraw);
@@ -60,14 +75,13 @@ static void jkspect (double j, double k, Obj *op);
  * return 0 if looks ok, else -1 and reason in msg[].
  */
 int
-UCACSetup (path, msg)
-char *path;
-char msg[];
+UCACSetup (char *path, char msg[])
 {
 	FILE *fp;
+	int ucacv;
 
 	/* sanity check the path by looking for the index file */
-	fp = openIndex (path, msg);
+	fp = openIndex (path, msg, &ucacv);
 	if (!fp)
 	    return (-1);
 	fclose (fp);
@@ -91,19 +105,23 @@ char msg[];
  * msg might contain a message regardless of the return value.
  */
 int
-UCACFetch (r0, d0, fov, fmag, opp, nopp, msg)
-double r0;	/* center RA, rads */
-double d0;	/* center Dec, rads */
-double fov;	/* field of view, rads */
-double fmag;	/* faintest mag */
-ObjF **opp;	/* *opp will be a malloced array of the ObjF in region */
-int nopp;       /* if opp: initial number of ObjF already in *opp */
-char msg[];	/* filled with error message if return -1 */
+UCACFetch (
+double r0,	/* center RA, rads */
+double d0,	/* center Dec, rads */
+double fov,	/* field of view, rads */
+double fmag,	/* faintest mag */
+ObjF **opp,	/* *opp will be a malloced array of the ObjF in region */
+int nopp,       /* if opp: initial number of ObjF already in *opp */
+char msg[] 	/* status or error message if msg[0] != '\0' on return */
+)
 {
-	int (*doucac)();			/* function to process */
+	int (*doucac)() = 0;		/* function to process */
 	int d0z, drovz;			/* dec center and radius, in 0-zones */
 	ObjFArray oa;			/* malloc accumulator */
 	int dz;				/* scanning dec zone number */
+	double zw = 0, zh = 0;		/* zone width and height */
+	int nzw = 0, nzh = 0;		/* number of zones wide and high */
+	int ucacv;			/* version 2, 3, or 4 */
 
 	/* init message */
 	msg[0] = '\0';
@@ -121,10 +139,37 @@ char msg[];	/* filled with error message if return -1 */
 	}
 
 	/* open the index file */
-	indexfp = openIndex (basedir, msg);
+	indexfp = openIndex (basedir, msg, &ucacv);
 	if (!indexfp)
 	    return (-1);
 
+	/* setup based on which version */
+	switch (ucacv) {
+	case 2:
+	    doucac = add2Bin;
+	    zw = ZW;
+	    zh = ZH;
+	    nzw = NZW;
+	    nzh = NZH;
+	    break;
+
+	case 3:
+	    doucac = add3Bin;
+	    zw = ZW;
+	    zh = ZH;
+	    nzw = NZW;
+	    nzh = NZH;
+	    break;
+
+	case 4:
+	    doucac = add4Bin;
+	    zw = ZW4;
+	    zh = ZH4;
+	    nzw = NZW4;
+	    nzh = NZH4;
+	    break;
+	}
+
 	/* init the array.
 	 * mem==NULL means we just keep a count but don't build the array.
 	 */
@@ -139,37 +184,34 @@ char msg[];	/* filled with error message if return -1 */
 	}
 
 	/* convert to zones */
-	d0z = (int)floor(raddeg(d0+PI/2)/ZH);
-	drovz = (int)ceil(raddeg(fov/2)/ZH);
-
-	/* choose function depending on catalog */
-	doucac = ucacv == 2 ? add2Bin : add3Bin;
+	d0z = (int)floor(raddeg(d0+PI/2)/zh);
+	drovz = (int)ceil(raddeg(fov/2)/zh);
 
 	/* scan each ra bin in each dec band */
 	for (dz = d0z - drovz; dz <= d0z + drovz; dz++) {
 	    int rz, rz0, rz1;			/* scanning ra zones */
-	    if (dz < 0 || dz >= NZH)
+	    if (dz < 0 || dz >= nzh)
 		continue;			/* over a pole */
-	    if (dz > 2*NZH-d0z-drovz || dz < drovz-d0z) {
+	    if (dz > 2*nzh-d0z-drovz || dz < drovz-d0z) {
 		rz0 = 0;			/* pole in view */
-		rz1 = NZW-1;
+		rz1 = nzw-1;
 	    } else {
 		int r0z, rrovz;			/* RA center and rad, 0-zones */
-		r0z = (int)floor(radhr(r0)/ZW);
+		r0z = (int)floor(radhr(r0)/zw);
 		if (fabs(d0) < PI/2) {
-		    rrovz = (int)ceil(radhr(fov/2)/ZW/cos(d0));
-		    if (rrovz > NZW/2)
-			rrovz = NZW/2;
+		    rrovz = (int)ceil(radhr(fov/2)/zw/cos(d0));
+		    if (rrovz > nzw/2)
+			rrovz = nzw/2;
 		} else
-		    rrovz = NZW/2;
+		    rrovz = nzw/2;
 		rz0 = r0z - rrovz;		/* normal patch */
 		rz1 = r0z + rrovz;
 	    }
 	    for (rz = rz0; rz <= rz1; rz++)
 		if ((*doucac) (&oa, rz, dz) < 0) {
 		    char rstr[32], dstr[32];
-		    fs_sexa (rstr, rz*ZW, 2, 60);
-		    fs_sexa (dstr, dz*ZH-90, 3, 60);
+		    fs_sexa (rstr, rz*zw, 2, 60);
+		    fs_sexa (dstr, dz*zh-90, 3, 60);
 		    sprintf (msg, "Error reading UCAC data near %s %s",rstr,dstr);
 		    if (!opp && oa.mem)
 			free ((void *)oa.mem);
@@ -189,6 +231,221 @@ char msg[];	/* filled with error message if return -1 */
 	return (oa.n);
 }
 
+/* add the stars in the given 0-based ra/dec zone to oap.
+ * return 0 if ok, else -1
+ * N.B. rz may wrap 
+ */
+static int
+add4Bin (ObjFArray *oap, int rz, int dz)
+{
+	int nskip, nnew;
+	U4Star *u;
+	int i;
+
+	/* beware of ra wrap */
+	if (rz < 0)
+	    rz += NZW4;
+	if (rz >= NZW4)
+	    rz -= NZW4;
+
+	/* n stars up through and including this patch */
+	if (read4Index (rz, dz, &nskip, &nnew) < 0)
+	    return (-1);
+
+	/* read the raw star records in this record */
+	u = malloc (nnew * sizeof(U4Star));
+	if (!u) {
+	    xe_msg (0, "UCAC: malloc(%d) failed", nnew*sizeof(U4Star));
+	    return (-1);
+	}
+	if (read4Raw (u, dz, nskip, nnew) < 0) {
+	    free ((char *)u);
+	    return (-1);
+	}
+
+	/* expand obj memory and crack if interested */
+	if (oap->mem) {
+	    char *moreobj = realloc (oap->mem, (oap->n + nnew)*sizeof(ObjF));
+	    if (!moreobj) {
+		xe_msg (0, "UCAC: malloc failed");
+		free (u);
+		return (-1);
+	    }
+	    oap->mem = (ObjF *)moreobj;
+
+	    for (i = 0; i < nnew; i++)
+		crack4 (u[i], dz+1, nskip+i+1, (Obj *)(oap->mem + oap->n + i));
+	}
+
+	/* always update count */
+	oap->n += nnew;
+
+	/* ok */
+	free ((char *)u);
+	return (0);
+}
+
+/* given 0-based ra and dec zone, return number of stars to skip in dec band
+ * and how many are in the zone.
+ * return 0 if ok, else -1
+ */
+static int
+read4Index (int rz, int dz, int *nskip, int *nnew)
+{
+	off_t offset;
+	UC i4[4];
+
+	offset = (rz*NZH4 + dz)*sizeof(i4);
+	if (fseek (indexfp, offset, SEEK_SET) < 0) {
+	    xe_msg (0, "UCAC: index n0 seek %ld: %s", offset, syserrstr());
+	    return (-1);
+	}
+	if (fread (i4, sizeof(i4), 1, indexfp) != 1) {
+	    xe_msg (0, "UCAC: index n0 read %d: %s", sizeof(i4), syserrstr());
+	    return (-1);
+	}
+	*nskip = I4(i4, 0);
+
+
+	offset = (NZW4*NZH4 + rz*NZH4 + dz)*sizeof(i4);
+	if (fseek (indexfp, offset, SEEK_SET) < 0) {
+	    xe_msg (0, "UCAC: index nn seek %ld: %s", offset, syserrstr());
+	    return (-1);
+	}
+	if (fread (i4, sizeof(i4), 1, indexfp) != 1) {
+	    xe_msg (0, "UCAC: index nn read %d: %s", sizeof(i4), syserrstr());
+	    return (-1);
+	}
+	*nnew  = I4(i4, 0);
+
+	return (0);
+}
+
+/* read the nnew raw star records starting after nskip records for the given
+ * 0-based dec zone.
+ * return 0 if ok, else -1
+ */
+static int
+read4Raw (U4Star u[], int dz, int nskip, int nnew)
+{
+	char fn[1024];
+	FILE *fp;
+
+	sprintf (fn, "%s/u4b/z%03d", basedir, dz+1);
+	fp = fopen (fn, "r");
+	if (!fp) {
+	    xe_msg (0, "UCAC: open %s: %s", fn, syserrstr());
+	    return (-1);
+	}
+	if (fseek (fp, nskip*sizeof(U4Star), SEEK_SET) < 0) {
+	    xe_msg (0, "UCAC: seek %ld %s: %s", nskip*sizeof(U4Star), fn, syserrstr());
+	    fclose (fp);
+	    return (-1);
+	}
+	if (fread (u, sizeof(U4Star), nnew, fp) != nnew) {
+	    xe_msg (0, "UCAC: read %d, %s: %s", sizeof(U4Star), fn, syserrstr());
+	    fclose (fp);
+	    return (-1);
+	}
+	fclose (fp);
+	return (0);
+}
+
+/* convert the raw UCAC record into the ObjF portion of op.
+ * proper motion applied to 2000, libastro::circum.c takes it from there.
+ */
+static void
+crack4 (U4Star u, int dz, int znm, Obj *op)
+{
+	int rawpmra, rawpmdec;
+	double epra, epdc;
+	double spd;
+
+	memset (op, 0, sizeof(ObjF));	/* N.B. ObjF, not Obj */
+
+	op->o_type = FIXED;
+	op->f_class = (u[14] == 0) ? ((u[13] == 0) ? 'S' : 'T') : 'D';
+	op->f_RA = degrad (I4(u,0)*DPMAS);
+	spd = I4(u,4)*DPMAS;
+	op->f_dec = degrad (spd - 90);
+	op->f_epoch = J2000;
+	set_fmag (op, I2(u,10)/1000.0);
+
+	epra = I2(u,20)*.01+1900;	/* pmra epoch, years */
+	epdc = I2(u,22)*.01+1900;	/* pmdec epoch, years */
+	rawpmra = I2(u,24);		/* .1 mas/yr, on sky */
+	rawpmdec = I2(u,26);		/* .1 mas/yr */
+	if (rawpmra == 32767 || rawpmdec == 32767) {
+	    int rnm = I4(u,68);
+	    if (readu4hpm (rnm, &rawpmra, &rawpmdec) < 0)
+		xe_msg (1, "Can not find proper motion for star UCAC4 %d", rnm);
+	}
+	op->f_pmRA =  degrad(rawpmra*0.1*DPMAS)/365.25/cos(op->f_dec);	/* want RA rad/day */
+	op->f_pmdec = degrad(rawpmdec*0.1*DPMAS)/365.25;		/* want Dec rad/day */
+	op->f_RA += op->f_pmRA * (2000.0 - epra)*365.25;		/* move to 2000 */
+	op->f_dec += op->f_pmdec * (2000.0 - epdc)*365.25;		/* move to 2000 */
+
+	jkspect (I2(u,34)*0.001, I2(u,38)*0.001, op);
+
+	sprintf (op->o_name, "UCAC4-%03d-%06d", dz, znm);
+}
+
+/* convert running star number to proper motion.
+ * return 0 if ok, else -1
+ */
+static int
+readu4hpm (int rnm, int *pmra, int *pmdec)
+{
+	typedef struct {
+	    int rnm;
+	    int zn;
+	    int rnz;
+	    int pmrc;
+	    int pmd;
+	    int ra;
+	    int spd;
+	    int maga;
+	} U4HPM;
+	static U4HPM *u4hpm;
+	static int nu4hpm;
+	int i;
+
+	/* cache file first time only */
+	if (!u4hpm) {
+	    char fn[1024];
+	    char line[128];
+	    FILE*fp;
+
+	    sprintf (fn, "%s/u4i/u4hpm.dat", basedir);
+	    fp = fopen (fn, "r");
+	    if (!fp) {
+		xe_msg (1, "Can not open %s", fn);
+		return (-1);
+	    }
+	    while (fgets (line, sizeof(line), fp)) {
+		u4hpm = (U4HPM *) realloc (u4hpm, (nu4hpm+1)*sizeof(U4HPM));
+		U4HPM *up = &u4hpm[nu4hpm++];
+		sscanf (line, "%d %d %d %d %d %d %d %d", &up->rnm, &up->zn, &up->rnz,
+				&up->pmrc, &up->pmd, &up->ra, &up->spd, &up->maga);
+	    }
+	    fclose (fp);
+
+	};
+
+	/* search for matching rnm */
+	for (i = 0; i < nu4hpm; i++) {
+	    if (rnm == u4hpm[i].rnm) {
+		*pmra = u4hpm[i].pmrc;
+		*pmdec = u4hpm[i].pmd;
+		return (0);
+	    }
+	}
+
+	xe_msg (1, "Can not find UCAC4 u4hpm entry for %d", rnm);
+	/* not found */
+	return (-1);
+}
+
 /* add the stars in the given 0-based ra/dec zone to oap.
  * return 0 if ok, else -1
  * N.B. rz may wrap 
@@ -280,7 +537,7 @@ read3Index (int rz, int dz, int *nskip, int *nnew)
 }
 
 /* read the nnew raw star records starting after nskip records for the given
- * 0-based ra/dec patch.
+ * 0-based dec zone.
  * return 0 if ok, else -1
  */
 static int
@@ -309,6 +566,24 @@ read3Raw (U3Star u[], int dz, int nskip, int nnew)
 	return (0);
 }
 
+/* convert the raw UCAC record into the ObjF portion of op */
+static void
+crack3 (U3Star u, Obj *op)
+{
+	memset (op, 0, sizeof(ObjF));	/* N.B. ObjF, not Obj */
+
+	sprintf (op->o_name, "3UCAC%09d", I4(u,80));
+	op->o_type = FIXED;
+	op->f_class = (u[14] == 0 || u[14] == 1) ? 'S' : 'T';
+	op->f_RA = degrad (I4(u,0)*DPMAS);
+	op->f_dec = degrad (I4(u,4)*DPMAS - 90.0);
+	op->f_epoch = J2000;
+	set_fmag (op, I2(u,10)/1000.0);
+	op->f_pmRA =  degrad(I4(u,28)/10.0*DPMAS)/365.0/cos(op->f_dec);
+	op->f_pmdec = degrad(I4(u,32)/10.0*DPMAS)/365.0;
+	jkspect (I2(u,44)*0.001, I2(u,48)*0.001, op);
+}
+
 /* add the stars in the given 0-based ra/dec zone to oap.
  * return 0 if ok, else -1
  * N.B. rz may wrap 
@@ -400,7 +675,7 @@ get2N (int rz, int dz, int *idp)
 }
 
 /* read the n raw star records starting after nskip records from the given
- * 0-based ra/dec patch.
+ * 0-based dec zone.
  * return 0 if ok, else -1
  */
 static int
@@ -446,48 +721,38 @@ crack2 (U2Star u, int id, Obj *op)
 	jkspect (I2(u,36)*0.001, I2(u,40)*0.001, op);
 }
 
-/* open index file in dir, return FILE * else NULL with message */
+/* open index file in dir, return version and FILE * else NULL with message */
 static FILE *
-openIndex (char dir[], char msg[])
+openIndex (char dir[], char msg[], int *ucacvp)
 {
 	static char u2[] = "u2index.da";	/* zone index file name */
 	static char u3[] = "u3index.unf";	/* zone index file name */
+	static char u4[] = "u4index.unf";	/* zone index file name */
 	char full[1024];
 	FILE *fp;
 
 	sprintf (full, "%s/%s", dir, u2);
 	fp = fopen (full, "r");
 	if (fp) {
-	    ucacv = 2;
+	    *ucacvp = 2;
 	} else {
 	    sprintf (full, "%s/%s", dir, u3);
 	    fp = fopen (full, "r");
 	    if (fp) {
-		ucacv = 3;
-	    } else
-		sprintf (msg, "Can not find %s or %s", u2, u3);
+		*ucacvp = 3;
+	    } else {
+		sprintf (full, "%s/u4i/%s", dir, u4);
+		fp = fopen (full, "r");
+		if (fp) {
+		    *ucacvp = 4;
+		} else {
+		    sprintf (msg, "Can not find %s or %s or %s", u2, u3, u4);
+		}
+	    }
 	}
 	return (fp);
 }
 
-/* convert the raw UCAC record into the ObjF portion of op */
-static void
-crack3 (U3Star u, Obj *op)
-{
-	memset (op, 0, sizeof(ObjF));	/* N.B. ObjF, not Obj */
-
-	sprintf (op->o_name, "3UCAC%09d", I4(u,80));
-	op->o_type = FIXED;
-	op->f_class = (u[14] == 0 || u[14] == 1) ? 'S' : 'T';
-	op->f_RA = degrad (I4(u,0)*DPMAS);
-	op->f_dec = degrad (I4(u,4)*DPMAS - 90.0);
-	op->f_epoch = J2000;
-	set_fmag (op, I2(u,10)/1000.0);
-	op->f_pmRA =  degrad(I4(u,28)/10.0*DPMAS)/365.0/cos(op->f_dec);
-	op->f_pmdec = degrad(I4(u,32)/10.0*DPMAS)/365.0;
-	jkspect (I2(u,44)*0.001, I2(u,48)*0.001, op);
-}
-
 /* given the J and K_s fields in the UCAC catalog, fill in f_spect[] in op.
  * from Brian Skiff off TASS "Stellar Catalogs" page, with artistic license.
  * http://stupendous.rit.edu/tass/catalogs/uvby.calib
@@ -525,8 +790,3 @@ jkspect (double j, double k, Obj *op)
 	    }
 	}
 }
-
-
-
-/* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: ucac.c,v $ $Date: 2010/08/09 04:57:42 $ $Revision: 1.6 $ $Name:  $"};
