@@ -4,9 +4,8 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <math.h>
-#if defined(__STDC__)
 #include <stdlib.h>
-#endif
+
 #include <X11/Xlib.h>
 #include <X11/keysym.h>
 #include <Xm/Xm.h>
@@ -14,77 +13,66 @@
 #include <Xm/Frame.h>
 #include <Xm/DrawingA.h>
 #include <Xm/Label.h>
+#include <Xm/PanedW.h>
 #include <Xm/PushB.h>
 #include <Xm/ToggleB.h>
 #include <Xm/Text.h>
 #include <Xm/Scale.h>
 
-#include "P_.h"
-#include "astro.h"
-#include "circum.h"
-#include "ps.h"
-#include "patchlevel.h"
-
-extern Widget toplevel_w;
-extern Colormap xe_cm;
-extern XtAppContext xe_app;
+#include "xephem.h"
 
 extern char helpcategory[];
 
-extern void get_something P_((Widget w, char *resource, XtArgVal value));
-extern void set_something P_((Widget w, char *resource, XtArgVal value));
-extern void sr_reg P_((Widget w, char *res, char *cat, int autosav));
-
-static void v_create_vshell P_((void));
-static void fill_msg P_((Widget w));
-static void v_popdown_cb P_((Widget w, XtPointer client, XtPointer call));
-static void v_ok_cb P_((Widget w, XtPointer client, XtPointer call));
-static void v_da_exp_cb P_((Widget w, XtPointer client, XtPointer call));
-static void v_draw P_((void));
-static void v_timer_cb P_((XtPointer client, XtIntervalId *id));
-static void drawComet P_((Display *dsp, Window win, GC gc, double ang, int rad,
-    int tlen, int w, int h));
-static void drawPlanet P_((Display *dsp, Window win, GC gc, int sx, int sy,
-    int w, int h));
-static void v_define_fgc P_((void));
+static void v_create_vshell (void);
+static void fill_msg (Widget w);
+static void v_popdown_cb (Widget w, XtPointer client, XtPointer call);
+static void v_ok_cb (Widget w, XtPointer client, XtPointer call);
+static void v_da_exp_cb (Widget w, XtPointer client, XtPointer call);
+static void v_draw (void);
+static void v_timer_cb (XtPointer client, XtIntervalId *id);
+static void drawComet (Display *dsp, Window win, GC gc, double ang, int rad,
+    int tlen, int w, int h);
+static void drawPlanet (Display *dsp, Window win, GC gc, int sx, int sy,
+    int w, int h);
+static void v_define_gc (void);
 
+/* text message */
 static char *msg[] = {
 "",
-"XEphem - An Interactive Astronomical Ephemeris Program for X Windows",
+"XEphem",
+"\"eks i 'fem\"",
+"",
+"An Interactive Astronomical Ephemeris Program for the X Window System",
 "+",
 "",
-"Copyright (c) 1990-2000 by Elwood Charles Downey",
+COPYRIGHT,
 "",
-"- Permission is granted to run XEphem for your personal non-profit and non-",
-"- commercial purposes only. You may modify the source code for your personal",
-"- use but you may not distribute your changes and you may not use any portion",
-"- of the source code (modified or not) in any other programs. Contact the",
-"- author if you would like to use any or all of XEphem for any other purpose.",
+"-Permission is granted to use and modify XEphem for personal or bona fide",
+"-educational purposes only. You may not redistribute any changes you make and",
+"-you may not use XEphem, in whole or in part, to create or augment commercial",
+"-applications. XEphem output products may be used without restriction as long",
+"-as they are credited. Contact the author to discuss all other situations.",
 "",
-"- NO REPRESENTATION IS MADE ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY",
-"- PURPOSE. IT IS PROVIDED \"AS IS\" WITHOUT EXPRESS OR IMPLIED WARRANTY.",
+"-NO REPRESENTATION IS MADE ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY",
+"-PURPOSE. IT IS PROVIDED \"AS IS\" WITHOUT EXPRESS OR IMPLIED WARRANTY.",
 "",
-"- Thank you for trying XEphem. I hope you like it. If you are using a free",
-"- copy it is completely functional in every way but did you know even more is",
-"- available on CDROM for a small fee? Here are some of the reasons you might",
-"- want to check it out:",
+"-If you are using the free version, purchasing the commercial edition adds:",
 "",
-"-   [] Includes 120+ page printed User Manual;",
-"-   [] Contains over 500 MB of additional data;",
-"-   [] Contains binaries built with real Motif for several platforms;",
-"-   [] Very easy system-wide or personal installation;",
-"-   [] Your support encourages further XEphem development.",
+"-  [ ] Prebuilt binaries and easy installation for many UNIX-like systems,",
+"-  [ ] Over 1 GB of data catalogs and lunar images,",
+"-  [ ] Over 400 high quality color deep sky images,",
+"-  [ ] Priority email support,",
+"-  [ ] Discounts on future releases,",
+"-  [ ] Your encouragement for continued XEphem development.",
 "",
-"- For full details, safe online ordering and to keep in touch with XEphem",
-"- developments set a bookmark at http://www.ClearSkyInstitute.com/xephem.",
+"-Stay in touch with XEphem at http://www.clearskyinstitute.com",
 "",
-"- Clear skies!",
-"- Elwood Downey",
-"- ecdowney@ClearSkyInstitute.com",
+"-Elwood Downey",
+"-ecdowney@ClearSkyInstitute.com",
 };
 
 
-#define	NMSGR	(sizeof(msg)/sizeof(msg[0]))
+#define	NMSGR	XtNumber(msg)	/* number of rows in message text */
 
 /* generate a random number between min and max, of the same type as the
  * highest type of either.
@@ -94,7 +82,8 @@ static char *msg[] = {
 static Widget vshell_w;		/* main shell */
 static Widget vda_w;		/* drawing area for mock solar system */
 static XtIntervalId v_timer_id;	/* timer for ss motion */
-static GC v_fgc;		/* drawing GC */
+static GC v_gc;			/* drawing GC */
+Pixel fg_p, bg_p;		/* two colors */
 static double rotrate;		/* rotation rate - filled on first manage */
 
 /* table of circular orbit radii to portray and the last screen coords.
@@ -102,9 +91,9 @@ static double rotrate;		/* rotation rate - filled on first manage */
  * so large we don't try and include everything.
  */
 typedef struct {
-    double r;		/* radius, AU */
-    double theta;	/* angle */
-    int x, y;		/* last X x,y coord drawn */
+    double r;			/* radius, AU */
+    double theta;		/* angle */
+    int x, y;			/* last X x,y coord drawn */
 } Orbit;
 
 #define	UNDEFX	(-1)		/* value of x when never drawn yet */
@@ -122,6 +111,7 @@ static Orbit orbit[] = {
 #define	DT	100		/* pause between screen steps, ms */
 #define	NSTARS	100		/* number of background stars to sprinkle in */
 #define	DPI	30		/* inner orbit motion per step, degrees*/
+#define	TXTCOLS	77		/* text columns wide */
 
 /* comet state and info */
 #define	CMAXPERI 30		/* max comet perihelion, pixels */
@@ -180,6 +170,7 @@ Cursor c;
 static void
 v_create_vshell()
 {
+	Widget pw_w;
 	Widget vform_w;
 	Widget ok_w;
 	Widget frame_w;
@@ -196,33 +187,41 @@ v_create_vshell()
 	XtSetArg (args[n], XmNdeleteResponse, XmUNMAP); n++;
 	vshell_w = XtCreatePopupShell ("About", topLevelShellWidgetClass,
 							toplevel_w, args, n);
+	setup_icon (vshell_w);
 	set_something (vshell_w, XmNcolormap, (XtArgVal)xe_cm);
         XtAddCallback (vshell_w, XmNpopdownCallback, v_popdown_cb, 0);
 	sr_reg (vshell_w, "XEphem*About.x", helpcategory, 0);
 	sr_reg (vshell_w, "XEphem*About.y", helpcategory, 0);
 
+	/* put text in top of paned window, DA and Ok in bottom */
+
+	n = 0;
+	pw_w = XmCreatePanedWindow (vshell_w, "VPW", args, n);
+	XtManageChild (pw_w);
+
+	/* make text widget for the version info */
+
+	XtSetArg (args[n], XmNscrolledWindowMarginWidth, 10); n++;
+	XtSetArg (args[n], XmNscrolledWindowMarginHeight, 10); n++;
+	XtSetArg (args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
+	XtSetArg (args[n], XmNcursorPositionVisible, False); n++;
+	XtSetArg (args[n], XmNeditable, False); n++;
+	XtSetArg (args[n], XmNcolumns, TXTCOLS); n++;
+	XtSetArg (args[n], XmNrows, NMSGR+1); n++;
+	text_w = XmCreateScrolledText (pw_w, "VText", args, n);
+	fill_msg (text_w);
+	XtManageChild (text_w);
+
+	/* form for the DA and Ok PB */
+
 	n = 0;
 	XtSetArg (args[n], XmNmarginHeight, 10); n++;
 	XtSetArg (args[n], XmNmarginWidth, 10); n++;
 	XtSetArg (args[n], XmNverticalSpacing, 10); n++;
 	XtSetArg (args[n], XmNhorizontalSpacing, 10); n++;
-	vform_w = XmCreateForm (vshell_w, "AF", args, n);
+	vform_w = XmCreateForm (pw_w, "AF", args, n);
 	XtManageChild (vform_w);
 
-	/* make text widget for the version info */
-
-	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNleftAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNrightAttachment, XmATTACH_FORM); n++;
-	XtSetArg (args[n], XmNeditMode, XmMULTI_LINE_EDIT); n++;
-	XtSetArg (args[n], XmNcursorPositionVisible, False); n++;
-	XtSetArg (args[n], XmNeditable, False); n++;
-	XtSetArg (args[n], XmNcolumns, 80); n++;
-	XtSetArg (args[n], XmNrows, NMSGR+1); n++;
-	text_w = XmCreateScrolledText (vform_w, "VText", args, n);
-	fill_msg (text_w);
-	XtManageChild (text_w);
-
 	/* make the "Ok" push button */
 
 	str = XmStringCreate("Ok", XmSTRING_DEFAULT_CHARSET);
@@ -239,8 +238,7 @@ v_create_vshell()
 	/* make a frame for the drawing area */
 
 	n = 0;
-	XtSetArg (args[n], XmNtopAttachment, XmATTACH_WIDGET); n++;
-	XtSetArg (args[n], XmNtopWidget, text_w); n++;
+	XtSetArg (args[n], XmNtopAttachment, XmATTACH_FORM); n++;
 	XtSetArg (args[n], XmNbottomAttachment, XmATTACH_WIDGET); n++;
 	XtSetArg (args[n], XmNbottomWidget, ok_w); n++;
 	XtSetArg (args[n], XmNleftAttachment, XmATTACH_POSITION); n++;
@@ -352,7 +350,7 @@ XtPointer call;
 	    }
 	default:
 	    printf ("Unexpected v_w event. type=%d\n", c->reason);
-	    exit(1);
+	    abort();
 	}
 
 	v_draw();
@@ -372,13 +370,17 @@ v_draw()
 	unsigned int bw, d;
 	int i;
 
-	if (!v_fgc)
-	    v_define_fgc();
+	if (!v_gc)
+	    v_define_gc();
 
 	XGetGeometry(dsp, win, &root, &x, &y, &w, &h, &bw, &d);
-	XClearWindow (dsp, win);
+	XSetForeground (dsp, v_gc, bg_p);
+	XSetFunction (dsp, v_gc, GXcopy);
+	XFillRectangle (dsp, win, v_gc, 0, 0, w, h);
 
 	/* draw the orbit ellipsii and forget last drawn locs */
+	XSetForeground (dsp, v_gc, fg_p^bg_p);
+	XSetFunction (dsp, v_gc, GXxor);
 	for (i = 0; i < NORBIT; i++) {
 	    int lx, ty;	/* left and top x */
 	    int nx, ny; /* width and height */
@@ -386,7 +388,7 @@ v_draw()
 	    nx = (int)(orbit[i].r/MAXRAD*w + 0.5);
 	    ty = (int)(h/2 - orbit[i].r/MAXRAD*h/2 + 0.5);
 	    ny = (int)(orbit[i].r/MAXRAD*h + 0.5);
-	    XDrawArc (dsp, win, v_fgc, lx, ty, nx-1, ny-1, 0, 360*64);
+	    XDrawArc (dsp, win, v_gc, lx, ty, nx-1, ny-1, 0, 360*64);
 	    orbit[i].x = UNDEFX;
 	}
 
@@ -394,14 +396,14 @@ v_draw()
 	radius = 0;
 
 	/* draw sun at the center */
-	drawPlanet (dsp, win, v_fgc, w/2-PR, h/2-PR, 2*PR-1, 2*PR-1);
+	drawPlanet (dsp, win, v_gc, w/2-PR, h/2-PR, 2*PR-1, 2*PR-1);
 
 	/* draw some background stars */
 	for (i = 0; i < NSTARS; i++) {
 	    int sx, sy;
 	    sx = RAND(0,w-1);
 	    sy = RAND(0,h-1);
-	    XDrawPoint (dsp, win, v_fgc, sx, sy);
+	    XDrawPoint (dsp, win, v_gc, sx, sy);
 	}
 }
 
@@ -434,9 +436,9 @@ XtIntervalId *id;
 	    if (px != orbit[i].x || py != orbit[i].y) {
 		/* erase then redraw at new pos, using the XOR GC */
 		if (orbit[i].x != UNDEFX)
-		    drawPlanet (dsp, win, v_fgc,
+		    drawPlanet (dsp, win, v_gc,
 				orbit[i].x-PR, orbit[i].y-PR, 2*PR-1, 2*PR-1);
-		drawPlanet (dsp, win, v_fgc, px-PR, py-PR, 2*PR-1, 2*PR-1);
+		drawPlanet (dsp, win, v_gc, px-PR, py-PR, 2*PR-1, 2*PR-1);
 		orbit[i].x = px;
 		orbit[i].y = py;
 	    }
@@ -446,7 +448,7 @@ XtIntervalId *id;
 	 * N.B. use radius == 0 to mean the very first loop.
 	 */
 	if (radius != 0)
-	    drawComet (dsp, win, v_fgc, angle, radius, taillen, w, h);
+	    drawComet (dsp, win, v_gc, angle, radius, taillen, w, h);
 
 	/* comet is definitely outside scene, set fresh initial conditions.
 	 */
@@ -472,7 +474,7 @@ XtIntervalId *id;
 	angle += (double)delta_area/(radius*radius);
 	radius = (int)(2*perihelion/(1.0 - cos(angle - rotation)));
 	taillen = (maxtail*perihelion*perihelion)/(radius*radius);
-	drawComet (dsp, win, v_fgc, angle, radius, taillen, w, h);
+	drawComet (dsp, win, v_gc, angle, radius, taillen, w, h);
 
 	/* restart timer */
 	v_timer_id = XtAppAddTimeOut (xe_app, DT, v_timer_cb, 0);
@@ -519,25 +521,20 @@ Window win;
 GC gc;
 int sx, sy, w, h;
 {
-	XPSFillArc (dsp, win, gc, sx, sy, w, h, 0, 360*64);
+	XFillArc (dsp, win, gc, sx, sy, w, h, 0, 360*64);
 }
 
 static void
-v_define_fgc()
+v_define_gc()
 {
 	Display *dsp = XtDisplay(vda_w);
 	Window win = XtWindow(vda_w);
-	XGCValues gcv;
-	unsigned int gcm;
-	Pixel fg, bg;
 
-	gcm = GCForeground | GCFunction;
-	get_something (vda_w, XmNforeground, (XtArgVal)&fg);
-	get_something (vda_w, XmNbackground, (XtArgVal)&bg);
-	gcv.foreground = fg ^ bg;
-	gcv.function = GXxor;
-	v_fgc = XCreateGC (dsp, win, gcm, &gcv);
+	get_something (vda_w, XmNforeground, (XtArgVal)&fg_p);
+	get_something (vda_w, XmNbackground, (XtArgVal)&bg_p);
+
+	v_gc = XCreateGC (dsp, win, 0L, NULL);
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: versionmenu.c,v $ $Date: 2000/10/07 03:48:18 $ $Revision: 1.16 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: versionmenu.c,v $ $Date: 2005/08/24 21:30:19 $ $Revision: 1.35 $ $Name:  $"};
