@@ -23,7 +23,6 @@
 #include "db.h"
 #include "dm.h"
 #include "sites.h"
-#include "plot.h"
 #include "indiapi.h"
 
 extern FILE *fopenh (char *name, char *how);
@@ -107,6 +106,7 @@ extern int srch_ison (void);
 extern int stopd_check (void);
 extern int strnncmp (char *s1, char *s2);
 extern int sv_ison (void);
+extern int sv_hznOpOk (Obj *op);
 extern int svf_filter_ok (Obj *op);
 extern int svf_ismanaged (void);
 extern int telIsOn (void);
@@ -318,6 +318,7 @@ extern void zero_mem (void *loc, unsigned len);
 #if defined (X_PROTOCOL)
 /* these require X Windows */
 
+#include "plot.h"
 #include "ps.h"
 #include "rotated.h"
 #include "skyeyep.h"
@@ -349,7 +350,8 @@ extern int gray_ramp (Display *dsp, Colormap cm, Pixel **pix);
 extern int isUp (Widget shell_w);
 extern int jpeg2pm (Display *dsp, Colormap cm, FILE *jpegfp, int *wp, int *hp, Pixmap *pmp, XColor xcols[256], char why[]);
 extern unsigned char *jpegRead(FILE *infile, int *width, int *height, unsigned char r[256], unsigned char g[256], unsigned char b[256], char why[]);
-extern int plot_cartesian (FILE *pfp, Widget widget, unsigned int nx, unsigned int ny, int flipx, int flipy, int grid, int xasdate);
+extern int plot_cartesian (DrawInfo *di, Widget widget, Dimension nx, Dimension ny);
+extern void plot_coords (Widget da_w, DrawInfo *di, int window_x, int window_y);
 extern void XCheck (XtAppContext app);
 extern void XPS_cursor (Cursor c);
 extern void ano_cb (Widget w, XtPointer client, XtPointer call);
@@ -438,6 +440,7 @@ extern void sun_cursor (Cursor c);
 extern void sun_manage (void);
 extern void sv_cursor (Cursor c);
 extern void sv_draw_obj (Display *dsp, Drawable win, GC gc, Obj *op, int x, int y, int diam, int dotsonly);
+extern void sv_draw_obj_x (Display *dsp, Drawable win, GC gc, Obj *op, int x, int y, int diam, int dotsonly, int flip_tb_x, int flip_lr_x, int aa_mode_x, int cyl_proj_x, double altdec_x, double azra_x, double vfov_x, double hfov_x, int w_x, int h_x);
 extern void sv_other (double altdec, double azra, int aa, double *altdecp, double *azrap);
 extern void svf_create (Widget shell_w);
 extern void svf_cursor (Cursor c);
@@ -454,5 +457,5 @@ extern void wtip (Widget w, char *tip);
 #endif /* _XEPHEM_H */
 
 /* For RCS Only -- Do Not Edit
- * @(#) $RCSfile: xephem.h,v $ $Date: 2008/10/28 04:11:28 $ $Revision: 1.60 $ $Name:  $
+ * @(#) $RCSfile: xephem.h,v $ $Date: 2012/12/30 17:01:02 $ $Revision: 1.63 $ $Name:  $
  */
