@@ -2,69 +2,68 @@
  */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <ctype.h>
 #include <math.h>
-#if defined(__STDC__)
-#include <stdlib.h>
-#else
-extern void *malloc();
-#endif
-#include <X11/Xlib.h>
+#include <dirent.h>
+#include <string.h>
+
 #include <Xm/Xm.h>
 #include <Xm/PushB.h>
+#include <Xm/RowColumn.h>
+#include <Xm/CascadeB.h>
+#include <Xm/Label.h>
 #include <Xm/Text.h>
 #include <Xm/TextF.h>
 
-#include "P_.h"
-#include "astro.h"
-#include "circum.h"
-
-extern Widget toplevel_w;
-extern char myclass[];
-
-extern int explodeGIF P_((unsigned char *rawpix, int nrawpix, int *wp,
-    int *hp, unsigned char *pixap[], unsigned char ra[], unsigned char ga[],
-    unsigned char ba[], char errmsg[]));
-extern void xe_msg P_((char *msg, int app_modal));
-extern void toHSV (double r, double g, double b, double *hp, double *sp,
-    double *vp);
-extern void toRGB (double h, double s, double v, double *rp, double *gp,
-    double *bp);
+#include "xephem.h"
 
 
-#define	MAXGRAY	30		/* max colors in a grayscale ramp */
+static void stopd_cb (Widget w, XtPointer data, XtPointer call);
+static Widget stopd_w;			/* dialog for user-stop */
+static int stopd_stopped;		/* flag set by the stopd PB */
 
-XFontStruct *getXResFont (char *rn);
-char *getXRes P_((char *name, char *def));
-int alloc_ramp P_((Display *dsp, XColor *basep, Colormap cm, Pixel pix[],
-    int maxn));
+#define	MAXGRAY	30			/* max colors in a grayscale ramp */
+
+/* createFSM aux info */
+typedef struct {
+    char **suffixes;			/* malloced list of suffixes */
+    int nsuffixes;			/* number of suffixes */
+    char *sharedir;			/* dir within ShareDir to scan */
+    XtCallbackProc cb;			/* callback for each PB */
+} FSMInfo;
 
-/* font and GCs we manage */
-static XFontStruct *viewsfsp;
-static XFontStruct *trackingfsp;
 /* names of resource colors for the planets.
- * N.B. should be in the same order as the defines in astro.h
+ * N.B. must be in the same order as the defines in astro.h
  */
-static char *objcolnames[NOBJ-3] = {
+static char *plcolnames[] = {
     "mercuryColor", "venusColor", "marsColor", "jupiterColor",
     "saturnColor", "uranusColor", "neptuneColor", "plutoColor",
     "sunColor", "moonColor"
 };
+
 /* name of resource for all other solar system objects */
 static char solsyscolname[] = "solSysColor";
-/* names of resource colors for stellar types.
- */
-static char *starcolnames[] = {
-    "hotStarColor", "mediumStarColor", "coolStarColor",
-    "otherStellarColor"
-};
-/* name of resource for all other objects */
-static char otherobjname[] = "otherObjColor";
 
-static GC objgcs[XtNumber(objcolnames)];
-static GC stargcs[XtNumber(starcolnames)];
+/* spectral colors are defined with X res names starSpectX or starSpectXX.
+ * resource is looked up first pick is requested
+ */
+static char sresbase[] = "starSpect";	/* base name for all spect res names */
+typedef struct {
+    char class[2];			/* 1 or 2 char spec class */
+    GC gc;				/* gc to use */
+} SpColor;
+static SpColor *spcolors;		/* malloced array as discovered */
+static int nspcolors;			/* entries in spcolors[] */
+
+/* font and GCs we manage */
+static XFontStruct *viewsfsp;
+static XFontStruct *trackingfsp;
+static GC pl_gc[XtNumber(plcolnames)];
 static GC solsys_gc;
+static GC esat_gc;
 static GC other_gc;
+static GC otherstar_gc;
 
 /* info used to make XmButton look like XmButton or XmLabel */
 static Arg look_like_button[] = {
@@ -97,8 +96,8 @@ XtArgVal value;
 	Arg a[1];
 
 	if (!w) {
-	    printf ("set_something (%s) called with w==0\n", resource);
-	    exit(1);
+	    printf ("set_something(w=%p, res=%s)\n", w, resource);
+	    abort();
 	}
 
 	XtSetArg (a[0], resource, value);
@@ -118,7 +117,7 @@ XtArgVal value;
 
 	if (!w) {
 	    printf ("get_something (%s) called with w==0\n", resource);
-	    exit(1);
+	    abort();
 	}
 
 	XtSetArg (a[0], resource, value);
@@ -141,7 +140,7 @@ char **txtp;
 
 	if (!w) {
 	    printf ("%s: called for %s with w==0\n", me, resource);
-	    exit(1);
+	    abort();
 	} else {
 	    XmString str;
 	    get_something(w, resource, (XtArgVal)&str); 
@@ -149,7 +148,7 @@ char **txtp;
 		/*
 		fprintf (stderr, "%s: can't get string resource %s\n", me,
 								resource);
-		exit (1);
+		abort();
 		*/
 		(void) strcpy (*txtp = XtMalloc(sizeof(hah)), hah);
 	    }
@@ -166,8 +165,8 @@ char *txt;
 	XmString str;
 
 	if (!w) {
-	    printf ("set_xmstring called for %s with w==0\n", resource);
-	    return;
+	    printf ("set_xmstring(w=%p, res=%s, txt=%s)\n", w, resource, txt);
+	    abort();
 	}
 
 	str = XmStringCreateLtoR (txt, XmSTRING_DEFAULT_CHARSET);
@@ -192,8 +191,8 @@ Widget w;
 						wa.map_state == IsViewable);
 }
 
-/* may be connected as the mapCallback to any convenience Dialog to
- * position it centered the cursor (allowing for the screen edges).
+/* may be connected as the mapCallback or XmNpopupCallback to
+ * center a window on the cursor (allowing for the screen edges).
  */
 /* ARGSUSED */
 void
@@ -259,11 +258,11 @@ Pixel *p;
 	if (!cval || !XAllocNamedColor (dsp, cm, cval, &defxc, &dbxc)) {
 	    char msg[128];
 	    if (!cval)
-		(void) sprintf (msg, "Can not find resource `%.80s'", cname);
+		sprintf (msg, "Can not find resource `%.80s'", cname);
 	    else
-		(void) sprintf (msg, "Can not XAlloc color `%.80s'", cval);
-	    (void) sprintf (msg+strlen(msg), " ... using White");
-	    xe_msg(msg, 0);
+		sprintf (msg, "Can not XAlloc color `%.80s'", cval);
+	    strcat (msg, " ... using White");
+	    xe_msg (0, msg);
 	    *p = WhitePixel (dsp, DefaultScreen(dsp));
 	    return (-1);
 	} else {
@@ -301,16 +300,18 @@ XFontStruct *fsp;
 
 	/* may not have created gcs yet */
 	if (!other_gc)
-	    return;
+	    make_objgcs();
 
 	/* update all Fonts in GCs */
 	fid = fsp->fid;
-	for (i = 0; i < XtNumber(objcolnames); i++)
-	    XSetFont (dsp, objgcs[i], fid);
-	for (i = 0; i < XtNumber(starcolnames); i++)
-	    XSetFont (dsp, stargcs[i], fid);
+	for (i = 0; i < XtNumber(plcolnames); i++)
+	    XSetFont (dsp, pl_gc[i], fid);
+	for (i = 0; i < nspcolors; i++)
+	    XSetFont (dsp, spcolors[i].gc, fid);
 	XSetFont (dsp, solsys_gc, fid);
 	XSetFont (dsp, other_gc, fid);
+	XSetFont (dsp, otherstar_gc, fid);
+	XSetFont (dsp, esat_gc, fid);
 }
 
 /* get the XFontStruct we want to use when drawing text while tracking cursor.
@@ -349,17 +350,18 @@ make_objgcs()
 	XFontStruct *fsp;
 	XGCValues gcv;
 	Pixel p;
-	int i;
+	int i, j;
 
+	/* always set font and foreground */
 	gcm = GCFont | GCForeground;
 	get_views_font (dsp, &fsp);
 	gcv.font = fsp->fid;
 
 	/* make the planet gcs */
-	for (i = 0; i < XtNumber(objgcs); i++) {
-	    (void) get_color_resource (toplevel_w, objcolnames[i], &p);
+	for (i = 0; i < XtNumber(pl_gc); i++) {
+	    (void) get_color_resource (toplevel_w, plcolnames[i], &p);
 	    gcv.foreground = p;
-	    objgcs[i] = XCreateGC (dsp, win, gcm, &gcv);
+	    pl_gc[i] = XCreateGC (dsp, win, gcm, &gcv);
 	}
 
 	/* make the gc for other solar system objects */
@@ -367,17 +369,45 @@ make_objgcs()
 	gcv.foreground = p;
 	solsys_gc = XCreateGC (dsp, win, gcm, &gcv);
 
-	/* make the star color gcs */
-	for (i = 0; i < XtNumber(stargcs); i++) {
-	    (void) get_color_resource (toplevel_w, starcolnames[i], &p);
-	    gcv.foreground = p;
-	    stargcs[i] = XCreateGC (dsp, win, gcm, &gcv);
-	}
-
-	/* make the gc for everything else */
-	(void) get_color_resource (toplevel_w, otherobjname, &p);
+	/* make the gc for fixed types other than stars */
+	(void) get_color_resource (toplevel_w, "otherObjColor", &p);
 	gcv.foreground = p;
 	other_gc = XCreateGC (dsp, win, gcm, &gcv);
+
+	/* make the gc for stars without a matching spectral color */
+	gcv.foreground = WhitePixel (dsp, DefaultScreen(dsp));
+	otherstar_gc = XCreateGC (dsp, win, gcm, &gcv);
+
+	/* build all spectral colors in resource database */
+	for (i = 0; i < nspcolors; i++)
+	    XFreeGC (dsp, spcolors[i].gc);
+	nspcolors = 0;
+	for (i = 'A'; i <= 'Z'; i++) {
+	    for (j = 0; j <= '9'; j = j ? j+1 : '0') {
+		char *v, res[sizeof(sresbase)+3];
+		SpColor *sp;
+
+		sprintf (res, "%s%c%c", sresbase, i, j);
+		v = getXRes (res, NULL);
+		if (v) {
+		    XColor defxc, dbxc;
+		    if (!XAllocNamedColor (dsp, xe_cm, v, &defxc, &dbxc))
+			defxc.pixel = WhitePixel (dsp, DefaultScreen(dsp));
+		    spcolors = (SpColor *) XtRealloc ((char *)spcolors,
+						(nspcolors+1)*sizeof(SpColor));
+		    sp = &spcolors[nspcolors++];
+		    gcv.foreground = defxc.pixel;
+		    sp->gc = XCreateGC (dsp, win, gcm, &gcv);
+		    sp->class[0] = i;
+		    sp->class[1] = j;
+		}
+	    }
+	}
+
+	/* make the gc for earth sats */
+	(void) get_color_resource (toplevel_w, "satellitesColor", &p);
+	gcv.foreground = p;
+	esat_gc = XCreateGC (dsp, win, gcm, &gcv);
 }
 
 /* given an object, return a GC for it.
@@ -393,36 +423,59 @@ GC *gcp;
 	if (!other_gc)
 	    make_objgcs();
 
-	if (op->o_type == PLANET && op->pl.pl_code < XtNumber(objgcs))
-	    *gcp = objgcs[op->pl.pl_code];
+	if (is_type (op, PLANETM))
+	    *gcp = pl_gc[op->pl_code];		/* moons share parent color */
 	else if (is_ssobj(op))
 	    *gcp = solsys_gc;
-	else if (is_type (op, FIXEDM))
-	    switch (op->f_class) {
-	    case 'Q': case 'J': case 'L': case 'T': case '\0':
-		*gcp = stargcs[3];
-		break;
-	    case 'S': case 'B': case 'D': case 'M': case 'V':
-		/* Sept 2000 S&T for article about latest spectral classes */
-		switch (op->f_spect[0]) {
-		case 'O': case 'B': case 'A': case 'W':
-		    *gcp = stargcs[0];
+	else if (is_type (op, EARTHSATM))
+	    *gcp = esat_gc;
+	else if (is_type (op, FIXEDM) || is_type (op, BINARYSTARM)) {
+	    int l, u, m = 0, d = -1;
+
+	    /* all but bonafide stars use "other" */
+	    if (is_type (op, FIXEDM)) {
+		switch (op->f_class) {
+		case 'T': case 'B': case 'D': case 'M': case 'S': case 'V':
 		    break;
-		case 'F': case 'G': case 'K':	/* FALLTHRU */
 		default:
-		    *gcp = stargcs[1];
-		    break;
-		case 'M': case 'L': case 'N': case 'R': case 'S': case 'C':
-		    *gcp = stargcs[2];
-		    break;
+		    *gcp = other_gc;
+		    return;
 		}
-		break;
-	    default:
-		*gcp = other_gc;
-		break;
 	    }
-	else
-	    *gcp = other_gc;
+
+	    /* binary search for perfect match */
+	    l = 0;
+	    u = nspcolors - 1;
+	    while (l <= u) {
+		m = (l+u)/2;
+		d = strncmp (spcolors[m].class, op->f_spect, 2);
+		if (d == 0)
+		    break;
+		if (d < 0)
+		    l = m+1;
+		else
+		    u = m-1;
+	    }
+
+	    /* select perfect or +/- 1 if same first char */
+	    if (d == 0 || op->f_spect[0] == spcolors[m].class[0])
+		*gcp = spcolors[m].gc;
+	    else if (m > 0 && op->f_spect[0] == spcolors[m-1].class[0])
+		*gcp = spcolors[m-1].gc;
+	    else if (m< nspcolors-1 && op->f_spect[0] == spcolors[m+1].class[0])
+		*gcp = spcolors[m+1].gc;
+	    else {
+		if (op->f_spect[0]) {
+		    xe_msg (0,
+		    "No color specified for %s class %c spectral class %.2s",
+					op->o_name, op->f_class, op->f_spect);
+		}
+		*gcp = otherstar_gc;
+	    }
+	} else {
+	    printf ("Unknown object type %d for coloring\n", op->o_type);
+	    abort();
+	}
 }
 
 /* given any widget built from an XmLabel return pointer to the first
@@ -441,11 +494,11 @@ XFontStruct **f;
 	get_something (w, XmNfontList, (XtArgVal)&fl);
 	if (XmFontListInitFontContext (&fc, fl) != True) {
 	    printf ("%s: No Font context!\n", me);
-	    exit(1);
+	    abort();
 	}
 	if (XmFontListGetNextFont (fc, &charset, f) != True) {
 	    printf ("%s: no font!\n", me);
-	    exit(1);
+	    abort();
 	}
 	XmFontListFreeFontContext (fc);
 }
@@ -459,11 +512,9 @@ char *rn;
 	char *fn = getXRes (rn, NULL);
 	Display *dsp = XtDisplay(toplevel_w);
 	XFontStruct *fsp;
-	char msg[1024];
 
 	if (!fn) {
-	    (void) sprintf (msg, "No resource `%s' .. using fixed", rn);
-	    xe_msg (msg, 0);
+	    xe_msg (0, "No resource `%s' .. using fixed", rn);
 	    fn = fixed;
 	}
 	
@@ -472,13 +523,11 @@ char *rn;
 	 */
 	fsp = XLoadQueryFont (dsp, fn);
 	if (!fsp) {
-	    (void) sprintf (msg, "No font `%s' for `%s' .. using fixed", fn,rn);
-	    xe_msg(msg, 0);
-
+	    xe_msg (0, "No font `%s' for `%s' .. using fixed", fn,rn);
 	    fsp = XLoadQueryFont (dsp, fixed);
 	    if (!fsp) {
 		printf ("Can't even get %s!\n", fixed);
-		exit(1);
+		abort();
 	    }
 	}
 
@@ -491,31 +540,24 @@ char *rn;
  * leave *greekgcp and greekfssp unchanged if there's any problems.
  */
 void
-loadGreek (dsp, win, greekgcp, greekfspp)
-Display *dsp;
-Drawable win;
-GC *greekgcp;
-XFontStruct **greekfspp;
+loadGreek (Display *dsp, Drawable win, GC *greekgcp, XFontStruct **greekfspp)
 {
 	static char grres[] = "viewsGreekFont";
 	XFontStruct *fsp;	/* local fast access */
 	GC ggc;			/* local fast access */
 	unsigned long gcm;
 	XGCValues gcv;
-	char buf[1024];
 	char *greekfn;
 	
 	greekfn = getXRes (grres, NULL);
 	if (!greekfn) {
-	    (void) sprintf (buf, "No resource: %s", grres);
-	    xe_msg (buf, 0);
+	    xe_msg (0, "No resource: %s", grres);
 	    return;
 	}
 
 	fsp = XLoadQueryFont (dsp, greekfn);
 	if (!fsp) {
-	    (void) sprintf (buf, "No font %.100s: %.800s", grres, greekfn);
-	    xe_msg (buf, 0);
+	    xe_msg (0, "No font %.100s: %.800s", grres, greekfn);
 	    return;
 	}
 
@@ -524,7 +566,7 @@ XFontStruct **greekfspp;
 	ggc = XCreateGC (dsp, win, gcm, &gcv);
 	if (!ggc) {
 	    XFreeFont (dsp, fsp);
-	    xe_msg ("Can not make Greek GC", 0);
+	    xe_msg (0, "Can not make Greek GC");
 	    return;
 	}
 
@@ -552,12 +594,8 @@ Pixel **pixp;
 
 	    white.red = white.green = white.blue = ~0;
 	    ngray = alloc_ramp (dsp, &white, cm, gramp, MAXGRAY);
-	    if (ngray < MAXGRAY) {
-		char msg[1024];
-		(void) sprintf (msg, "Wanted %d but only found %d grays.",
-							     MAXGRAY, ngray);
-		xe_msg (msg, 0);
-	    }
+	    if (ngray < MAXGRAY)
+		xe_msg (0, "Wanted %d but only found %d grays.",MAXGRAY,ngray);
 	}
 
 	*pixp = gramp;
@@ -571,12 +609,7 @@ Pixel **pixp;
  *   color 0/1 uses the background/foreground of a GC.
  */
 int
-alloc_ramp (dsp, basep, cm, pix, maxn)
-Display *dsp;
-XColor *basep;
-Colormap cm;
-Pixel pix[];
-int maxn;
+alloc_ramp (Display *dsp, XColor *basep, Colormap cm, Pixel pix[], int maxn)
 {
 	int nalloc, nunique;
 	double r, g, b, h, s, v;
@@ -643,9 +676,8 @@ int maxn;
 /* create an XImage of size wXh.
  * return XImage * if ok else NULL and xe_msg().
  */
-static XImage *
-create_xim (w, h)
-int w, h;
+XImage *
+create_xim (int w, int h)
 {
 	Display *dsp = XtDisplay(toplevel_w);
 	XImage *xip;
@@ -696,11 +728,7 @@ int w, h;
 /* like XFreeColors but frees the pixels in xcols[nxcols]
  */
 void
-freeXColors (dsp, cm, xcols, nxcols)
-Display *dsp;
-Colormap cm;
-XColor xcols[];
-int nxcols;
+freeXColors (Display *dsp, Colormap cm, XColor xcols[], int nxcols)
 {
 	unsigned long *xpix = (Pixel*)XtMalloc(nxcols * sizeof(unsigned long));
 	int i;
@@ -719,15 +747,15 @@ int nxcols;
  * return 0 if ok, else -1 with err[] containing a reason why not.
  */
 int
-gif2X (dsp, cm, gif, ngif, wp, hp, gifpix, xcols, err)
-Display *dsp;		/* X server */
-Colormap cm;		/* colormap for xcols[] */
-unsigned char gif[];	/* raw (still compressed) gif file contents */
-int ngif;		/* bytes in gif[] */
-int *wp, *hp;		/* size of exploded gif */
-unsigned char **gifpix;	/* ptr to array we malloc for gif pixels */
-XColor xcols[256];	/* X pixels and colors when indexed by gifpix */
-char err[];		/* error message if we return -1 */
+gif2X (
+Display *dsp,		/* X server */
+Colormap cm,		/* colormap for xcols[] */
+unsigned char gif[],	/* raw (still compressed) gif file contents */
+int ngif,		/* bytes in gif[] */
+int *wp, int *hp,	/* size of exploded gif */
+unsigned char **gifpix,	/* ptr to array we malloc for gif pixels */
+XColor xcols[256],	/* X pixels and colors when indexed by gifpix */
+char err[])		/* error message if we return -1 */
 {
 	unsigned char gifr[256], gifg[256], gifb[256];
 	int i;
@@ -759,14 +787,13 @@ char err[];		/* error message if we return -1 */
  * return 0 if ok, else fill why[] and return -1.
  */
 int
-gif2pm (dsp, cm, gif, ngif, wp, hp, pmp, why)
-Display *dsp;
-Colormap cm;
-unsigned char gif[];
-int ngif;
-int *wp, *hp;
-Pixmap *pmp;
-char why[];
+gif2pm (Display *dsp,
+Colormap cm,
+unsigned char gif[],
+int ngif,
+int *wp, int *hp,
+Pixmap *pmp,
+char why[])
 {
 	Window win = RootWindow(dsp, DefaultScreen(dsp));
 	unsigned char *gifpix;
@@ -825,6 +852,7 @@ char why[];
  * we just return it as a string -- caller can do whatever.
  * return def if can't find it anywhere.
  * N.B. memory returned is _not_ malloced so leave it be.
+ * N.B. see setXRes for how newlines are handled in the Xrm.
  */
 char *
 getXRes (name, def)
@@ -850,7 +878,12 @@ char *def;
 	return (res);
 }
 
-/* set the given application (ie, myclass.name) resource */
+/* set the given application (ie, myclass.name) resource.
+ * N.B. the combination of setXRes/getXRes has the effect that each backslash-n
+ *   in val comes back as a real nl and everything including and after the
+ *   the first real nl is discarded. if you think about it, this is the same
+ *   behavior as writing to and reading back one res to an app-defaults file.
+ */
 void
 setXRes (name, val)
 char *name, *val;
@@ -858,7 +891,7 @@ char *name, *val;
 	XrmDatabase db = XrmGetDatabase (XtDisplay(toplevel_w));
 	char buf[1024];
 
-	sprintf (buf, "%s.%s:%s", myclass, name, val);
+	sprintf (buf, "%s.%s:%s", myclass, name, val ? val : "");
 	XrmPutLineResource (&db, buf);
 }
 
@@ -983,6 +1016,7 @@ Widget sw_w;
 	/* you would think setting XmNvalue would be enough but seems we
 	 * must trigger the callback too, at least on MKS
 	 */
+	memset (&sbcs, 0, sizeof(sbcs));
 	sbcs.reason = XmCR_VALUE_CHANGED;
 	sbcs.event = NULL;	/* ? */
 
@@ -1032,6 +1066,7 @@ Widget w;
 /* check the given XmText or XmTextField value:
  * if empty, fill with "x/y", or just "x" if !y.
  * then set size to accommodate if setcols.
+ * while we're at it, fix the text cursor.
  */
 void
 defaultTextFN (w, setcols,  x, y)
@@ -1053,6 +1088,8 @@ char *x, *y;
 
 	if (setcols)
 	    textColumns (w);
+
+	fixTextCursor (w);
 }
 
 /* turn cursor on/off to follow focus */
@@ -1085,6 +1122,36 @@ Widget w;
 	XtAddCallback (w, XmNlosingFocusCallback, textFixCursorCB, 0);
 }
 
+/* convert str to all lowercase IN PLACE */
+char *
+strtolower (char *str)
+{
+	char *s = str;
+
+	/* actually faster to /not/ call isupper() first */
+	do
+	    *s = tolower (*s);
+	while (*s++);
+
+	return (str);
+}
+
+/* check whether op is within its valid date range at np.
+ * if not make a note and return -1 else return 0.
+ */
+int
+dateOK (Now *np, Obj *op)
+{
+	char dbuf[32];
+
+	if (dateRangeOK (np, op) == 0)
+	    return (0);
+
+	fs_date (dbuf, pref_get(PREF_DATE_FORMAT), mjd);
+	xe_msg (0, "Elements for %s are not valid on %s", op->o_name, dbuf); 
+	return (-1);
+}
+
 /* set up look_like_button[] and look_like_label[] */
 void
 setButtonInfo()
@@ -1131,8 +1198,298 @@ int whether;
 	    look_like_inited = 1;
 	}
 
-	XtSetValues (w, whether ? look_like_button : look_like_label, 6);
+	if (whether)
+	    XtSetValues (w, look_like_button, XtNumber(look_like_button));
+	else
+	    XtSetValues (w, look_like_label, XtNumber(look_like_label));
+}
+
+/* pop up a dialog to allow aborting a lengthy operation.
+ * then check occasionally using stopd_check()
+ */
+void
+stopd_up()
+{
+	/* create if first time */
+	if (!stopd_w) {
+	    Widget rc_w, w;
+	    Arg args[20];
+	    int n;
+
+	    /* create shell */
+
+	    n = 0;
+	    XtSetArg(args[n], XmNcolormap, xe_cm); n++;
+	    XtSetArg(args[n], XmNtitle, "xephem Stop");  n++;
+	    XtSetArg(args[n], XmNiconName, "Stop");  n++;
+	    XtSetArg(args[n], XmNdeleteResponse, XmDO_NOTHING);  n++;
+	    stopd_w = XtCreatePopupShell("NetStop", topLevelShellWidgetClass,
+							toplevel_w, args, n);
+	    setup_icon (stopd_w);
+	    set_something (stopd_w, XmNcolormap, (XtArgVal)xe_cm);
+	    XtAddCallback (stopd_w, XmNpopupCallback, prompt_map_cb, NULL);
+
+	    /* rc for controls */
+
+	    n = 0;
+	    XtSetArg(args[n], XmNisAligned, True);  n++;
+	    XtSetArg(args[n], XmNentryAlignment, XmALIGNMENT_CENTER);  n++;
+	    XtSetArg(args[n], XmNspacing, 10);  n++;
+	    XtSetArg(args[n], XmNmarginHeight, 10);  n++;
+	    XtSetArg(args[n], XmNmarginWidth, 10);  n++;
+	    rc_w = XmCreateRowColumn (stopd_w, "SRC", args, n);
+	    XtManageChild (rc_w);
+
+	    /* label and stop button */
+
+	    n = 0;
+	    w = XmCreateLabel (rc_w, "SL", args, n);
+	    set_xmstring (w, XmNlabelString, " Press Stop to cancel... ");
+	    XtManageChild (w);
+
+	    n = 0;
+	    w = XmCreatePushButton (rc_w, "Stop", args, n);
+	    XtAddCallback (w, XmNactivateCallback, stopd_cb, NULL);
+	    XtManageChild (w);
+	}
+
+	/* bring it up for sure */
+	XtPopdown (stopd_w);    /* so Popup causes position and raise */
+	XtPopup (stopd_w, XtGrabNone);
+	set_something (stopd_w, XmNiconic, (XtArgVal)False);
+	XFlush (XtDisplay(stopd_w));
+	XmUpdateDisplay (stopd_w);
+	XSync (XtDisplay(stopd_w), 0);
+
+	/* init button not pressed */
+	stopd_stopped = 0;
+}
+
+/* bring down the user-stop dialog */
+void
+stopd_down()
+{
+	if (stopd_w)
+	    XtPopdown (stopd_w);
+	stopd_stopped = 0;
+}
+
+/* after calling stopd_up(), poll this to check whether the user wants to stop.
+ * return -1 to stop, else 0.
+ */
+int
+stopd_check()
+{
+	/* check for user button presses */
+	XCheck (xe_app);
+
+	if (stopd_stopped) {
+	    stopd_stopped = 0;
+	    return (-1);
+	}
+	return (0);
+}
+
+/* called when the user presses the Stop button */
+/* ARGSUSED */
+static void
+stopd_cb (w, client, call)
+Widget w;
+XtPointer client;
+XtPointer call;
+{
+	xe_msg (0, "User stop");
+	stopd_stopped = 1;
+}
+
+/* compare two pointers to strings, qsort style */
+static int
+strc_qs (const void *v1, const void *v2)
+{
+	return (strcmp (*(char **)v1, *(char **)v2));
+}
+
+/* called when a pulldown built by createFSM, w, is coming up.
+ * client is an FSBInfo.
+ */
+static void
+FSMfillcb (Widget w, XtPointer client, XtPointer call)
+{
+	FSMInfo *fsm = (FSMInfo *) client;
+	WidgetList chil;
+	Cardinal nchil;
+	Widget pb_w;
+	char **files;
+	int nfiles;
+	char dirs[2][1024];
+	int i, j, k;
+
+	/* init dirs and file list, private first */
+	sprintf (dirs[0], "%s", getPrivateDir());
+	sprintf (dirs[1], "%s/%s", getShareDir(), fsm->sharedir);
+	files = NULL;
+	nfiles = 0;
+
+	/* scan each dir for each suffix, use only first one seen */
+	for (i = 0; i < 2; i++) {
+	    DIR *dirp = opendir (dirs[i]);
+	    struct dirent *dp;
+	    if (!dirp)
+		continue;
+	    while ((dp = readdir(dirp)) != NULL) {
+		for (j = 0; j < fsm->nsuffixes; j++) {
+		    int sl = strlen(fsm->suffixes[j]);
+		    int fl = strlen(dp->d_name);
+		    int dl = fl - sl;
+		    if (dl > 0 && !strcmp (dp->d_name+dl, fsm->suffixes[j])) {
+			for (k = 0; k < nfiles; k++)
+			    if (strcmp (dp->d_name, files[k]) == 0)
+				break;	/* already seen */
+			if (k == nfiles) {
+			    files = (char **) XtRealloc ((char *)files,
+						    (nfiles+1)*sizeof(char*));
+			    files[nfiles++] = XtNewString (dp->d_name);
+			}
+		    }
+		}
+	    }
+	    closedir (dirp);
+	}
+
+	/* sort */
+	qsort (files, nfiles, sizeof(char *), strc_qs);
+
+	/* put each file into pulldown, add if more now */
+	get_something (w, XmNnumChildren, (XtArgVal)&nchil);
+	for (i = 0; i < nfiles; i++) {
+	    if (i < nchil) {
+		/* get fresh pointer in case it moves when grown */
+		get_something (w, XmNchildren, (XtArgVal)&chil);
+		pb_w = chil[i];
+	    } else {
+		/* add new */
+		pb_w = XmCreatePushButton (w, "FSMPB", NULL, 0);
+		XtAddCallback (pb_w, XmNactivateCallback, fsm->cb, NULL);
+		nchil++;
+	    }
+	    set_xmstring (pb_w, XmNlabelString, files[i]);
+	    XtFree (files[i]);
+	    XtManageChild (pb_w);
+	}
+	XtFree ((char *)files);
+
+	/* multi-column if really long list */
+	if (nfiles >= 3)
+	    set_something (w,XmNnumColumns,(XtArgVal)((int)(sqrt(nfiles/3.0))));
+	else
+	    set_something (w, XmNnumColumns, (XtArgVal)1);
+
+	/* turn off any extra PBs */
+	get_something (w, XmNchildren, (XtArgVal)&chil);
+	for (; i < nchil; i++)
+	    XtUnmanageChild (chil[i]);
+}
+
+/* create a pulldown that fills itself with names of all files in Shared/sdir
+ * and Private with any of the given suffixes. If parent p is a MenuBar we
+ * build a pulldown triggered by a cascade button and return the cascade button,
+ * else we build and return an option menu. cb will be called when the user
+ * selects an item in the menu, it should get file name from the labelString
+ * resource of the w.
+ * N.B. we assume memory for sdir and each entry in suffix[] are persistent.
+ * N.B. we malloc one new FSMInfo for each pulldown.
+ * N.B. each suffix must include leading '.'
+ */
+Widget
+createFSM (Widget p, char **suffix, int nsuffix, char *sdir, XtCallbackProc cb)
+{
+	FSMInfo *fsm;
+	Widget pd_w, mgr_w;
+	unsigned char rctype;
+	Arg args[20];
+	int n;
+
+	/* gather the info */
+	fsm = (FSMInfo *) XtMalloc (sizeof(FSMInfo));
+	fsm->suffixes = (char **) XtMalloc (nsuffix * sizeof(char *));
+	fsm->nsuffixes = nsuffix;
+	memcpy (fsm->suffixes, suffix, nsuffix * sizeof(char *));
+	fsm->sharedir = sdir;
+	fsm->cb = cb;
+
+	/* create the pulldown, allow for multicolumn, connect fsm cb */
+	n = 0;
+	pd_w = XmCreatePulldownMenu (p, "FSM", args, n);
+	set_something (pd_w, XmNpacking, (XtArgVal)XmPACK_COLUMN);
+	XtAddCallback (pd_w, XmNmapCallback, FSMfillcb, (XtPointer)fsm);
+
+	/* create the pulldown's manager */
+	n = 0;
+	XtSetArg (args[n], XmNsubMenuId, pd_w); n++;
+	if (XmIsRowColumn(p) && (get_something (p, XmNrowColumnType,
+				    (XtArgVal)&rctype), rctype == XmMENU_BAR)) {
+	    mgr_w = XmCreateCascadeButton (p, "FSBCB", args, n);
+	    set_xmstring (mgr_w, XmNlabelString, "Files");
+	} else {
+	    /* put one entry for something to show in putton */
+	    Widget pb_w = XmCreatePushButton (pd_w, "XX", NULL, 0);
+	    XtAddCallback (pb_w, XmNactivateCallback, cb, NULL);
+	    XtManageChild (pb_w);
+	    set_xmstring (pb_w, XmNlabelString, "Make selection");
+	    mgr_w = XmCreateOptionMenu (p, "FSBOM", args, n);
+	}
+	XtManageChild (mgr_w);
+
+	return (mgr_w);
+}
+
+/* look up xcp->pixel in xe_cm and fill in xcp->red/green/blue.
+ * keep a cache of known values to reduce server round trips.
+ * if xcp == NULL reset the cache collection.
+ */
+void
+pixCache (XColor *xcp)
+{
+	static XColor *xc;
+	static int nxc;
+	int t, b;
+	
+	/* just reset if no xcp */
+	if (!xcp) {
+	    if (xc) {
+		free (xc);
+		xc = NULL;
+	    }
+	    nxc = 0;
+	    return;
+	}
+
+	/* binary search */
+	t = nxc - 1;
+	b = 0;
+	while (b <= t) {
+	    int m = (t+b)/2;
+	    XColor *mxcp = &xc[m];
+	    if (xcp->pixel == mxcp->pixel) {
+		*xcp = *mxcp;
+		return;
+	    }
+	    if (xcp->pixel < mxcp->pixel)
+		t = m-1;
+	    else
+		b = m+1;
+	}
+
+	/* not found, get the r/g/b */
+	XQueryColor (XtDisplay(toplevel_w), xe_cm, xcp);
+
+	/* add to cache in ascending order */
+	xc = xc ? realloc(xc, (nxc+1)*sizeof(XColor)) : malloc(sizeof(XColor));
+	for (t = nxc-1; t >= 0 && xc[t].pixel > xcp->pixel; --t)
+	    xc[t+1] = xc[t];
+	xc[t+1] = *xcp;
+	nxc++;
 }
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: xmisc.c,v $ $Date: 2000/11/27 23:27:13 $ $Revision: 1.24 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: xmisc.c,v $ $Date: 2006/04/10 09:00:06 $ $Revision: 1.57 $ $Name:  $"};
