@@ -1,11 +1,9 @@
 #include <stdio.h>
 #include <math.h>
 
-#include "P_.h"
 #include "astro.h"
 
-static void precess_hiprec P_((double mjd1, double mjd2, double *ra,
-    double *dec));
+static void precess_hiprec (double mjd1, double mjd2, double *ra, double *dec);
 
 
 #define	DCOS(x)		cos(degrad(x))
@@ -18,9 +16,9 @@ static void precess_hiprec P_((double mjd1, double mjd2, double *ra,
  * N.B. ra and dec are modifed IN PLACE.
  */
 void
-precess (mjd1, mjd2, ra, dec)
-double mjd1, mjd2;	/* initial and final epoch modified JDs */
-double *ra, *dec;	/* ra/dec for mjd1 in, for mjd2 out */
+precess (
+double mjd1, double mjd2,	/* initial and final epoch modified JDs */
+double *ra, double *dec)	/* ra/dec for mjd1 in, for mjd2 out */
 {
 	precess_hiprec (mjd1, mjd2, ra, dec);
 }
@@ -41,9 +39,9 @@ double *ra, *dec;	/* ra/dec for mjd1 in, for mjd2 out */
  * 96-06-20 Hayo Hase <hase@wettzell.ifag.de>: theta_a corrected
  */
 static void
-precess_hiprec (mjd1, mjd2, ra, dec)
-double mjd1, mjd2;	/* initial and final epoch modified JDs */
-double *ra, *dec;	/* ra/dec for mjd1 in, for mjd2 out */
+precess_hiprec (
+double mjd1, double mjd2,	/* initial and final epoch modified JDs */
+double *ra, double *dec)	/* ra/dec for mjd1 in, for mjd2 out */
 {
 	static double last_mjd1 = -213.432, last_from;
 	static double last_mjd2 = -213.432, last_to;
@@ -129,9 +127,9 @@ double *ra, *dec;	/* ra/dec for mjd1 in, for mjd2 out */
 
 #if 0
 static void
-precess_fast (mjd1, mjd2, ra, dec)
-double mjd1, mjd2;	/* initial and final epoch modified JDs */
-double *ra, *dec;	/* ra/dec for mjd1 in, for mjd2 out */
+precess_fast (
+double mjd1, double mjd2,	/* initial and final epoch modified JDs */
+double *ra, double *dec)	/* ra/dec for mjd1 in, for mjd2 out */
 {
 #define	N	degrad (20.0468/3600.0)
 #define	M	hrrad (3.07234/3600.0)
@@ -145,4 +143,4 @@ double *ra, *dec;	/* ra/dec for mjd1 in, for mjd2 out */
 #endif
 
 /* For RCS Only -- Do Not Edit */
-static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: precess.c,v $ $Date: 1998/07/17 23:35:22 $ $Revision: 1.2 $ $Name:  $"};
+static char *rcsid[2] = {(char *)rcsid, "@(#) $RCSfile: precess.c,v $ $Date: 2003/03/20 08:51:37 $ $Revision: 1.4 $ $Name:  $"};
